//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// AgentSmooth.cpp: implementation of the AgentSmooth class.
//
//////////////////////////////////////////////////////////////////////

#include "AgentSmooth.h"
#include "Individual.h"
#include "Scaffold.h"
#include "Vertex.h"
#include <cmath>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AgentSmooth::AgentSmooth()
{

}

AgentSmooth::AgentSmooth(int ID, double weight, double smoothness)
: Agent(ID, weight)
{
	_smoothness = smoothness;
}

AgentSmooth::~AgentSmooth()
{

}

double AgentSmooth::evaluateScaffold(Individual* individual)
{
	if(_weight==0)
		return 0;
	double zPos, totDiff = 0, diff = 0, score = 0;
	for(int j=0; j<individual->numberOfScaffolds(); j++)	{
		Scaffold* scaffold = individual->getScaffold(j);
		vector<Vertex*> neighbours;
		totDiff = 0;
		int nNeighbours, nVer = scaffold->numberOfVertices();
		//This is a local criteria
		for(int i=0; i<nVer; i++)	{
			diff = 0;
			scaffold->getVertex(i)->adjacentVertices(neighbours);
			nNeighbours = neighbours.size();
			zPos = scaffold->getVertex(i)->getPosition().getZ();
			//Calcualte the difference to the neighbours
			for(int j=0; j<nNeighbours; j++)
				diff += fabs( zPos - neighbours[j]->getPosition().getZ() );
			//See how this is compared to the desired difference
			totDiff += fabs(diff - _smoothness);
		}
		//Normalize by dividing with the number of vertices
		score += _weight*totDiff/(double)nVer;
	}
	score = score==HUGE_VAL ? 100000 : score;
	//Store the value with the individual
	individual->setSmoothnessFitness(score);
	return score;
}