//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// AgentSoftBoundary.cpp: implementation of the AgentSoftBoundary class.
//
//////////////////////////////////////////////////////////////////////

#include "AgentSoftBoundary.h"
#include "Individual.h"
#include "Scaffold.h"
#include <iostream>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AgentSoftBoundary::AgentSoftBoundary()
{

}

AgentSoftBoundary::AgentSoftBoundary(int ID, double weight)
:	Agent(ID, weight)
{

}

AgentSoftBoundary::~AgentSoftBoundary()
{

}

double AgentSoftBoundary::evaluateScaffold(Individual* individual)
{
	if(_weight==0)
		return 0;
	double fitness = 0;
	for(int j=0; j<individual->numberOfScaffolds(); j++)	{
		Scaffold* scaffold = individual->getScaffold(j);
		int nOut = 0, nVer = scaffold->numberOfVertices();
		//See how many vertices are outside
		for(int i=0; i<nVer; i++)
			nOut += scaffold->getVertex(i)->isOutside() ? 1 : 0;

		//Normalize and give fitness penalty
		fitness += _weight*(double)nOut/(double)nVer;
	}
	//Store the value with the individual
	individual->setSoftBoundaryFitness(fitness);
	return fitness;
}