//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// AgentSymmetry.cpp: implementation of the AgentSymmetry class.
//
//////////////////////////////////////////////////////////////////////

#include "AgentSymmetry.h"
#include "Individual.h"
#include "Scaffold.h"
#include "Vertex.h"
#include "Vector.h"
#include <cmath>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AgentSymmetry::AgentSymmetry()
{

}

AgentSymmetry::AgentSymmetry(int ID, double weight, double ratio)
: Agent(ID, weight)
{
	_ratio = ratio;
}

AgentSymmetry::~AgentSymmetry()
{

}

double AgentSymmetry::evaluateScaffold(Individual* individual)
{
	if(_weight==0)
		return 0;
	double fitness = 0;
	for(int j=0; j<individual->numberOfScaffolds(); j++)	{
		Scaffold* scaffold = individual->getScaffold(j);
		int xcount = 0, ycount = 0, nVer = scaffold->numberOfVertices();
		Vector center;
		scaffold->getCenter(center);
		//Count the number of vertices that are on one side of the symmetry axis.
		for(int i=0; i<nVer; i++)	{
			xcount += scaffold->getVertex(i)->getPosition().getX() > center.getX() ? 1 : 0;
			ycount += scaffold->getVertex(i)->getPosition().getY() > center.getY() ? 1 : 0;
		}
		//The skewness is the fitness
		fitness += _weight*( fabs((double)xcount/(double)nVer - _ratio) + fabs((double)ycount/(double)nVer - _ratio) );
	}
	//Store the value with the individual
	individual->setSymmetryFitness(fitness);
	return fitness;
}