//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// AgentUndulating.cpp: implementation of the AgentUndulating class.
//
//////////////////////////////////////////////////////////////////////

#include "AgentUndulating.h"
#include "Individual.h"
#include "Scaffold.h"
#include <iostream>
#include <cmath>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AgentUndulating::AgentUndulating()
{

}

AgentUndulating::AgentUndulating(int ID, double weight, double undulation)
: Agent(ID, weight)
{
	_undulation = undulation;
}

AgentUndulating::~AgentUndulating()
{

}

double AgentUndulating::evaluateScaffold(Individual* individual)
{
	if(_weight==0)
		return 0;
	double fitness = 0;
	for(int j=0; j<individual->numberOfScaffolds(); j++)	{
		Scaffold* scaffold = individual->getScaffold(j);
		int nVer = scaffold->numberOfVertices();
		double maxZ = -100000, minZ = 100000, tempZ, diff;
		//Find the global max and min for the z-direction
		for(int i=0; i<nVer; i++)	{
			tempZ = scaffold->getVertex(i)->getPosition().getZ();
			maxZ = maxZ > tempZ ? maxZ : tempZ;
			minZ = minZ < tempZ ? minZ : tempZ;
		}
		//Normalize and give fitness penalty
		diff = fabs(maxZ - minZ);
		diff = diff < 1000000 ? diff : 0;	//We should never get these values unless an underflow has occured
		fitness += _weight*fabs(diff - _undulation);
	}
	//Store the value with the individual
	individual->setUndulatingFitness(fitness);
	return fitness;
}