//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Attractor.cpp: implementation of the Attractor class.
//
//////////////////////////////////////////////////////////////////////

#include <iostream>
#include <sstream>
#include <maya/MGlobal.h>
#include "Attractor.h"
#include "MELUtilities.h"
#include <cmath>




//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Attractor::Attractor()
{

}

Attractor::Attractor(Vector pos, int ID, bool attr, int exp, double konst, bool drawn, bool singularity)
{
	_attracting = attr;
	_exponent = exp;
	_const = konst;
	_pos = pos;
	_ID = ID;
	_drawn = drawn;
	_singularity = singularity;
	//cout << "Constructed attractor " << ID << " with position " << _pos.printVector().c_str() << ", singularity " << boolalpha << singularity << ", const " << konst << " and exponent " << exp << endl;
}

Attractor::~Attractor()
{

}

Vector Attractor::getForce(Vector pos)
{
	//Calculate the direction
	Vector dir, diff = _pos - pos;
	dir = _attracting ? diff : diff*(-1);
	double dist = diff.getLength();
	dir.normalize();
	//cout << "The distance between the attractor and the point is " << dist << endl;
	dir = dir*magnitude(dist, _const, _exponent, _singularity);
	//cout << "The force from attractor " << _ID << " is " << dir.printVector().c_str() << endl;
	return dir;
}

void Attractor::drawAttractor(string attrLayer, string repLayer)
{
	ostringstream parts, group, move, layer;
	double x, y, z;
	_pos.getVector(x,y,z);

	//If we use the commandline, we may add attractors there and they are not drawn
	if(!_drawn)	{
		if(_attracting)	{
			//Set layer for drawing
			//layer << "editDisplayLayerGlobals -cdl " << attrLayer;
			eval(layer.str());
			//parts << "polyCube -ch off -n genr8Attr0" << _ID << " -w 3 -h 1 -d 1; polyCube -ch off -n genr8Attr1" << _ID << " -w 1 -h 3 -d 1; polyCube -ch off -n genr8Attr2" << _ID << " -w 1 -h 1 -d 3;";
			parts << "polyCylinder -ch off -n " << ATTR_NAME << _ID << " -r 0.2 -h 1 -axis 0 0 1";
			eval(parts.str());
			if(x>=0)
				move << "setAttr " << ATTR_NAME << _ID << ".translateX " << x << "; ";
			else
				move << "setAttr " << ATTR_NAME << _ID << ".translateX -" << -x << "; ";
			if(y>=0)
				move << "setAttr " << ATTR_NAME << _ID << ".translateY " << y << "; ";
			else
				move << "setAttr " << ATTR_NAME << _ID << ".translateY -" << -y << "; ";
			if(z>=0)
				move << "setAttr " << ATTR_NAME << _ID << ".translateZ " << z << "; ";
			else
				move << "setAttr " << ATTR_NAME << _ID << ".translateZ -" << -z << "; ";
			eval(move.str());
		}
		else	{
			//layer << "editDisplayLayerGlobals -cdl " << repLayer;
			eval(layer.str());
			//parts << "polyCube -ch off -n genr8Repellor" << _ID << " -w 3 -h 1 -d 1";
			parts << "polyCylinder -ch off -n " << REP_NAME << _ID << " -r 0.2 -h 1";
			eval(parts.str());
			if(x>=0)
				move << "setAttr " << REP_NAME << _ID << ".translateX " << x << "; ";
			else
				move << "setAttr " << REP_NAME << _ID << ".translateX -" << -x << "; ";
			if(y>=0)
				move << "setAttr " << REP_NAME << _ID << ".translateY " << y << "; ";
			else
				move << "setAttr " << REP_NAME << _ID << ".translateY -" << -y << "; ";
			if(z>=0)
				move << "setAttr " << REP_NAME << _ID << ".translateZ " << z << "; ";
			else
				move << "setAttr " << REP_NAME << _ID << ".translateZ -" << -z << "; ";
			eval(move.str());
		}
	}
}

void Attractor::getRepellorPosition(int ID, bool attr)
{
	ostringstream posX, posY, posZ, change;
	//Get the position
	//The user is allowed to move the repellor, so we must find out where it is and update the variables while we are at it
	if(attr)	{
		posX << "getAttr " << ATTR_NAME << ID << ".translateX";
		_pos.setX(getDoubleResult(posX.str()));
		change << "optionVar -fv " << ATTR_POS_X << ID << " " << _pos.getX() << ";";
		posY << "getAttr " << ATTR_NAME << ID << ".translateY";
		_pos.setY(getDoubleResult(posY.str()));
		change << "optionVar -fv " << ATTR_POS_Y << ID << " " << _pos.getY() << ";";
		posZ << "getAttr " << ATTR_NAME << ID << ".translateZ";
		_pos.setZ(getDoubleResult(posZ.str()));
		change << "optionVar -fv " << ATTR_POS_Z << ID << " " << _pos.getZ() << ";";
	}
	else	{
		posX << "getAttr " << REP_NAME << ID << ".translateX";
		_pos.setX(getDoubleResult(posX.str()));
		change << "optionVar -fv " << REP_POS_X << ID << " " << _pos.getX() << ";";
		posY << "getAttr " << REP_NAME << ID << ".translateY";
		_pos.setY(getDoubleResult(posY.str()));
		change << "optionVar -fv " << REP_POS_Y << ID << " " << _pos.getY() << ";";
		posZ << "getAttr " << REP_NAME << ID << ".translateZ";
		_pos.setZ(getDoubleResult(posZ.str()));
		change << "optionVar -fv " << REP_POS_Z << ID << " " << _pos.getZ() << ";";
	}
	eval(change.str());
}