//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Axiom.cpp: implementation of the Axiom class.
//
//////////////////////////////////////////////////////////////////////

#include "Axiom.h"
#include <iostream>
#include <sstream>
#include "BNF.h"
#include "Genotype.h"
#include "Edge.h"
#include "Sentence.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Axiom::Axiom()
{
	
}

Axiom::Axiom(BNF* grammar, BNFtype bnfType)
: NonTerminal(grammar, bnfType)
{
	_name = AXIOM;
}

Sentence* Axiom::getProductionRule(int gene, bool terminal)
{
	int prod;
	ostringstream str;
	if(_BNFtype==SYMMETRIC)	{
		prod = gene % 4;
		switch(prod)	{
		//The axiom is always a regular polygon
		case 0 :	str << EDGE_X << " + " << EDGE_Y << " + " << EDGE_X << " + " << EDGE_Y;
					break;
		case 1 :	str << EDGE_X << " + " << EDGE_Y << " + " << EDGE_X << " + " << EDGE_Y << " + " << EDGE_X << " + " << EDGE_Y;
					break;
		case 2 :	str << EDGE_X << " + " << EDGE_Y << " + " << EDGE_Z << " + " << EDGE_X << " + " << EDGE_Y << " + " << EDGE_Z;
					break;
		default :	str << EDGE_X << " + " << EDGE_Y << " + " << EDGE_X << " + " << EDGE_Y << " + " << EDGE_X << " + " << EDGE_Y << " + " << EDGE_X << " + " << EDGE_Y;
					break;
		}
	}
	else
		str << EDGE << " " << START_EDGE << " " START_EDGE << " { " START_EDGE << " }";
	return new Sentence(str.str());
}

Axiom::~Axiom()
{

}

void Axiom::reconstruct(Sentence* sent, Genotype* &genome, int multi)
{
	//cout << "Reconstructing Axiom: " << endl;
	sent->printSentence();
	int nEdges = sent->numberOfWords("+") + 1;
	if(nEdges < 3)	{
		cout << "Illegal Axiom." << endl;
		return;
	}
	genome->addRandGene();
	//Fix the first three edges
	Edge* edg = new Edge;
	edg->reconstruct(sent->getWord(0)->getEdgeType(), genome);
	edg = new Edge;
	edg->reconstruct(sent->getWord(2)->getEdgeType(), genome);
	edg = new Edge;
	edg->reconstruct(sent->getWord(4)->getEdgeType(), genome);
	
	//Fix the rest
	for(int i=0; i<nEdges-3; i++)	{
		//Add a gene that gives us a edge
		genome->addGene(i+1);
		//Make sure that we get the right type
		edg = new Edge;
		edg->reconstruct(sent->getWord(6 + 2*i)->getEdgeType(), genome);
	}
	//Make sure that we do not get any more edges
	genome->addGene(0);
}

