//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// BNF.cpp: implementation of the BNF class.
//
//////////////////////////////////////////////////////////////////////

#include "BNF.h"
#include "Genotype.h"
#include <iostream>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

BNF::BNF()
{
	//_nEdgeTypes = 2;
	//cout << "Creating LSystem" << endl;
	_LSys = new LSystem();
}

BNF::BNF(Genotype* genome, BNFtype bnfType, int maxEdgeTypes)
{
	_nEdgeTypes = 2;
	_genome = genome;
	_genomeLength = _genome->getLength();
	_index = 0;
	_maxEdgeTypes = maxEdgeTypes;
	_LSys = new LSystem(this, bnfType);
}

BNF::~BNF()
{

}

//This function determines the edge type for a newly created edge
//The probability for introducing new types decreases with number
//of current edges.

int BNF::getEdge(bool newEdge)
{
	int type, nTypes = getEdgeTypes(), gene = getNextGene();
	bool max = _maxEdgeTypes!=-1 && _maxEdgeTypes <= nTypes;
	if(!newEdge || max)
		type = gene % nTypes;
	else
		type = gene % (nTypes+1);
	if(type == nTypes)
		incEdgeTypes();
	return type;
}

void BNF::printLSystem()
{
	_LSys->printNonTerminal();
}

Sentence* BNF::getFullSentence()
{
	Sentence* sent = new Sentence();
	_LSys->getFullSentence(sent);
	return sent;
}

int BNF::getNextGene()
{
	int gene = _genome->getGene(_index++ % _genomeLength);
	//cout << "Returning gene " << gene << " at position " << _index - 1 << endl;
	return gene;
}

void BNF::getNextGene(int &gene, int &index)
{
	gene = _genome->getGene(_index % _genomeLength);
	index = _index;
	_index++;
}

void BNF::addGene(int gene)
{
	_genome->addGene(gene);
}

Genotype* BNF::constructGenome(Sentence* sent)
{
	_genome = new Genotype(0);
	sent->printSentence();
	_LSys->reconstruct(sent, _genome);
	//Angle
	int anglepos = sent->findWord("Angle");
	if(anglepos!=-1)
		reconstructAngle(sent->getWord(anglepos+1), _genome);
	//Sync
	reconstructSync(_genome, sent->findWord("Sync")!=-1);
	//BranchAngle
	int branchanglepos = sent->findWord("BranchAngle");
	if(branchanglepos!=-1)
		reconstructBranchAngle(sent->getWord(branchanglepos+1), _genome);
	return _genome;
}


void BNF::reconstructAngle(Word* word, Genotype* &genome)
{
	double angle = atof( word->getName().c_str() );
	angle++;	//Correct for rounding errors
	//cout << "Reconstructing Angle " << angle << endl;
	genome->addGene(angle/15 - 2);
}

void BNF::reconstructBranchAngle(Word* word, Genotype* &genome)
{
	double branchangle = atof( word->getName().c_str() );
	branchangle++;	//Correct for rounding errors
	genome->addGene(branchangle/15 - 2);
}

void BNF::reconstructSync(Genotype* &genome, bool sync)
{
	if(sync)
		genome->addGene(1);
	else
		genome->addGene(0);
}

