//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// BNF.h: interface for the BNF class.
//
//	The BNF is an interface class between the Individual and the classes that are
//	used to construct the grammar. BNF keeps track of global variables for the expansion
//	of the grammar.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BNF_H__E9F51B0D_17B7_11D5_95D7_00C04F797C24__INCLUDED_)
#define AFX_BNF_H__E9F51B0D_17B7_11D5_95D7_00C04F797C24__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "LSystem.h"

class Genotype;

class BNF  
{
public:
	BNF();
	BNF(Genotype* genome, BNFtype bnfType, int maxEdgeTypes);
	virtual ~BNF();

	int			getEdgeTypes()	{return _nEdgeTypes;};
	void		incEdgeTypes()	{_nEdgeTypes++;};
	LSystem*	getLSystem()	{return _LSys;};
	int			getEdge(bool newEdge);
	void		printLSystem();
	Sentence*	getFullSentence();
	int			getNextGene();
	void		getNextGene(int &gene, int &index);
	void		addGene(int gene);
	//Reverse engineering
	Genotype*	constructGenome(Sentence* sent);
protected:
	//Reverse engineering
	void		reconstructSync(Genotype* &genome, bool sync);
	void		reconstructAngle(Word* word, Genotype* &genome);
	void		reconstructBranchAngle(Word* word, Genotype* &genome);
	
	//Data
	//The tree structure
	LSystem*	_LSys;
	//The number of edgetypes in the grammar, and the number allowed. -1 means that there is no limit
	int			_nEdgeTypes, _maxEdgeTypes;
	//A pointer to the genome is useful when we reconstruct and read the genome
	Genotype*	_genome;
	int			_genomeLength;
	int			_index; //Keep track of where in the genome we are reading
	
};

#endif // !defined(AFX_BNF_H__E9F51B0D_17B7_11D5_95D7_00C04F797C24__INCLUDED_)
