//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Boundary.h: interface for the Boundary class.
//
//	This is a parent class for the boundaries and it holds such info that is relevant 
//	for all surfaces (polys and nurbs). It keeps track of all the walls and it is the
//	boundary class that determines in what way a point is affected by a wall.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BOUNDARY_H__8FCA6D1F_1C6D_11D5_95D7_00C04F797C24__INCLUDED_)
#define AFX_BOUNDARY_H__8FCA6D1F_1C6D_11D5_95D7_00C04F797C24__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define MIN_DIST 1000

#include "Field.h"
#include "PolygonSurface.h"
#include "NurbsSurface.h"
#include <vector>

using namespace std;

//Enumerate the possible wall behaviours.
typedef enum {DEFAULT_WB, CUT_OFF, ACTIVE_WALLS} wallBehaviour;

class MDagPath;

class Boundary : public Field
{
public:
	Boundary();
	Boundary(wallBehaviour wallType, double wc = 0, int we = 0);
	virtual ~Boundary();

	//Get distance or vector to the closest wall
	double	distanceToClosestWall(Vector pos);
	Vector	vectorToClosest(Vector pos);
	
	Vector	getWallDisplacement(Vector pos, Vector step);
	bool	passedWall(Vector pos, Vector step);	//Use for soft boundaries
	
	void	addPolySurface(MDagPath surf);
	void	addNurbsSurface(MDagPath surf);
private:
	//Methods
	double	setThreshold(double tol);
	bool	wallTooClose(double dist, double closest, double step);
	Vector	getWallForce(Vector pos, Vector step, Vector intersect);
	double	getWallForce(double pos, double step, double intersect);	//For 1-D steps
	//Data
	//Store the boundaries in vectors
	vector<PolygonSurface*>	_polySurfs;
	int		_nPolySurfs;
	vector<NurbsSurface*>	_nurbsSurfs;
	int		_nNurbsSurfs;
	//Data about wall behaviour
	wallBehaviour _wallBehaviour;
	double  _threshold;	//If you are futher from a wall than this, do not care about walls
	double	_const;
	int		_exponent;
};

#endif // !defined(AFX_BOUNDARY_H__8FCA6D1F_1C6D_11D5_95D7_00C04F797C24__INCLUDED_)
