//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// File: Cmd.cpp
//
// MEL Command: genr8
//
// Author: Maya SDK Wizard featuring Martin Hemberg
//\nFor help, type \"genr8 -h\"

//////////////////////////////////////////////////////////////
//
//	General notes about the code
//
//	I try to avoid using Maya or Windows specific functions as much as possible.
//	This to gain speed and make porting easier.
//	We generally work with pointers rather than the actual objects.
//	The reason for this is because it is faster and easier to work with pointers.
//	Also, for the scaffold model, all the parts need to have information about each other,
//	that is easily accomplished by giving them pointers to each other.
//
//	At many places I use -1 to represent invalid data or infinity (the context usually makes clear which).
//	This instead of using a unsigned and a bool.
//
//	Unfortunately I have not been able to make the debug version working, there are instructions
//	in the Maya help-files. However, I do not htink that it would be a tremendous help,
//	printing messages is quite easy and it is good for the character.
//
//	A bigger problem is that there is something wrong with workspace info. Thus it is not
//	possible to see where each function is being called. This is of great help when trying to 
//	comprehend the code, so it would probably be worth your while to fix this if you intend to
//	decipher my code.
//
//////////////////////////////////////////////////////////////

// Includes everything needed to register a simple MEL command with Maya.
// 
//This is the only MFC-stuff needed. It is used in modifyDialog to open the file browser
#include <afxdlgs.h>
#include <afx.h>
#include <iostream>
#include <sstream>
#include <maya/MGlobal.h>
#include <maya/MSimple.h>
#include <maya/MSyntax.h>
#include <maya/MArgParser.h>
#include <maya/MArgDatabase.h>
#include <maya/MString.h>
#include <maya/MStringArray.h>
#include <maya/MItSelectionList.h>
#include <maya/MSelectionList.h>
#include <maya/MDagPath.h>
#include <maya/MDagPathArray.h>
#include <maya/MFnNurbsCurve.h>
#include <maya/MPointArray.h>
#include <maya/MPxContext.h>
#include <maya/MPxContextCommand.h>
#include <maya/MPxToolCommand.h>
#include <maya/MToolsInfo.h>
#include <maya/M3dView.h>
#include <maya/MCursor.h>
#include <maya/MPoint.h>
#include <maya/MFnPlugin.h>
#include "World.h"
#include "MELUtilities.h"
#include "Vector.h"
#include "Sentence.h"
#include "Word.h"
#include "NonTerminal.h"
#include <vector>
#include <cmath>
#include <ctime>
#include <string>

//We are grateful that people actually use this software
#define THANK_YOU_GENR8					"\nThank you for using GENR8 v0.98!\nEmergent Design Group, MIT 2001.\nImplemented by Martin Hemberg.\nIf you would like a GUI, use -gui.\nFor help, type \"genr8 -h\"\n"
#define THANK_YOU_DEGENR8				"\nThank You for using degenr8 v0.98.\n"
#define THANK_YOU_REGENR8				"\nThank You for using regenr8 v0.98.\n"
#define THANK_YOU_REGN8					"\nThank You for using regn8 v0.98.\n"
#define THANK_YOU_REPELLOR				"\nThank You for using repellor v0.98.\n"
#define THANK_YOU_ATTRACTOR				"\nThank You for using attractor v0.98.\n"
#define THANK_YOU_SAVE_GRAMMAR			"\nThank You for using saveGrammar v0.98.\n"
#define THANK_YOU_SAVE_GENOME			"\nThank You for using saveGenome v0.98.\n"
#define THANK_YOU_MODIFY_DIALOG			"\nThank You for using modifyDialog v0.98.\n"
//Default values, flag names and MEL-variable names, alphabetically (almost) ordered
#define ACTIVE_WALLS_VAR				"activeWalls"
#define ATTRACTOR_CONSTANT				10
#define ATTRACTOR_CONSTANT_VAR			"attractorConstant"
#define ATTRACTOR_EXPONENT				2
#define ATTRACTOR_EXPONENT_VAR			"attractorExponent"
#define ATTRACTOR_SINGULAR				false
#define ATTRACTOR_SINGULAR_VAR			"attractorSingular"

#define BNF_TYPE						"default"
#define BNF_TYPE_VAR					"BNFType"
#define BRANCH_ANGLE_VAR				"branchAngle"
#define BRANCH_LENGTH					-1
#define BRANCH_LENGTH_VAR				"branchLength"
#define BSPLINES_VAR					"BSplines"

#define CHANGE_SCALE_VAR				"changeScale"
#define CONTINUE						false
#define CONTINUE_VAR					"continueVar"	//For some reason, Maya does not allow us to call it "continue"
#define COPY_INDIVIDUAL					0
#define COPY_INDIVIDUAL_VAR				"copyIndividual"
#define COPY_STYLE						"exact"
#define COPY_STYLE_VAR					"copyStyle"
#define CROSSOVER_POINTS				1
#define CROSSOVER_POINTS_VAR			"crossoverPoints"
#define CUT_OFF_VAR						"cutOff"

#define DEFAULT_BNF_VAR					"defaultBNF"
#define DEFAULT_WB_VAR					"defaultWallBehaviour"
#define DELETE_VAR						"deleteMELValues"
#define DIFFERENT_GRAMMARS				false
#define DIFFERENT_GRAMMARS_VAR			"differentGrammars"
#define DISPLAY_STATS_VAR				"displayStats"
#define DRAW_EACH_STEP					false
#define DRAW_EACH_STEP_VAR				"drawEachStep"

#define ELITES							1
#define ELITES_VAR						"elites"
#define ENCODED_GRAMMAR_VAR				"encodedGrammar"
#define EXACT_COPIES					true
#define EXACT_COPIES_VAR				"exactCopies"

#define FITNESS_LAYOUT					"$fitnessLayout"
#define	FIXED_CENTER					false
#define	FIXED_CENTER_VAR				"fixedCenter"
#define FIXED_PERIMETER					false
#define FIXED_PERIMETER_VAR				"fixedPerimeter"
#define FRACTAL							"noFractal"
#define FRACTAL_VAR						"fractal"
#define FRACTAL_RADIO_GROUP				"$fractalRadioGroup"
#define FULL_RANDOM_REWRITE				false
#define FULL_RANDOM_REWRITE_VAR			"fullRandomRewrite"
#define FULL_RANDOM_REWRITE_BOX			"$fullRandomRewriteBox"

#define GENERATIONS						0
#define GENERATIONS_VAR					"generations"
#define GENETIC_ENGINE_LAYOUT			"$geneticEngineLayout"
#define GENOME_LENGTH					50
#define GENOME_LENGTH_VAR				"genomeLength"
#define GRAVITY_MAGNITUDE				0
#define GRAVITY_MAGNITUDE_VAR			"gravityMagnitude"
//#define GRAVITY_DIRECTION				Z
#define GRAVITY_DIRECTION_VAR			"gravityDirection"
#define GRAVITY_VAR						"gravity"
#define GROW_STEPS_VAR					"growSteps"
#define GROW_WINDOW						"$genr8GrowIndividualWindow"
#define GUI_VAR							"graphicUserInterfacce"

#define HELP_VAR						"help"

#define INPUT_WINDOW					"$genr8InputWindow"

#define KOCH_CURVE						"kochCurve"

#define LOAD_GENOME						"genome.txt"
#define LOAD_GENOME_VAR					"loadGenome"
#define LOAD_GRAMMAR					""
#define LOAD_GRAMMAR_VAR				"loadGrammar"
#define LOAD_SETTINGS_VAR				"loadSettings"

#define MAX_BNF_DEPTH					5
#define MAX_BNF_DEPTH_VAR				"maxBNFDepth"
#define MAX_BNF_DEPTH_FIELD				"$maxBNFDepthField"
#define MAX_EDGE_TYPES					-1
#define MAX_EDGE_TYPES_VAR				"maxEdgeTypes"
#define MAX_EDGE_TYPES_FIELD			"$maxEdgeTypesField"
#define MUTATION_RATE					2
#define MUTATION_RATE_VAR				"mutationRate"

#define NUMBER_OF_COPIES				0
#define NUMBER_OF_COPIES_VAR			"numberOfCopies"
#define NUMBER_OF_SCAFFOLDS				1
#define NUMBER_OF_SCAFFOLDS_VAR			"numberOfScaffolds"

#define OUTPUT_WINDOW					"$genr8OutputWindow"

#define POPULATION_SIZE					1
#define POPULATION_SIZE_VAR				"populationSize"
#define	POLYGONS_VAR					"polygons"
#define PREDEFINED_VAR					"predefined"
#define PROBABILISTIC_BNF_VAR			"probabilistic"

#define QUADRATIC_KOCH					"quadraticKoch"

#define RANDOM_SEED						-1
#define RANDOM_SEED_VAR					"randomSeed"
#define REPELLING_POINTS_CONSTANT		0
#define REPELLING_POINTS_CONSTANT_VAR	"repellingPointsConstant"
#define REPELLING_POINTS_EXPONENT		0
#define REPELLING_POINTS_EXPONENT_VAR	"repellingPointsExponent"
#define REPELLING_POINTS_VAR			"repellingPoints"
#define REPELLOR_CONSTANT				10
#define REPELLOR_CONSTANT_VAR			"repellorConstant"
#define REPELLOR_EXPONENT				2
#define REPELLOR_EXPONENT_VAR			"repellorExponent"
#define REPELLOR_SINGULAR				false
#define REPELLOR_SINGULAR_VAR			"repellorSingular"
#define RANDOM_NOISE					0
#define RANDOM_NOISE_VAR				"randomNoise"
#define RANDOM_REWRITE					0
#define RANDOM_REWRITE_VAR				"randomRewrite"
#define REMOVE_VAR						"remove"
#define REVERSIBLE_BNF_VAR					"reversible"

#define SAVE_GENOME						"genome.txt"
#define SAVE_GENOME_VAR					"saveGenome"
#define SAVE_GRAMMAR					""
#define SAVE_GRAMMAR_VAR				"saveGrammar"
#define SAVE_GRAMMAR_FIELD_VAR			"$saveGrammarField"
#define SAVE_SETTINGS_VAR				"saveSettings"
#define SAVE_VISIBLE_VAR				"saveVisible"
#define SCALE							1.5
#define SCALE_VAR						"scale"
#define SCALE_X							1.5
#define SCALE_X_VAR						"scaleX"
#define SCALE_X_FIELD					"$scaleXField"
#define SCALE_Y							1.5
#define SCALE_Y_VAR						"scaleY"
#define SCALE_Y_FIELD					"$scaleYField"
#define SCALE_Z							1.5
#define SCALE_Z_VAR						"scaleZ"
#define SCALE_Z_FIELD					"$scaleZField"
#define SEED_POPULATION_VAR				"seedPopulation"
#define SET_FITNESS_RANK				-1
#define SET_FITNESS_RANK_VAR			"setFitnessRank"
#define SET_FITNESS_VALUE				0
#define SET_FITNESS_VALUE_VAR			"setFitnessValue"
#define SET_FITNESS_VAR					"setFitness"
#define SIZE_X							10
#define SIZE_X_VAR						"sizeX"
#define SIZE_Y							10
#define SIZE_Y_VAR						"sizeY"
#define SIZE_VAR						"size"
#define SIZE_WEIGHT						1
#define SIZE_WEIGHT_VAR					"sizeWeight"
#define SMOOTHNESS						0
#define SMOOTHNESS_VAR					"smoothness"
#define SMOOTHNESS_WEIGHT				1
#define SMOOTHNESS_WEIGHT_VAR			"smoothnessWeight"
#define SOFT_BOUNDARY_VAR				"softBoundary"
#define SOFT_BOUNDARY_WEIGHT			0
#define SOFT_BOUNDARY_WEIGHT_VAR		"softBoundaryWeight"
#define START_LENGTH					1
#define START_LENGTH_VAR				"startLength"
#define START_POSITION_X				0
#define START_POSITION_X_VAR			"startPositionX"
#define START_POSITION_Y				0
#define START_POSITION_Y_VAR			"startPositionY"
#define START_POSITION_Z				0
#define START_POSITION_Z_VAR			"startPositionZ"
#define START_POSITION_VAR				"startPosition"
#define STEPS							4
#define STEPS_VAR						"steps"
#define SURFACE_VAR						"surface"
#define SUBDIVISIONS					0
#define SUBDIVISIONS_VAR				"subdivisions"
#define SUBDIVISIONS_WEIGHT				1
#define SUBDIVISIONS_WEIGHT_VAR			"subdivisionsWeight"
#define SYMMETRIC_BNF_VAR				"symmetricBNF"
#define SYMMETRY						0.5
#define SYMMETRY_VAR					"symmetry"
#define SYMMETRY_WEIGHT					1
#define SYMMETRY_WEIGHT_VAR				"symmetryWeight"

#define TILES							0
#define TILES_VAR						"tiles"
#define TILES_RADIO_GROUP				"$tilesRadioGroup"
#define TOURNAMENT_SIZE					4
#define TOURNAMENT_SIZE_VAR				"tournamentSize"

#define UNDULATION						0
#define UNDULATION_VAR					"undulation"
#define UNDULATION_WEIGHT				1
#define UNDULATION_WEIGHT_VAR			"undulationWeight"
#define UNIFORM_CELLS					false
#define UNIFORM_CELLS_VAR				"uniformCells"

#define VIEW_INDIVIDUAL					-1
#define VIEW_INDIVIDUAL_VAR				"viewIndividual"

#define WALL_BEHAVIOUR_VAR				"wallBehaviour"
#define WALL_EXPONENT					2
#define WALL_EXPONENT_VAR				"wallExponent"
#define WALL_EXPONENT_FIELD				"$wallExponentField"
#define WALL_CONSTANT					50
#define WALL_CONSTANT_VAR				"wallConstant"
#define WALL_CONSTANT_FIELD				"$wallConstantField"

using namespace std;

//Pointless but fun function
string getStupidStatement(int state = -1);

string getStupidStatement(int state)
{
	int i = state==-1 ? rand() % 22 : state;
	switch(i)	{
	case 0:	return "Have a nice day!\n";
	case 1:	return "Don't worry - Be Happy!\n";
	case 2:	return "Big Brother is watching you\n";
	case 3:	return "Kill time before time kills you.\n";
	case 4:	return "Save the whales!\n";
	case 5:	return "Always look on the bright side of life.\n";
	case 6:	return "When the going gets weird, the weird turns pro.\n";
	case 7:	return "All animals are equal, but some animals are more equal than other.\n";
	case 8:	return "Just because I am crazy doesn't mean that I am wrong.\n";
	case 9: return "Emergent Design Group - We put the fun in fundamentalist.\n";
	case 10:	return "Better luck next time.\n";
	case 11:	return "Sorry, wrong number.\n";
	case 12:	return "If everything else fails, try reading the manual.\n";
	case 13:	return "In this tool we obey the laws of thermodynamics.\n";
	case 14:	return "I hope you enjoyed using GENR8.\n";
	case 15:	return "It takes two to lie. One to lie and one to listen.\n";
	case 16:	return "You tried your best and you failed miserably. The lesson is \'never try\'.\n";
	case 17:	return "Lord help me, I'm just not that bright.\n";
	case 18:	return "Beer. Now there's a temporary solution.\n";
	case 19:	return "You are not expected to understand this.\n";
	case 20:	return "The purpose of GENR8 is to inflate weak ideas, obscure poor reasoning, and inhibit clarity.\n";
	default:	return "Martin Hemberg is a nice guy.\n";
	}
}

//Here is the actual genr8 command
//Some of this I do not understand, I just copied from the plug-ins that exist in Maya
class genr8 : public MPxCommand
{
public:
				genr8();
	virtual		~genr8();

	MStatus		doIt( const MArgList& );
	bool		isUndoable() const	{return false;};

	static		void* creator();

private:
	//Methods
	MSyntax setSyntax();
	World*	parseArgs( const MArgList& args );
	void	help();
	void	parseCurve(MDagPath curve);
	void	initFitnessVector();
	void	fixAttractorsAndRepellors();
	void	createInputWindow();
	void	createOutputWindow();
	void	growIndividualWindow(int ind, int steps);
	void	getMELValues(bool remove);
	//Data, these are all the parameters for genr8
	World*		theWorld;
	double		_scale, _scaleX, _scaleY, _scaleZ, _startLength, _repellingPointsConstant, _wallConstant, _gravityMagnitude, _smoothness, _smoothnessWeight, _subdivisions, _subdivisionsWeight, _sizeX, _sizeY, _sizeWeight, _symmetry, _symmetryWeight, _softBoundaryWeight, _randomNoise, _attractorConstant, _repellorConstant, _branchLength, _undulation, _undulationWeight;
	int			_steps, _branchAngle, _wallExponent, _repellingPointsExponent, _tiles, _populationSize, _tournamentSize, _elites, _mutationRate, _maxBNFDepth, _generations, _genomeLength, _randomRewrite, _copyIndivdual, _numberOfCopies, _attractorExponent, _repellorExponent, _numberOfScaffolds, _crossoverPoints;
	bool		_fixedPerimeter, _fixedCenter, _drawEachStep, _fullRandomRewrite, _continue, _exactCopies, _showStats, _differentGrammars, _uniformCells, _attractorSingular, _repellorSingular;
	int			_randomSeed, _viewIndividual, _maxEdgeTypes, _growIndividual;
	Direction	_gravityDirection;
	surfaceRepresentation _surfaceRepresentation;
	wallBehaviour _wallBehaviour;
	string		_loadGenomeFile, _saveGenomeFile, _loadGrammarFile, _saveGrammarFile, _cmd, _encodedGrammar;
	Fractals	_fractal;
	Vector		_startPosition;
	BNFtype		_bnfType;
	vector<double>	_startLengths, _masses;
	vector<Vector>	_startPositions, _seed;
	//These should maybe be excluded later
	vector<Vector>	attrPos, repPos;
	double			limitedBranches;
	string			saveFile, loadScaffold, saveScaffold;
	vector<double>	fitness;
	bool			userSeed;
};
//Constructor
genr8::genr8()
{
	//Set the default values
	_attractorConstant = ATTRACTOR_CONSTANT;
	_attractorExponent = ATTRACTOR_EXPONENT;
	_attractorSingular = ATTRACTOR_SINGULAR;

	_bnfType = REGULAR;
	_branchAngle = BRANCH_ANGLE;
	_branchLength = BRANCH_LENGTH;

	_cmd = "genr8 ";
	_continue = CONTINUE;
	_copyIndivdual = COPY_INDIVIDUAL;
	_crossoverPoints = CROSSOVER_POINTS;
	
	_differentGrammars = DIFFERENT_GRAMMARS;
	_drawEachStep = DRAW_EACH_STEP;
	
	_elites = ELITES;
	_exactCopies = EXACT_COPIES;

	_fixedPerimeter = FIXED_PERIMETER;
	_fixedCenter = FIXED_CENTER;
	_fractal = NO_FRACTAL;
	_fullRandomRewrite = FULL_RANDOM_REWRITE;
	
	_generations = GENERATIONS;
	_genomeLength = GENOME_LENGTH;
	_gravityMagnitude = GRAVITY_MAGNITUDE;
	_gravityDirection = Z;
	_growIndividual = -1;
	
	_loadGenomeFile = LOAD_GENOME;
	_loadGrammarFile = LOAD_GRAMMAR;
	
	_maxBNFDepth = MAX_BNF_DEPTH;
	_maxEdgeTypes = MAX_EDGE_TYPES;
	_mutationRate = MUTATION_RATE;
	
	_numberOfCopies = NUMBER_OF_COPIES;
	_numberOfScaffolds = NUMBER_OF_SCAFFOLDS;
	
	_populationSize = POPULATION_SIZE;
	
	_randomNoise = RANDOM_NOISE;
	_randomRewrite = RANDOM_REWRITE;
	_randomSeed = RANDOM_SEED;
	_repellingPointsExponent = REPELLING_POINTS_EXPONENT;
	_repellingPointsConstant = REPELLING_POINTS_CONSTANT;
	_repellorConstant = REPELLOR_CONSTANT;
	_repellorExponent = REPELLOR_EXPONENT;
	_repellorSingular = REPELLOR_SINGULAR;
	
	_saveGenomeFile = SAVE_GENOME;
	_saveGrammarFile = SAVE_GRAMMAR;
	_scale = SCALE;
	_scaleX = SCALE_X;
	_scaleY = SCALE_Y;
	_scaleZ = SCALE_Z;
	_showStats = false;
	_sizeX = SIZE_X;
	_sizeY = SIZE_Y;
	_sizeWeight = SIZE_WEIGHT;
	_smoothness = SMOOTHNESS;
	_smoothnessWeight = SMOOTHNESS_WEIGHT;
	_softBoundaryWeight = SOFT_BOUNDARY_WEIGHT;
	_startPosition.setX(START_POSITION_X);
	_startPosition.setY(START_POSITION_Y);
	_startPosition.setZ(START_POSITION_Z);
	_startLength = START_LENGTH;
	_steps = STEPS;
	_subdivisions = SUBDIVISIONS;
	_subdivisionsWeight = SUBDIVISIONS_WEIGHT;
	_surfaceRepresentation = SURFACE;
	_symmetry = SYMMETRY;
	_symmetryWeight = SYMMETRY_WEIGHT;

	_tiles = TILES;
	_tournamentSize = TOURNAMENT_SIZE;
	
	_undulation = UNDULATION;
	_undulationWeight = UNDULATION_WEIGHT;
	_uniformCells = UNIFORM_CELLS;
	
	_viewIndividual = VIEW_INDIVIDUAL;
	
	_wallBehaviour = DEFAULT_WB;
	_wallExponent = WALL_EXPONENT;
	_wallConstant = WALL_CONSTANT;
	
	//Set the fitness vector
	initFitnessVector();
	
	userSeed = false;
	limitedBranches = -1;
}
//destructor
genr8::~genr8()
{
	delete theWorld;
}
//I do not know why I need this, I think it is called when genr8 is executed through MEL
void* genr8::creator()
{
	return new genr8();
}

//Here it all begins
MStatus genr8::doIt( const MArgList& args )
//
//	Description:
//		implements the MEL  command.
//
//	Arguments:
//		args - the argument list that was passes to the command from MEL
//
//	Return Value:
//		MS::kSuccess - command succeeded
//		MS::kFailure - command failed (returning this value will cause the 
//                     MEL script that is being run to terminate unless the
//                     error is caught using a "catch" statement.
//
{
	//This is what we always do first...
	printf("\n\nHello world!\n\n");
	//cout << "Parsing args" << endl;
	//Create the world, it contains everything that we need to know to grow a surface.
	//It relies on C++ and I am boos there and I rely on Maya as little as possible.
	World* theWorld = parseArgs(args);
	if(theWorld==NULL)
		return MS::kSuccess;

	//Get the objects from the scene
	MStatus stat = MS::kSuccess, *status = new MStatus(MS::kSuccess);
	MSelectionList list;
	MDagPath dag;
		
	//First, remove the attractors and repellors
	MGlobal::unselectByName("genr8rep*");
	MGlobal::unselectByName("genr8attr*");
	//Get the selection list
	MGlobal::getActiveSelectionList(list);
	//Get the polygon surfaces
	for ( MItSelectionList poly( list, MFn::kMesh ); !poly.isDone(); poly.next() )	{
		poly.getDagPath(dag);
		//Add the surfaces to the Boundary-object
		theWorld->getBoundary()->addPolySurface(dag);
	}
	//Get the nurbs-surfaces
	for ( MItSelectionList nurbs( list, MFn::kNurbsSurface ); !nurbs.isDone(); nurbs.next() )	{
		nurbs.getDagPath(dag);
		//Add the surfaces to the Boundary-object
		theWorld->getBoundary()->addNurbsSurface(dag);
	}
	//Get the curves, they can be used as user-defined seeds
	for ( MItSelectionList curves( list, MFn::kNurbsCurve ); !curves.isDone(); curves.next() )	{
		curves.getDagPath(dag);
		userSeed = true;
		//If there are several curves on the selection list, we get the last one.
	}
	if(userSeed)
		parseCurve(dag);
	//Add the attractors and repellors to the scene
	fixAttractorsAndRepellors();
	// Since this class is derived off of MPxCommand, you can use the 
	// inherited methods to return values and set error messages
	//

	//Do some of the initialization that could not be done in the constructor.
	//if(loadScaffold!="")
	//	theWorld->initialize(loadScaffold, steps, drawEach, cont, loadGenome, fitness, viewInd);
	/*else*/ 
	if(_seed.size() > 2)
		_generations = theWorld->initialize(_masses, _seed, _steps, _drawEachStep, _continue, _loadGenomeFile, fitness, _viewIndividual, _copyIndivdual, _numberOfCopies, _exactCopies) ? _generations : 0;
	else
		_generations = theWorld->initialize(_startPositions, _startLengths, _steps, _drawEachStep, _continue, _loadGenomeFile, fitness, _viewIndividual, _copyIndivdual, _numberOfCopies, _exactCopies) ? _generations : 0;
	//Main loop
	for(int i=0; i<_generations; i++)	{
		//It only fails if it is interrupted by the user
		if(!theWorld->step(_steps, _drawEachStep, _viewIndividual))
			break;
	}
	//Save stuff
	if(saveFile!="")
		theWorld->saveScaffolds(saveFile);
	if(_saveGrammarFile!="")
		theWorld->saveGrammar(_saveGrammarFile);
	if(_saveGenomeFile!="")
		theWorld->saveGenomes(_saveGenomeFile);
	
	//It is so annoying to have the curves on the selectionlist, because you tend to forget it and stuff goes to some place very warm...
	MGlobal::unselectByName("genr8*");

	//Show stats
	if(_showStats)
		createOutputWindow();
	else if(_growIndividual!=-1)
		growIndividualWindow(_growIndividual, _steps);
	//Generate stupid message
	string mess = THANK_YOU_GENR8 + getStupidStatement();
	setResult( mess.c_str() );
	printf( THANK_YOU_GENR8 );
	printf( getStupidStatement().c_str() );
	return stat;
}

//Define all the flags
MSyntax genr8::setSyntax()
{
	MSyntax syntax;

	syntax.enableQuery(false);
	syntax.enableEdit(false);
	syntax.addFlag("aw", ACTIVE_WALLS_VAR);
	syntax.addFlag("ac", ATTRACTOR_CONSTANT_VAR, MSyntax::kDouble);
	syntax.addFlag("ae", ATTRACTOR_EXPONENT_VAR, MSyntax::kUnsigned);
	syntax.addFlag("as", ATTRACTOR_SINGULAR_VAR);

	syntax.addFlag("ba", BRANCH_ANGLE_VAR, MSyntax::kUnsigned);
	syntax.addFlag("ba", BRANCH_LENGTH_VAR, MSyntax::kDouble);
	syntax.addFlag("bs", BSPLINES_VAR);
	
	syntax.addFlag("c", CONTINUE_VAR);
	syntax.addFlag("ci", COPY_INDIVIDUAL_VAR, MSyntax::kUnsigned, MSyntax::kUnsigned);
	syntax.addFlag("co", CUT_OFF_VAR);
	syntax.addFlag("cp", CROSSOVER_POINTS_VAR, MSyntax::kUnsigned);
	
	syntax.addFlag("dg", DIFFERENT_GRAMMARS_VAR);
	syntax.addFlag("ds", DRAW_EACH_STEP_VAR);
	
	syntax.addFlag("e", ELITES_VAR, MSyntax::kUnsigned);
	syntax.addFlag("e", ENCODED_GRAMMAR_VAR, MSyntax::kUnsigned, MSyntax::kString);
	
	syntax.addFlag("fp", FIXED_PERIMETER_VAR);
	syntax.addFlag("fc", FIXED_CENTER_VAR);
	syntax.addFlag("frr", FULL_RANDOM_REWRITE_VAR);
	
	syntax.addFlag("ge", GENERATIONS_VAR, MSyntax::kUnsigned);
	syntax.addFlag("g", GRAVITY_VAR, MSyntax::kString, MSyntax::kDouble);
	syntax.addFlag("gl", GENOME_LENGTH_VAR, MSyntax::kUnsigned);
	syntax.addFlag("gl", GROW_STEPS_VAR, MSyntax::kUnsigned);
	syntax.addFlag("gui", GUI_VAR);
	
	syntax.addFlag("kc", KOCH_CURVE);
	
	syntax.addFlag("lge", LOAD_GENOME_VAR, MSyntax::kString);
	syntax.addFlag("lg", LOAD_GRAMMAR_VAR, MSyntax::kString);
	
	syntax.addFlag("h", HELP_VAR);
	
	syntax.addFlag("md", MAX_BNF_DEPTH_VAR, MSyntax::kUnsigned);
	syntax.addFlag("me", MAX_EDGE_TYPES_VAR, MSyntax::kUnsigned);
	syntax.addFlag("mr", MUTATION_RATE_VAR, MSyntax::kUnsigned);
	
	syntax.addFlag("ns", NUMBER_OF_SCAFFOLDS_VAR, MSyntax::kUnsigned);

	syntax.addFlag("p", POLYGONS_VAR);
	syntax.addFlag("ps", POPULATION_SIZE_VAR, MSyntax::kUnsigned);
	syntax.addFlag("pr", PROBABILISTIC_BNF_VAR);
	
	syntax.addFlag("qk", QUADRATIC_KOCH);
	
	syntax.addFlag("rp", REPELLING_POINTS_VAR, MSyntax::kDouble, MSyntax::kUnsigned);
	syntax.addFlag("rc", REPELLOR_CONSTANT_VAR, MSyntax::kDouble);
	syntax.addFlag("re", REPELLOR_EXPONENT_VAR, MSyntax::kUnsigned);
	syntax.addFlag("rs", REPELLOR_EXPONENT_VAR);
	syntax.addFlag("rns", RANDOM_SEED_VAR, MSyntax::kUnsigned);
	syntax.addFlag("rn", RANDOM_NOISE_VAR, MSyntax::kDouble);
	syntax.addFlag("rr", RANDOM_REWRITE_VAR, MSyntax::kUnsigned);
	syntax.addFlag("rv", REVERSIBLE_BNF_VAR);
	
	syntax.addFlag("sge", SAVE_GENOME_VAR, MSyntax::kString);
	syntax.addFlag("sg", SAVE_GRAMMAR_VAR, MSyntax::kString);
	syntax.addFlag("s", SCALE_VAR, MSyntax::kDouble);
	syntax.addFlag("sx", SCALE_X_VAR, MSyntax::kDouble);
	syntax.addFlag("sy", SCALE_Y_VAR, MSyntax::kDouble);
	syntax.addFlag("sz", SCALE_Z_VAR, MSyntax::kDouble);
	syntax.addFlag("sdp", SEED_POPULATION_VAR, MSyntax::kUnsigned, MSyntax::kUnsigned);
	syntax.addFlag("sf", SET_FITNESS_VAR, MSyntax::kUnsigned, MSyntax::kDouble);
	syntax.addFlag("si", SIZE_VAR, MSyntax::kDouble, MSyntax::kDouble);
	syntax.addFlag("siw", SIZE_WEIGHT_VAR, MSyntax::kDouble);
	syntax.addFlag("sm", SMOOTHNESS_VAR, MSyntax::kDouble);
	syntax.addFlag("smw", SMOOTHNESS_WEIGHT_VAR, MSyntax::kDouble);
	syntax.addFlag("sbw", SOFT_BOUNDARY_WEIGHT_VAR, MSyntax::kDouble);
	syntax.addFlag("sl", START_LENGTH_VAR, MSyntax::kDouble);
	syntax.makeFlagMultiUse(START_LENGTH_VAR);
	syntax.addFlag("sp", START_POSITION_VAR, MSyntax::kDouble, MSyntax::kDouble, MSyntax::kDouble);
	syntax.makeFlagMultiUse(START_POSITION_VAR);
	syntax.addFlag("n", STEPS_VAR, MSyntax::kUnsigned);
	syntax.addFlag("su", SUBDIVISIONS_VAR, MSyntax::kDouble);
	syntax.addFlag("suw", SUBDIVISIONS_WEIGHT_VAR, MSyntax::kDouble);
	syntax.addFlag("syb", SYMMETRIC_BNF_VAR);
	syntax.addFlag("sy", SYMMETRY_VAR, MSyntax::kDouble);
	syntax.addFlag("syw", SYMMETRY_WEIGHT_VAR, MSyntax::kDouble);
	
	syntax.addFlag("t", TILES_VAR, MSyntax::kUnsigned);
	syntax.addFlag("ts", TOURNAMENT_SIZE_VAR, MSyntax::kUnsigned);
	
	syntax.addFlag("un", UNDULATION_VAR, MSyntax::kDouble);
	syntax.addFlag("unw", UNDULATION_WEIGHT_VAR, MSyntax::kDouble);
	syntax.addFlag("uc", UNIFORM_CELLS_VAR);
	
	syntax.addFlag("vi", VIEW_INDIVIDUAL_VAR, MSyntax::kUnsigned);
	
	syntax.addFlag("wc", WALL_CONSTANT_VAR, MSyntax::kDouble);
	syntax.addFlag("we", WALL_EXPONENT_VAR, MSyntax::kUnsigned);
	
	syntax.addFlag("lb", "limitedBranchLength", MSyntax::kDouble);
	syntax.addFlag("a", "attractor", MSyntax::kDouble, MSyntax::kDouble, MSyntax::kDouble);
	syntax.makeFlagMultiUse("a");
	syntax.addFlag("r", "repeller", MSyntax::kDouble, MSyntax::kDouble, MSyntax::kDouble);
	syntax.makeFlagMultiUse("r");
	syntax.addFlag("ls", "loadScaffold", MSyntax::kString);
	//syntax.addFlag("ss", "saveScaffold", MSyntax::kString);
	syntax.makeFlagMultiUse("lge");
	syntax.makeFlagMultiUse("sf");
	
	//Secret flags, not to be documented
	syntax.addFlag("dy", DISPLAY_STATS_VAR);
	syntax.addFlag("rm", REMOVE_VAR);
	syntax.addFlag("de", DELETE_VAR);

	return syntax;
}


World* genr8::parseArgs( const MArgList& args )
{
	MStatus *status = new MStatus(MS::kSuccess);
	MSyntax	syntax = setSyntax();
	//Parse the arguments
	MArgParser argParser(syntax, args, status);
	if(*status==MS::kFailure)	{
		theWorld = NULL;
		return theWorld;
	}
	
	MString Mstr;
	MArgList argList;
	double x = 0, y = 0, z = 0, fit;
	int ID;
	unsigned tmp;
	Vector pos;
	bool removeMEL = false;
	//Help flag was used
	if(argParser.isFlagSet(HELP_VAR, status))	{
		//Help
		help();
		theWorld = NULL;
	}
	else if(argParser.isFlagSet(DELETE_VAR, status))	{
		//Just clean up
		getMELValues(true);
		theWorld = NULL;
	}
	else	{
		//Command line
		if(argParser.isFlagSet(REMOVE_VAR, status))
			removeMEL = true;
		//We always get the MEL-values from the optionvars. This can lead to unexpected results if
		//the environment is not properly cleaned. Cleaning is done by hitting cancel in the GUI.
		getMELValues(removeMEL);
		
		if(argParser.isFlagSet(DISPLAY_STATS_VAR, status))
			_showStats = true;
		//However, we can always override with the command-line parameters since they are checked later.
		//The order is mostly historical, but it can not be completly alphabetically ordered because some 
		//parameters must be checked before others.
		//I do not bother checking if the arguments are valid in most cases, I have faith in the user...
		if(argParser.isFlagSet(GRAVITY_VAR, status))	{
			argParser.getFlagArgument(GRAVITY_VAR, 0, Mstr);
			if(Mstr.asChar()=="x")
				_gravityDirection = Z;
			else if(Mstr.asChar()=="y")
				_gravityDirection = Y;
			argParser.getFlagArgument(GRAVITY_VAR, 1, _gravityMagnitude);
		}
		if(argParser.isFlagSet(KOCH_CURVE, status))
			_fractal = KOCH;
		else if(argParser.isFlagSet(QUADRATIC_KOCH, status))
			_fractal = Q_KOCH;
		else if(argParser.isFlagSet(TILES_VAR, status))	{
			argParser.getFlagArgument(TILES_VAR, 0, tmp);
			//Check if legal
			_tiles = tmp==3 || tmp==4 ? tmp : 0;
		}
		if(argParser.isFlagSet("lb", status))
			argParser.getFlagArgument("lb", 0, limitedBranches);

		if(argParser.isFlagSet(BRANCH_ANGLE_VAR, status))	{
			argParser.getFlagArgument(BRANCH_ANGLE_VAR, 0, tmp);
			_branchAngle = tmp;
		}
		argParser.getFlagArgument(BRANCH_LENGTH_VAR, 0, _branchLength);

		if(argParser.isFlagSet(BSPLINES_VAR, status))
			_surfaceRepresentation = BSPLINES;
		else if(argParser.isFlagSet(POLYGONS_VAR, status))
			_surfaceRepresentation = POLYGONS;
		
		if(argParser.isFlagSet(UNIFORM_CELLS_VAR, status))
			_uniformCells = true;

		if(argParser.getFlagArgument(SCALE_VAR, 0, _scale))	{
			_scaleX = _scale;
			_scaleY = _scale;
			_scaleZ = _scale;
		}
		argParser.getFlagArgument(SCALE_X_VAR, 0, _scaleX);
		argParser.getFlagArgument(SCALE_Y_VAR, 0, _scaleY);
		argParser.getFlagArgument(SCALE_Z_VAR, 0, _scaleZ);

		if(argParser.isFlagSet(STEPS_VAR, status))	{
			argParser.getFlagArgument(STEPS_VAR, 0, tmp);
			_steps = tmp;
		}
		
		if(argParser.isFlagSet(ACTIVE_WALLS_VAR, status))
			_wallBehaviour = ACTIVE_WALLS;
		if(argParser.isFlagSet(CUT_OFF_VAR, status))
			_wallBehaviour = CUT_OFF;
		argParser.getFlagArgument(RANDOM_NOISE_VAR, 0, _randomNoise);
		if(argParser.isFlagSet(FIXED_PERIMETER_VAR, status))
			_fixedPerimeter = true;
		if(argParser.isFlagSet(FIXED_CENTER_VAR, status))
			_fixedCenter = true;
		if(argParser.isFlagSet(POPULATION_SIZE_VAR, status))	{
			argParser.getFlagArgument(POPULATION_SIZE_VAR, 0, tmp);
			_populationSize = tmp;
			//Set the fitness vector
			initFitnessVector();
		}
		
		if(argParser.isFlagSet(GENERATIONS_VAR, status))	{
			argParser.getFlagArgument(GENERATIONS_VAR, 0, tmp);
			_generations = tmp;
		}
		if(argParser.isFlagSet(DRAW_EACH_STEP_VAR, status))
			_drawEachStep = true;
		
		if(argParser.numberOfFlagUses(START_LENGTH_VAR)>0)	{
			for(int m=0; m<argParser.numberOfFlagUses(START_LENGTH_VAR); m++)	{
				argParser.getFlagArgumentList(START_LENGTH_VAR, m, argList);
				argList.get(0, x);
				_startLengths.push_back(x);
			}
		}
		else
			_startLengths.push_back(START_LENGTH);
	
		if(argParser.numberOfFlagUses(START_POSITION_VAR)>0)	{
			for(int l=0; l<argParser.numberOfFlagUses(START_POSITION_VAR); l++)	{
				argParser.getFlagArgumentList(START_POSITION_VAR, l, argList);
				argList.get(0, x);
				argList.get(1, y);
				argList.get(2, z);
				pos.setVector(x, y, z);
				_startPositions.push_back(pos);
			}
		}
		else
			_startPositions.push_back(_startPosition);	
		
		for(int j=0; j<argParser.numberOfFlagUses("a"); j++)	{
			argParser.getFlagArgumentList("a", j, argList);
			argList.get(0, x);
			argList.get(1, y);
			argList.get(2, z);
			pos.setVector(x, y, z);
			attrPos.push_back(pos);
		}
		for(int k=0; k<argParser.numberOfFlagUses("r"); k++)	{
			argParser.getFlagArgumentList("r", k, argList);
			argList.get(0, x);
			argList.get(1, y);
			argList.get(2, z);
			pos.setVector(x, y, z);
			repPos.push_back(pos);
		}
		argParser.getFlagArgument(ATTRACTOR_CONSTANT_VAR, 0, _attractorConstant);
		argParser.getFlagArgument(REPELLOR_CONSTANT_VAR, 0, _repellorConstant);
		argParser.getFlagArgument(WALL_CONSTANT_VAR, 0, _wallConstant);
		if(argParser.isFlagSet(ATTRACTOR_EXPONENT_VAR, status))	{
			argParser.getFlagArgument(ATTRACTOR_EXPONENT_VAR, 0, tmp);
			_attractorExponent = tmp;
		}
		if(argParser.isFlagSet(REPELLOR_EXPONENT_VAR, status))	{
			argParser.getFlagArgument(REPELLOR_EXPONENT_VAR, 0, tmp);
			_repellorExponent = tmp;
		}
		if(argParser.isFlagSet(ATTRACTOR_SINGULAR_VAR, status))
			_attractorSingular = true;
		if(argParser.isFlagSet(REPELLOR_SINGULAR_VAR, status))
			_repellorSingular = true;
		if(argParser.isFlagSet(WALL_EXPONENT_VAR, status))	{
			argParser.getFlagArgument(WALL_EXPONENT_VAR, 0, tmp);
			_wallExponent = tmp;
		}
		if(argParser.isFlagSet(REPELLING_POINTS_VAR, status))	{
			argParser.getFlagArgument(REPELLING_POINTS_VAR, 0, _repellingPointsConstant);
			argParser.getFlagArgument(REPELLING_POINTS_VAR, 0, tmp);
			_repellingPointsExponent = tmp;
		}
		if(argParser.isFlagSet(RANDOM_SEED_VAR, status))	{
			argParser.getFlagArgument(RANDOM_SEED_VAR, 0, tmp);
			_randomSeed = tmp;
		}
		
		if(argParser.isFlagSet(ENCODED_GRAMMAR_VAR, status))	{
			argParser.getFlagArgument(ENCODED_GRAMMAR_VAR, 1, tmp);
			_growIndividual = tmp;
			argParser.getFlagArgument(ENCODED_GRAMMAR_VAR, 1, Mstr);
			_encodedGrammar = Mstr.asChar();
			getOptionVar(GROW_STEPS_VAR, _steps, true);
			//cout << "Encoded grammar is " << _encodedGrammar.c_str() << endl << "and steps is " << _steps << endl;
		}
		else if(argParser.isFlagSet(LOAD_GRAMMAR_VAR, status))	{
			argParser.getFlagArgument(LOAD_GRAMMAR_VAR, 0, Mstr);
			_loadGrammarFile = Mstr.asChar();
		}
		
		if(argParser.isFlagSet(SAVE_GRAMMAR_VAR, status))	{
			argParser.getFlagArgument(SAVE_GRAMMAR_VAR, 0, Mstr);
			_saveGrammarFile = Mstr.asChar();
		}
		
		Mstr.clear();
		argParser.getFlagArgument("ls", 0, Mstr);
		loadScaffold = Mstr.asChar();

		Mstr.clear();
		//argParser.getFlagArgument("ss", 0, Mstr);
		saveScaffold = Mstr.asChar();

		if(argParser.isFlagSet(CONTINUE_VAR, status))
			_continue = true;
		else	{
			Mstr = _loadGenomeFile.c_str();
			argParser.getFlagArgument(LOAD_GENOME_VAR, 0, Mstr);
			_loadGenomeFile = Mstr.asChar();
		}
		Mstr = _saveGenomeFile.c_str();
		argParser.getFlagArgument(SAVE_GENOME_VAR, 0, Mstr);
		_saveGenomeFile = Mstr.asChar();
		argParser.getFlagArgument(SMOOTHNESS_VAR, 0, _smoothness);
		argParser.getFlagArgument(SMOOTHNESS_WEIGHT_VAR, 0, _smoothnessWeight);
		argParser.getFlagArgument(SUBDIVISIONS_VAR, 0, _subdivisions);
		argParser.getFlagArgument(SUBDIVISIONS_WEIGHT_VAR, 0, _subdivisionsWeight);
		argParser.getFlagArgument(SIZE_VAR, 0, _sizeX);
		argParser.getFlagArgument(SIZE_VAR, 1, _sizeY);
		argParser.getFlagArgument(SIZE_WEIGHT_VAR, 0, _sizeWeight);
		argParser.getFlagArgument(SYMMETRY_VAR, 0, _symmetry);
		argParser.getFlagArgument(SYMMETRY_WEIGHT_VAR, 0, _symmetryWeight);
		argParser.getFlagArgument(SOFT_BOUNDARY_WEIGHT_VAR, 0, _softBoundaryWeight);
		argParser.getFlagArgument(UNDULATION_VAR, 0, _undulation);
		argParser.getFlagArgument(UNDULATION_WEIGHT_VAR, 0, _undulationWeight);

		if(argParser.isFlagSet(MUTATION_RATE_VAR, status))	{
			argParser.getFlagArgument(MUTATION_RATE_VAR, 0, tmp);
			_mutationRate = tmp;
		}
		if(argParser.isFlagSet(CROSSOVER_POINTS_VAR, status))	{
			argParser.getFlagArgument(CROSSOVER_POINTS_VAR, 0, tmp);
			_crossoverPoints = tmp;
		}
		if(argParser.isFlagSet(TOURNAMENT_SIZE_VAR, status))	{
			argParser.getFlagArgument(TOURNAMENT_SIZE_VAR, 0, tmp);
			_tournamentSize = tmp;
		}
		if(argParser.isFlagSet(ELITES_VAR, status))	{
			argParser.getFlagArgument(ELITES_VAR, 0, tmp);
			_elites = tmp;
		}
		if(argParser.isFlagSet(RANDOM_REWRITE_VAR, status))	{
			argParser.getFlagArgument(RANDOM_REWRITE_VAR, 0, tmp);
			tmp = _randomRewrite;
		}
		if(argParser.isFlagSet(FULL_RANDOM_REWRITE_VAR, status))
			_fullRandomRewrite = true;
		if(argParser.isFlagSet(GENOME_LENGTH_VAR, status))	{
			argParser.getFlagArgument(GENOME_LENGTH_VAR, 0, tmp);
			_genomeLength = tmp;
		}
		if(argParser.isFlagSet(VIEW_INDIVIDUAL_VAR, status))	{
			argParser.getFlagArgument(VIEW_INDIVIDUAL_VAR, 0, tmp);
			_viewIndividual = tmp;
		}
		if(argParser.isFlagSet(COPY_INDIVIDUAL_VAR, status))	{
			argParser.getFlagArgument(COPY_INDIVIDUAL_VAR, 0, tmp);
			_copyIndivdual = tmp;
			argParser.getFlagArgument(COPY_INDIVIDUAL_VAR, 1, tmp);
			_numberOfCopies = tmp;
		}
		if(argParser.isFlagSet(SEED_POPULATION_VAR, status))	{
			argParser.getFlagArgument(SEED_POPULATION_VAR, 0, tmp);
			_copyIndivdual = tmp;
			argParser.getFlagArgument(SEED_POPULATION_VAR, 1, tmp);
			_numberOfCopies = tmp;
			_exactCopies = false;
		}
		if(argParser.isFlagSet(REVERSIBLE_BNF_VAR, status))
			_bnfType = REVERSIBLE;
		else if(argParser.isFlagSet(PROBABILISTIC_BNF_VAR, status))
			_bnfType = PROBABILISTIC;
		else if(argParser.isFlagSet(SYMMETRIC_BNF_VAR, status))
			_bnfType = SYMMETRIC;

		for(int i=0; i<argParser.numberOfFlagUses(SET_FITNESS_VAR); i++)	{
			argParser.getFlagArgumentList(SET_FITNESS_VAR, i, argList);
			argList.get(0, ID);
			argList.get(1, fit);
			fitness[(int)ID] = fit;
		}
		if(argParser.isFlagSet(MAX_BNF_DEPTH_VAR, status))	{
			argParser.getFlagArgument(MAX_BNF_DEPTH_VAR, 0, tmp);
			_maxBNFDepth = tmp;
		}
		if(argParser.isFlagSet(MAX_EDGE_TYPES_VAR, status))	{
			argParser.getFlagArgument(MAX_EDGE_TYPES_VAR, 0, tmp);
			_maxEdgeTypes = tmp;
		}
		if(argParser.isFlagSet(NUMBER_OF_SCAFFOLDS_VAR, status))	{
			argParser.getFlagArgument(NUMBER_OF_SCAFFOLDS_VAR, 0, tmp);
			_numberOfScaffolds = tmp;
		}
		if(argParser.isFlagSet(DIFFERENT_GRAMMARS_VAR, status))
			_differentGrammars = true;

		if(_randomSeed==-1)	
			_randomSeed = (unsigned int)time(NULL);
		
		srand(_randomSeed);
		//If the user for some reason uses this flag, we want the GUI to be updated with the 
		//values that was used previously.
		if(argParser.isFlagSet(GUI_VAR, status))	{
			//GUI
			createInputWindow();
			//We end this command here and start a new one when the user presses button in the GUI
			//In that way, we are in the Maya environment all the time.
			theWorld = NULL;
			return theWorld;
		}
		//Now we can create the world, it is a lot of parameters...
		theWorld = new World(attrPos, repPos, _attractorSingular, _repellorSingular, _gravityDirection, _gravityMagnitude, _wallBehaviour, _tiles, _randomNoise, _fixedPerimeter, _fixedCenter, _attractorConstant, _repellorConstant, _wallConstant, _attractorExponent, _repellorExponent, _wallExponent, _repellingPointsConstant, _repellingPointsExponent, _populationSize, _fractal, _surfaceRepresentation, _scaleX, _scaleY, _scaleZ, _loadGrammarFile, limitedBranches, _branchAngle, _tournamentSize, _elites, _softBoundaryWeight!=0, _bnfType, _genomeLength, _randomRewrite, _fullRandomRewrite, _maxBNFDepth, _maxEdgeTypes, _numberOfScaffolds, _differentGrammars, _crossoverPoints, _uniformCells, _branchLength, _encodedGrammar);
		//We reduce the length of the parameterlist somewhat by moving some stuff to here
		theWorld->createAgents(_smoothness, _smoothnessWeight, _subdivisions, _subdivisionsWeight, _sizeX, _sizeY, _sizeWeight, _symmetry, _symmetryWeight, _softBoundaryWeight, _undulation, _undulationWeight);
		theWorld->getGeneticEngine()->setMutationProbability(_mutationRate);
		
	}
	
	return theWorld;
}

void genr8::help()
{
	//Use the Maya-function for firing up the browser
	string help;
	help = getStringResults("getenv \"MAYA_LOCATION\"");
	help += "/bin/plug-ins/GENR8_helpfiles/help.html";
	help = "doBrowserHelp {\"UserAbsolute\", \"" + help + "\"};";
	eval(help);
}

//Use this to get the data from the user-defined seed

void genr8::parseCurve(MDagPath curveDag)
{
	Vector point;
	double vec[4];
	//Build the curve
	MStatus *stat = new MStatus(MS::kSuccess);
	MFnNurbsCurve* curve = new MFnNurbsCurve(curveDag,stat);
	MPointArray cvs;
	//Get the CVs
	curve->getCVs(cvs, MSpace::kWorld);
	for(int i=0; i<cvs.length(); i++)	{
		cvs[i].get(vec);
		point.setVector(vec[0], vec[1], vec[2]);
		_seed.push_back(point);
		//Get the weight from the point and use it as mass
		_masses.push_back(vec[4]);
	}
	//Check if curve is closed, if so, we must remove the last vertex, to avoid overlap in our internal model
	if(_seed[0]==_seed[_seed.size() - 1])
		_seed.erase(_seed.begin());
}

//Since the user can set the fitness values by hand, we provide the genetic engine
//with a vector containing the fitness values for each ind. If the value is -1, the user
//has not been fiddling around with it.

void genr8::initFitnessVector()
{
	fitness.clear();
	for(int i=0; i<_populationSize; i++)
		fitness.push_back(-1);
	//Check if the values have been set by hand
	//In this vector we first store the ID of the individual and then the new value
	vector<double> setfit;
	getOptionVar(SET_FITNESS_VAR, setfit, true);
	for(int j=0; j<setfit.size()/2; j++)	{
		if(setfit[2*j]<_populationSize)
			fitness[setfit[2*j]] = setfit[2*j+1];
	}
}

//Use this function to get data about the attractors and repellors placed in the scene

void genr8::fixAttractorsAndRepellors()
{
	//Add the attractors and repellors that have already been drawn in the scene
	MSelectionList repList, attrList;
	//cout << "Getting repellors from scene..." << endl;
	int nRepellors, nAttractors, exponent;
	double konst;
	bool singular;
	Vector pos;
	nRepellors = getIntResult("optionVar -q nRepellors");
	//We look at the optionVars stored by repellor to get the data for the repellors
	for(int i=0; i<nRepellors; i++)	{
		ostringstream exp, kons, posX, posY, posZ, change, sing;
		exp << "optionVar -q " << REPELLOR_EXPONENT_VAR << i;
		exponent = getIntResult(exp.str());
		kons << "optionVar -q " << REPELLOR_CONSTANT_VAR << i;
		konst = getDoubleResult(kons.str());
		sing << "optionVar -q " << REPELLOR_SINGULAR_VAR << i;
		singular = getIntResult(sing.str())!=0;
		//Get the position
		//The user is allowed to move the repellor, so we must find out where it is and update the variables while we are at it
		posX << "getAttr " << REP_NAME << i << ".translateX";
		pos.setX(getDoubleResult(posX.str()));
		change << "optionVar -fv " << REP_POS_X << i << " " << pos.getX() << ";";
		posY << "getAttr " << REP_NAME << i << ".translateY";
		pos.setY(getDoubleResult(posY.str()));
		change << "optionVar -fv " << REP_POS_Y << i << " " << pos.getY() << ";";
		posZ << "getAttr " << REP_NAME << i << ".translateZ";
		pos.setZ(getDoubleResult(posZ.str()));
		change << "optionVar -fv " << REP_POS_Z << i << " " << pos.getZ() << ";";
		eval(change.str());

		theWorld->getForce()->newAttractor(false, pos, exponent, konst, true);
	}

	//cout << "Getting attractors from scene..." << endl;
	nAttractors = getIntResult("optionVar -q nAttractors");
	//We look at the optionVars stored by repellor to get the data for the repellors
	for(int j=0; j<nAttractors; j++)	{
		ostringstream exp, kons, posX, posY, posZ, change, sing;
		exp << "optionVar -q " << ATTRACTOR_EXPONENT << j;
		exponent = getIntResult(exp.str());
		kons << "optionVar -q " << ATTRACTOR_CONSTANT << j;
		konst = getDoubleResult(kons.str());
		sing << "optionVar -q " << ATTRACTOR_SINGULAR_VAR << i;
		singular = getIntResult(sing.str())!=0;
		//Get the position
		//The user is allowed to move the attractor, so we must find out where it is and update the variables while we are at it
		posX << "getAttr " << ATTR_NAME << j << ".translateX";
		pos.setX(getDoubleResult(posX.str()));
		change << "optionVar -fv " << ATTR_POS_X << j << " " << pos.getX() << ";";
		posY << "getAttr " << ATTR_NAME << j << ".translateY";
		pos.setY(getDoubleResult(posY.str()));
		change << "optionVar -fv " << ATTR_POS_Y << j << " " << pos.getY() << ";";
		posZ << "getAttr " << ATTR_NAME << j << ".translateZ";
		pos.setZ(getDoubleResult(posZ.str()));
		change << "optionVar -fv " << ATTR_POS_Z << j << " " << pos.getZ() << ";";
		eval(change.str());

		theWorld->getForce()->newAttractor(true, pos, exponent, konst, true);
	}
	//We can not draw the attractors and repellors until now because it will screw up the selectionlist...
	theWorld->getForce()->drawAttractors();
}

void genr8::createInputWindow()
{
	//Not very easy to read....
	ostringstream layout, show;
	//Make window
	layout << "string " << INPUT_WINDOW << " = `window -title genr8 -sizeable on -titleBar on -minimizeButton on -maximizeButton on -widthHeight 400 500`;";
	//string* window = getStringPointerResult(wind.str());
	//Fix layout
	layout << "scrollLayout; columnLayout -adjustableColumn on;";
	//Scaffold
	layout << "frameLayout -label \"Scaffold\" -collapsable on -collapse on;columnLayout;";
	layout << "intFieldGrp -label \"Steps\" -value1 " << _steps << " -changeCommand \"optionVar -intValue " << STEPS_VAR << " #1\";";
	layout << "floatFieldGrp -label \"Scale\" -extraLabel \">= 0\" -value1 " << _scale << " -changeCommand \"modifyDialog -" << CHANGE_SCALE_VAR << " #1\";";
	layout << "string " << SCALE_X_FIELD << " = `floatFieldGrp -label \"ScaleX\" -extraLabel \">= 0\" -value1 " << _scaleX << " -changeCommand \"optionVar -fv " << SCALE_X_VAR << " #1\"`;";
	layout << "string " << SCALE_Y_FIELD << " = `floatFieldGrp -label \"ScaleY\" -extraLabel \">= 0\" -value1 " << _scaleY << " -changeCommand \"optionVar -fv " << SCALE_Y_VAR << " #1\"`;";
	layout << "string " << SCALE_Z_FIELD << " = `floatFieldGrp -label \"ScaleZ\" -extraLabel \">= 0\" -value1 " << _scaleZ << " -changeCommand \"optionVar -fv " << SCALE_Z_VAR << " #1\"`;";
	layout << "radioButtonGrp -numberOfRadioButtons 3 -label \"Representation\" -select " << (int)_surfaceRepresentation + 1 << " -labelArray3 \"Surface\" \"B-Splines\" \"Polygons\" -onCommand1 \"modifyDialog -" << SURFACE_VAR << "\" -onCommand2 \"modifyDialog -" << BSPLINES_VAR << "\" -onCommand3 \"modifyDialog -" << POLYGONS_VAR << "\";";
	layout << "checkBoxGrp -label \"Uniform Cells\" -columnAlign 1 center -value1 " << (int)_uniformCells << " -changeCommand \"optionVar -iv " << UNIFORM_CELLS_VAR << " #1\";";
	layout << "checkBoxGrp -label \"Fixed Perimeter\" -columnAlign 1 center -value1 " << (int)_fixedPerimeter << " -changeCommand \"optionVar -iv " << FIXED_PERIMETER_VAR << " #1\";";
	layout << "checkBoxGrp -label \"Fixed Center\" -columnAlign 1 center -value1 " << (int)_fixedCenter << " -changeCommand \"optionVar -iv " << FIXED_CENTER_VAR << " #1\";";
	layout << "checkBoxGrp -label \"Draw Each Step\" -columnAlign 1 center -value1 " << (int)_drawEachStep << " -changeCommand \"optionVar -iv " << DRAW_EACH_STEP_VAR << " #1\";";
	layout << "floatFieldGrp -label \"Start Length\" -extraLabel \">= 0\" -value1 " << _startLength << " -changeCommand \"optionVar -fv " << START_LENGTH_VAR << " #1\";";
	layout << "floatFieldGrp -label \"Start Position X\" -value1 " << _startPosition.getX() << " -changeCommand \"optionVar -fv " << START_POSITION_X_VAR << " #1\";";
	layout << "floatFieldGrp -label \"Start Position Y\" -value1 " << _startPosition.getY() << " -changeCommand \"optionVar -fv " << START_POSITION_Y_VAR << " #1\";";
	layout << "floatFieldGrp -label \"Start Position Z\" -value1 " << _startPosition.getZ() << " -changeCommand \"optionVar -fv " << START_POSITION_Z_VAR << " #1\";";
	layout << "intFieldGrp -label \"Branch Angle\" -extraLabel \"0<x<90\" -value1 " << _branchAngle << " -changeCommand \"optionVar -intValue " << BRANCH_ANGLE_VAR << " #1\";";
	layout << "floatFieldGrp -label \"Branch Length\" -value1 " << _branchLength << " -changeCommand \"optionVar -floatValue " << BRANCH_LENGTH_VAR << " #1\";";
	layout << "setParent ..;setParent ..;";
	//Environment
	layout << "frameLayout -label \"Environment\" -collapsable on -collapse on;columnLayout;";
	layout << "floatFieldGrp -label \"Gravity Magnitude\" -value1 " << _gravityMagnitude << " -changeCommand \"optionVar -floatValue " << GRAVITY_MAGNITUDE_VAR << " #1\";";
	if(_gravityDirection==X)
		layout << "radioButtonGrp -numberOfRadioButtons 3 -select 1 -label \"Gravity Direction\" -labelArray3 \"x\" \"y\" \"z\" -onCommand1 \"optionVar -stringValue " << GRAVITY_DIRECTION_VAR << " x\" -onCommand2 \"optionVar -stringValue " << GRAVITY_DIRECTION_VAR << " y\" -onCommand3 \"optionVar -stringValue " << GRAVITY_DIRECTION_VAR << " z\";";
	else if(_gravityDirection==Y)
		layout << "radioButtonGrp -numberOfRadioButtons 3 -select 2 -label \"Gravity Direction\" -labelArray3 \"x\" \"y\" \"z\" -onCommand1 \"optionVar -stringValue " << GRAVITY_DIRECTION_VAR << " x\" -onCommand2 \"optionVar -stringValue " << GRAVITY_DIRECTION_VAR << " y\" -onCommand3 \"optionVar -stringValue " << GRAVITY_DIRECTION_VAR << " z\";";
	else
		layout << "radioButtonGrp -numberOfRadioButtons 3 -select 3 -label \"Gravity Direction\" -labelArray3 \"x\" \"y\" \"z\" -onCommand1 \"optionVar -stringValue " << GRAVITY_DIRECTION_VAR << " x\" -onCommand2 \"optionVar -stringValue " << GRAVITY_DIRECTION_VAR << " y\" -onCommand3 \"optionVar -stringValue " << GRAVITY_DIRECTION_VAR << " z\";";
	//layout << "radioButtonGrp -numberOfRadioButtons 3 -select " << (int)_wallBehaviour + 1 << " -label \"Wall Behaviour\" -labelArray3 \"Default\" \"Cut Off\" \"Active Walls\" -onCommand1 \"optionVar -intValue " << WALL_BEHAVIOUR_VAR << " " << DEFAULT_WB_VAR << "\" -onCommand2 \"optionVar -stringValue " << WALL_BEHAVIOUR_VAR << " " << CUT_OFF_VAR << "\" -onCommand3 \"modifyDialog -" << ACTIVE_WALLS_VAR << " on;\" -offCommand3 \"modifyDialog -" << ACTIVE_WALLS_VAR << " off;\";";
	layout << "radioButtonGrp -numberOfRadioButtons 2 -select " << (int)_wallBehaviour + 1 << " -label \"Wall Behaviour\" -labelArray2 \"Default\" \"Cut Off\" -onCommand1 \"optionVar -intValue " << WALL_BEHAVIOUR_VAR << " " << DEFAULT_WB_VAR << "\" -onCommand2 \"optionVar -stringValue " << WALL_BEHAVIOUR_VAR << " " << CUT_OFF_VAR << "\";";
	layout << "floatFieldGrp -label \"Random Noise\" -extraLabel \">= 0\" -value1 " << _randomNoise << " -changeCommand \"optionVar -fv " << RANDOM_NOISE_VAR << " #1\";";
	layout << "intFieldGrp -label \"Random Rewrite\" -extraLabel \"0<x<100\" -value1 " << _randomRewrite << " -changeCommand \"optionVar -iv " << RANDOM_REWRITE_VAR << " #1\";";
	//layout << "string " << WALL_CONSTANT_FIELD << " = `floatFieldGrp -enable false -label \"Wall Constant\" -value1 " << _wallConstant << " -changeCommand \"optionVar -fv " << WALL_CONSTANT_VAR << " #1\"`;";
	//layout << "string " << WALL_EXPONENT_FIELD << " = `intFieldGrp -enable false -label \"Wall Exponent\" -value1 " << _wallExponent << " -changeCommand \"optionVar -iv " << WALL_EXPONENT_VAR << " #1\"`;";
	layout << "floatFieldGrp -label \"Repelling Points Constant\" -value1 " << _repellingPointsConstant << " -changeCommand \"optionVar -fv " << REPELLING_POINTS_CONSTANT_VAR << " #1\";";
	layout << "intFieldGrp -label \"Repelling Points Exponent\" -value1 " << _repellingPointsExponent << " -changeCommand \"optionVar -iv " << REPELLING_POINTS_EXPONENT_VAR << " #1\";";
	layout << "intFieldGrp -label \"Random Seed\" -changeCommand \"optionVar -iv " << _randomSeed << " #1\";";
	layout << "setParent..;setParent..;";
	//Grammar
	layout << "frameLayout -label \"Grammar\" -collapsable on -collapse on; columnLayout;";
	if(_tiles!=0)	{
		layout << "string " << TILES_RADIO_GROUP << " = `radioButtonGrp -numberOfRadioButtons 2 -label \"Tiles\" -select " << _tiles - 2 << " -labelArray2 \"3\" \"4\" -onCommand1 \"modifyDialog -p true -t 3\" -onCommand2 \"modifyDialog -p true -t 4\"`;";
		layout << "radioButtonGrp -shareCollection " << TILES_RADIO_GROUP << " -numberOfRadioButtons 2 -label \"Fractals\" -labelArray2 \"Koch curve\" \"Quadratic Koch\" -onCommand1 \"modifyDialog -p true -f " << KOCH_CURVE << "\" -onCommand2 \"modifyDialog -p true -f " << QUADRATIC_KOCH << "\";";
		layout << "radioButtonGrp -shareCollection " << TILES_RADIO_GROUP << " -numberOfRadioButtons 4 -label \"BNF\" -labelArray4 \"Default\" \"Reversible\" \"Symmetric\" \"Probabilistic\" -onCommand1 \"modifyDialog -p false -b " << DEFAULT_BNF_VAR << "\" -onCommand2 \"modifyDialog -p false -b " << REVERSIBLE_BNF_VAR << "\" -onCommand3 \"modifyDialog -p false -b " << SYMMETRIC_BNF_VAR << "\" -onCommand4 \"modifyDialog -p false -b " << PROBABILISTIC_BNF_VAR << "\";";
	}
	else if(_fractal!=NO_FRACTAL)	{
		layout << "string " << TILES_RADIO_GROUP << " = `radioButtonGrp -numberOfRadioButtons 2 -label \"Tiles\" -labelArray2 \"3\" \"4\" -onCommand1 \"modifyDialog -p true -t 3\" -onCommand2 \"modifyDialog -p true -t 4\"`;";
		layout << "radioButtonGrp -shareCollection " << TILES_RADIO_GROUP << " -numberOfRadioButtons 2 -label \"Fractals\" -select " << _fractal << " -labelArray2 \"Koch curve\" \"Quadratic Koch\" -onCommand1 \"modifyDialog -p true -f " << KOCH_CURVE << "\" -onCommand2 \"modifyDialog -p true -f " << QUADRATIC_KOCH << "\";";
		layout << "radioButtonGrp -shareCollection " << TILES_RADIO_GROUP << " -numberOfRadioButtons 4 -label \"BNF\" -labelArray4 \"Default\" \"Reversible\" \"Symmetric\" \"Probabilistic\" -onCommand1 \"modifyDialog -p false -b " << DEFAULT_BNF_VAR << "\" -onCommand2 \"modifyDialog -p false -b " << REVERSIBLE_BNF_VAR << "\" -onCommand3 \"modifyDialog -p false -b " << SYMMETRIC_BNF_VAR << "\" -onCommand4 \"modifyDialog -p false -b " << PROBABILISTIC_BNF_VAR << "\";";
	}
	else	{
		layout << "string " << TILES_RADIO_GROUP << " = `radioButtonGrp -numberOfRadioButtons 2 -label \"Tiles\" -labelArray2 \"3\" \"4\" -onCommand1 \"modifyDialog -p true -t 3\" -onCommand2 \"modifyDialog -p true -t 4\"`;";
		layout << "radioButtonGrp -shareCollection " << TILES_RADIO_GROUP << " -numberOfRadioButtons 2 -label \"Fractals\" -labelArray2 \"Koch curve\" \"Quadratic Koch\" -onCommand1 \"modifyDialog -p true -f " << KOCH_CURVE << "\" -onCommand2 \"modifyDialog -p true -f " << QUADRATIC_KOCH << "\";";
		layout << "radioButtonGrp -shareCollection " << TILES_RADIO_GROUP << " -numberOfRadioButtons 4 -select 1 -label \"BNF\" -labelArray4 \"Default\" \"Reversible\" \"Symmetric\" \"Probabilistic\" -onCommand1 \"modifyDialog -p false -b " << DEFAULT_BNF_VAR << "\" -onCommand2 \"modifyDialog -p false -b " << REVERSIBLE_BNF_VAR << "\" -onCommand3 \"modifyDialog -p false -b " << SYMMETRIC_BNF_VAR << "\" -onCommand4 \"modifyDialog -p false -b " << PROBABILISTIC_BNF_VAR << "\";";
	}
	layout << "string " << MAX_BNF_DEPTH_FIELD << " = `intFieldGrp -label \"Max BNF Depth\" -extraLabel \">= 0\" -value1 " << _maxBNFDepth << " -cc \"optionVar -iv " << MAX_BNF_DEPTH_VAR << " #1\"`;";
	layout << "string " << MAX_EDGE_TYPES_FIELD << " = `intFieldGrp -label \"Max Edge Types\" -value1 " << _maxEdgeTypes << " -cc \"optionVar -iv " << MAX_EDGE_TYPES_VAR << " #1\"`;";
	layout << "string " << FULL_RANDOM_REWRITE_BOX << " = `checkBoxGrp -enable off -label \"Full Random Rewrite\" -columnAlign 1 center -value1 " << (int)_fullRandomRewrite << " -cc \"optionVar -iv " << FULL_RANDOM_REWRITE_VAR << " #1\"`;";
	layout << "button -label \"Load Grammar\" -command \"modifyDialog -" << LOAD_GRAMMAR_VAR << "\";";
	layout << "setParent..;setParent..;";
	//Genetic Engine
	layout << "string " << GENETIC_ENGINE_LAYOUT << " = `frameLayout -label \"Genetic Engine\" -collapsable on -collapse on`;";
	layout << "columnLayout; intFieldGrp -label \"Population Size\" -extraLabel \">= 1\" -value1 " << _populationSize << " -cc \"optionVar -iv " << POPULATION_SIZE_VAR << " #1\";";
	layout << "intFieldGrp -label \"Generations\" -extraLabel \">= 0\" -value1 " << _generations << " -cc \"optionVar -iv " << GENERATIONS_VAR << " #1\";";
	layout << "intFieldGrp -label \"Mutation Rate\" -extraLabel \"0<x<100\" -value1 " << _mutationRate << " -cc \"optionVar -iv " << MUTATION_RATE_VAR << " #1\";";
	layout << "intFieldGrp -label \"Crossover Points\" -extraLabel \">= 1\" -value1 " << _crossoverPoints << " -cc \"optionVar -iv " << CROSSOVER_POINTS_VAR << " #1\";";
	layout << "intFieldGrp -label \"Tournament Size\" -value1 " << _tournamentSize << " -cc \"optionVar -iv " << TOURNAMENT_SIZE_VAR << " #1\";";
	layout << "intFieldGrp -label \"Elites\" -value1 " << _elites << " -cc \"optionVar -iv " << ELITES_VAR << " #1\";";
	layout << "intFieldGrp -label \"Genome Length\" -value1 " << _genomeLength << " -changeCommand \"optionVar -iv " << GENOME_LENGTH_VAR << " #1\";";
	layout << "intFieldGrp -label \"View Individual\" -changeCommand \"optionVar -iv " << _viewIndividual << " #1\";";
	layout << "checkBoxGrp -label \"Continue\" -columnAlign 1 center -value1 " << _continue << " -changeCommand \"optionVar -iv " << CONTINUE_VAR << " #1\";";
	layout << "button -label \"Load Genome\" -command \"modifyDialog -" << LOAD_GENOME_VAR << "\";";
	//extrude -ch true -rn false -po 0 -et 2 -ucp 1 -fpt 1 -upn 1 -rotation 0 -scale 1 -rsp 1 "curve5" "loftedSurface1->curve4" ;

	layout << "setParent..;setParent..;";
	//Fitness
	layout << "string " << FITNESS_LAYOUT << " = `frameLayout -label \"Fitness\" -collapsable on -collapse on`; columnLayout;";
	layout << "floatFieldGrp -label \"Size X\" -extraLabel \">= 0\" -value1 " << _sizeX << " -cc \"optionVar -fv " << SIZE_X_VAR << " #1\";";
	layout << "floatFieldGrp -label \"Size Y\" -extraLabel \">= 0\" -value1 " << _sizeY << " -cc \"optionVar -fv " << SIZE_Y_VAR << " #1\";";
	layout << "floatFieldGrp -label \"Size Weight\" -extraLabel \">= 0\" -value1 " << _sizeWeight << " -cc \"optionVar -fv " << SIZE_WEIGHT_VAR << " #1\";";
	layout << "floatFieldGrp -label \"Smoothness\" -extraLabel \">= 0\" -value1 " << _smoothness << " -cc \"optionVar -fv " << SMOOTHNESS_VAR << " #1\";";
	layout << "floatFieldGrp -label \"Smoothness Weight\" -extraLabel \">= 0\" -value1 " << _smoothnessWeight << " -cc \"optionVar -fv " << SMOOTHNESS_WEIGHT_VAR << " #1\";";
	layout << "floatFieldGrp -label \"Soft Boundary Weight\" -extraLabel \">= 0\" -value1 " << _softBoundaryWeight << " -cc \"optionVar -fv " << SOFT_BOUNDARY_WEIGHT_VAR << " #1\";";
	layout << "floatFieldGrp -label \"Subdivisions\" -extraLabel \"0<x<1\" -value1 " << _subdivisions << " -cc \"optionVar -fv " << SUBDIVISIONS_VAR << " #1\";";
	layout << "floatFieldGrp -label \"Subdivisions Weight\" -extraLabel \">= 0\" -value1 " << _subdivisionsWeight << " -cc \"optionVar -fv " << SUBDIVISIONS_WEIGHT_VAR << " #1\";";
	layout << "floatFieldGrp -label \"Symmetry\" -extraLabel \"0<x<1\" -value1 " << _symmetry << " -cc \"optionVar -fv " << SYMMETRY_VAR << " #1\";";
	layout << "floatFieldGrp -label \"Symmetry Weight\" -extraLabel \">= 0\" -value1 " << _symmetryWeight << " -cc \"optionVar -fv " << SYMMETRY_WEIGHT_VAR << " #1\";";
	layout << "floatFieldGrp -label \"Undulation\" -extraLabel \">= 0\" -value1 " << _undulation << " -cc \"optionVar -fv " << UNDULATION_VAR << " #1\";";
	layout << "floatFieldGrp -label \"Undulation Weight\" -extraLabel \">= 0\" -value1 " << _undulationWeight << " -cc \"optionVar -fv " << UNDULATION_WEIGHT_VAR << " #1\";";
	layout << "setParent..;setParent..;";
	//Buttons
	//layout << "setParent ..; columnLayout;";
	layout << "button -label \"GENR8\" -align left -command \"deleteUI -window " << INPUT_WINDOW << ";genr8 -" << REMOVE_VAR << ";\";";
	layout << "button -label \"GENR8 and Show Stats\" -align left -command \"deleteUI -window " << INPUT_WINDOW << ";genr8 -" << DISPLAY_STATS_VAR << ";\";";
	layout << "button -label \"Save Settings\" -align center -command \"modifyDialog -" << SAVE_SETTINGS_VAR << "\";";
	layout << "button -label \"Load Settings\" -align center -command \"genr8 -" << DELETE_VAR << ";modifyDialog -" << LOAD_SETTINGS_VAR << ";deleteUI -window " << INPUT_WINDOW << ";genr8 -" << GUI_VAR << "\";";
	layout << "button -label \"Help\" -align left -command \"genr8 -" << HELP_VAR << "\";";
	layout << "button -label \"Cancel\" -align left -command \"genr8 -" << DELETE_VAR << "; deleteUI -window " << INPUT_WINDOW << "\";";
	eval(layout.str());

	//Show window
	show << "showWindow " << INPUT_WINDOW;
	eval(show.str());
	
}

void genr8::growIndividualWindow(int ind, int steps)
{
	//Not very easy to read....
	ostringstream layout, show;
	//Make window
	layout << "string " << GROW_WINDOW << ind << " = `window -title \"genr8 - Individual " << ind << "\" -sizeable on -titleBar on -minimizeButton on -maximizeButton on -widthHeight 400 500`;";
	//Fix layout
	layout << "scrollLayout; columnLayout -adjustableColumn on;";
	//One button for each step
	for(int k=0; k<steps+1; k++)
		layout << "button -label \"Show step " << k << "\" -command \"regenr8 -n " << ind << " " << k << " on\";";
	layout << "button -label \"Show all steps\" -command \"";
	for(int i=0; i<steps+1; i++)
		layout << "regenr8 -n " << ind << " " << i << " on; ";
	layout << "\";";
	layout << "button -label \"Hide all steps\" -command \"";
	for(int j=0; j<steps+1; j++)
		layout << "regenr8 -n " << ind << " " << j << " off; ";
	layout << "\";";
	layout << "button -label \"Cancel\" -command \"regenr8 -a off; deleteUI -window " << GROW_WINDOW << ind << "\";";
	layout << "setParent..;setParent..;";
	eval(layout.str());

	//Show window
	show << "showWindow " << GROW_WINDOW << ind;
	eval(show.str());
}

void genr8::createOutputWindow()
{
	ostringstream layout, show;
	//Make window
	layout << "string " << OUTPUT_WINDOW << " = `window -title genr8 -sizeable on -titleBar on -minimizeButton on -maximizeButton on -widthHeight 400 500`;";
	//Fix layout
	layout << "scrollLayout; columnLayout -adjustableColumn on;";
	//Scaffold
	double fit;
	int ID, numberOfLines;
	for(int i=0; i<_populationSize; i++)	{
		fit = theWorld->getIndividualRanked(i)->getImpression();
		ID = theWorld->getIndividualRanked(i)->getID();
		Individual* ind = theWorld->getIndividualRanked(i);
		
		layout << "frameLayout -label \""<< i << " Individual" << ID << ", Fitness " << fit << "\" -collapsable on -collapse on -collapseCommand \"regenr8 -s " << ID << " off\" -expandCommand \"regenr8 -s " << ID << " on\";";
		layout << "columnLayout;";
		//Grammar
		layout << "frameLayout -label \"Grammar\" -collapsable on; columnLayout;";
		numberOfLines = ind->getSentence()->numberOfLines();
		for(int j=0; j<numberOfLines; j++)
			layout << "text -align \"left\" -label \""<< ind->getSentence()->getLine(j)->str().c_str() << "\";";
		layout << "button -label \"Save Genome\" -command \"modifyDialog -" << SAVE_GENOME_VAR << " " << encodeString(ind->getGenome()->str()).c_str() << "\";";
		layout << "button -label \"Save Grammar\" -command \"modifyDialog -" << SAVE_GRAMMAR_VAR << " " << encodeString(ind->getSentence()->str()).c_str() << "\";";
		layout << "button -label \"Save Visible\" -command \"select -clear; select -visible; ExportSelection; select -clear\";";
		layout << "intFieldGrp -label \"Grow individual\" -extraLabel \"steps\" -cc \"optionVar -iv " << GROW_STEPS_VAR << " #1\";";
		layout << "button -label \"Grow Individual\" -command \"regenr8 -a off; genr8 -" << POPULATION_SIZE_VAR << " 1 -" << GENERATIONS_VAR << " 0 -" << ENCODED_GRAMMAR_VAR << " " << i << " " << (encodeString(ind->getSentence()->str())).c_str() << " -" << DRAW_EACH_STEP_VAR << "\";";
		//layout << "textFieldButtonGrp -label \"Grow Individual\" -text " << _steps << " -buttonLabel \"Steps\" -buttonCommand \"degenr8 -c -l; genr8 -" << POPULATION_SIZE_VAR << " 1 -" << ENCODED_GRAMMAR_VAR << " " << (encodeString(ind->getSentence()->str())).c_str() << " -" << DRAW_EACH_STEP_VAR << " -" << STEPS_VAR << " #1\";";
		layout << "setParent ..;setParent ..;";
		//Fitness
		layout << "frameLayout -label \"Fitness\" -collapsable on; columnLayout;";
		layout << "floatFieldGrp -label \"Fitness\" -value1 " << fit << " -cc \"optionVar -floatValueAppend " << SET_FITNESS_VAR << " " << i << " -fva " << SET_FITNESS_VAR << " #1\";";
		//Agents
		layout << "floatFieldGrp -label \"Size\" -enable1 off -value1 " << ind->getSizeFitness() << ";";
		layout << "floatFieldGrp -label \"Smoothness\" -enable1 off -value1 " << ind->getSmoothnessFitness() << ";";
		layout << "floatFieldGrp -label \"Soft Boundary\" -enable1 off -value1 " << ind->getSoftBoundaryFitness() << ";";
		layout << "floatFieldGrp -label \"Subdivisions\" -enable1 off -value1 " << ind->getSubdivisionsFitness() << ";";
		layout << "floatFieldGrp -label \"Symmetry\" -enable1 off -value1 " << ind->getSymmetryFitness() << ";";
		layout << "floatFieldGrp -label \"Undulation\" -enable1 off -value1 " << ind->getUndulatingFitness() << ";";
		layout << "setParent ..;setParent ..;";
		layout << "setParent ..;setParent ..;";
	}
	//Copying
	layout << "frameLayout -label \"Copying\" -collapsable on -collapse on; columnLayout;";
	layout << "intFieldGrp -label \"Copy Individual Ranked\" -value1 " << _copyIndivdual << " -changeCommand \"optionVar -iv " << COPY_INDIVIDUAL_VAR << " #1\";";
	layout << "intFieldGrp -label \"Number Of Copies\" -value1 " << _numberOfCopies << " -changeCommand \"optionVar -iv " << NUMBER_OF_COPIES_VAR << " #1\";";
	layout << "radioButtonGrp -numberOfRadioButtons 2 -label \"Copy Style\" -labelArray2 \"Exact\" \"Mutate\" -onCommand1 \"optionVar -iv " << EXACT_COPIES_VAR << " 1\" -onCommand2 \"optionVar -iv " << EXACT_COPIES_VAR << " 0\";";
	layout << "setParent ..;setParent ..;";
	//Buttons
	layout << "button -label \"Continue GENR8\" -align center -command \"deleteUI -window " << OUTPUT_WINDOW << "; degenr8 -c -l; genr8 -" << CONTINUE_VAR << " -" << DISPLAY_STATS_VAR << ";\";";
	layout << "button -label \"GENR8 and Change Parameters\" -align center -command \"deleteUI -window " << OUTPUT_WINDOW << "; degenr8 -c -l; genr8 -" << GUI_VAR << ";\";";
	if(_populationSize==1 && _drawEachStep)	{
		for(int k=0; k<_steps+1; k++)
			layout << "button -label \"Show step " << k << "\" -command \"regenr8 -n 0 " << k << " on\";";
	}
	layout << "button -label \"Show All\" -align center -command \"regenr8 -a on\";";
	layout << "button -label \"Hide All\" -align center -command \"regenr8 -a off\";";
	layout << "button -label \"degenr8\" -align center -command \"degenr8\";";
	layout << "button -label \"Cancel\" -align center -command \"genr8 -" << DELETE_VAR << ";deleteUI -window " << OUTPUT_WINDOW << "\";";
	eval(layout.str());

	//Show window
	show << "showWindow " << OUTPUT_WINDOW;// << window->c_str();
	eval(show.str());
	
}

//Get the values of the optionvars from the scene. These are created by the GUI when the user modifies a parameter.

void genr8::getMELValues(bool remove)
{
	string tmp;
	ostringstream startpos, reppoints, copyind;
	double x = 0, y = 0, z = 0, start = 1;
	//Scaffold
	getOptionVar(STEPS_VAR, _steps, _cmd, true, remove);
	getOptionVar(SCALE_VAR, _scale, _cmd, true, remove);
	getOptionVar(SCALE_X_VAR, _scaleX, _cmd, true, remove);
	getOptionVar(SCALE_Y_VAR, _scaleY, _cmd, true, remove);
	getOptionVar(SCALE_Z_VAR, _scaleZ, _cmd, true, remove);
	getOptionVar(UNIFORM_CELLS_VAR, _uniformCells, _cmd, true, remove);
	if(getOptionVar(SURFACE_VAR, tmp, _cmd, true, remove))	{
		if(tmp==BSPLINES_VAR)
			_surfaceRepresentation = BSPLINES;
		else if(tmp==POLYGONS_VAR)
			_surfaceRepresentation = POLYGONS;
		else
			_surfaceRepresentation = SURFACE;
	}
	getOptionVar(FIXED_PERIMETER_VAR, _fixedPerimeter, _cmd, true, remove);
	getOptionVar(FIXED_CENTER_VAR, _fixedCenter, _cmd, true, remove);
	getOptionVar(DRAW_EACH_STEP_VAR, _drawEachStep, _cmd, true, remove);
	if(getOptionVar(START_LENGTH_VAR, start, _cmd, true, remove))
		_startLengths.push_back(start);
	getOptionVar(START_POSITION_X_VAR, x, remove);
	getOptionVar(START_POSITION_Y_VAR, y, remove);
	getOptionVar(START_POSITION_Z_VAR, z, remove);
	_startPosition.setVector(x, y, z);
	startpos << " -" << START_POSITION_VAR << " " << x << " " << y << " " << z;
	_cmd += startpos.str();
	getOptionVar(BRANCH_ANGLE_VAR, _branchAngle, _cmd, true, remove);
	getOptionVar(BRANCH_LENGTH_VAR, _branchLength, _cmd, true, remove);
	//Environment
	if(getOptionVar(GRAVITY_DIRECTION_VAR, tmp, _cmd, true, remove))	{
		if(tmp=="x")
			_gravityDirection=Z;
		else if(tmp=="y")
			_gravityDirection=Y;
		getOptionVar(GRAVITY_MAGNITUDE_VAR, _gravityMagnitude, _cmd, true, remove);
	}
	if(getOptionVar(WALL_BEHAVIOUR_VAR, tmp, _cmd, true, remove))	{
		if(tmp==CUT_OFF_VAR)
			_wallBehaviour = CUT_OFF;
		else if(tmp==ACTIVE_WALLS_VAR)
			_wallBehaviour = ACTIVE_WALLS;
		else
			_wallBehaviour = DEFAULT_WB;
	}
	getOptionVar(RANDOM_NOISE_VAR, _randomNoise, _cmd, true, remove);
	getOptionVar(RANDOM_REWRITE_VAR, _randomRewrite, _cmd, true, remove);
	getOptionVar(FULL_RANDOM_REWRITE_VAR, _fullRandomRewrite, _cmd, true, remove);
	getOptionVar(WALL_CONSTANT_VAR, _wallConstant, _cmd, true, remove);
	getOptionVar(WALL_EXPONENT_VAR, _wallExponent, _cmd, true, remove);
	if(getOptionVar(REPELLING_POINTS_CONSTANT_VAR, _repellingPointsConstant, _cmd, true, remove))	{
		getOptionVar(REPELLING_POINTS_EXPONENT_VAR, _repellingPointsExponent, _cmd, true, remove);
		reppoints << " -" << REPELLING_POINTS_VAR << " " << _repellingPointsConstant << " " << _repellingPointsExponent;
	}
	getOptionVar(VIEW_INDIVIDUAL_VAR, _viewIndividual, _cmd, true, remove);
	getOptionVar(RANDOM_SEED_VAR, _randomSeed, _cmd, true, remove);
	//Grammar
	if(getOptionVar(FRACTAL_VAR, tmp, remove))	{
		if(tmp==KOCH_CURVE)	{
			_fractal = KOCH;
			_cmd += " -";
			_cmd += KOCH_CURVE;
		}
		else if(tmp==QUADRATIC_KOCH)	{
			_fractal = Q_KOCH;
			_cmd += " -";
			_cmd += QUADRATIC_KOCH;
		}
		else
			getOptionVar(TILES_VAR, _tiles, _cmd, true, remove);
	}
	else
		getOptionVar(TILES_VAR, _tiles, _cmd, true, remove);

	if(_tiles==0)	{
		if(getOptionVar(BNF_TYPE_VAR, tmp, remove))	{
			if(tmp==REVERSIBLE_BNF_VAR)	{
				_bnfType = REVERSIBLE;
				_cmd += " -";
				_cmd += REVERSIBLE_BNF_VAR;
			}
			else if(tmp==SYMMETRIC_BNF_VAR)	{
				_bnfType = SYMMETRIC;
				_cmd += " -";
				_cmd += SYMMETRIC_BNF_VAR;
			}
			else if(tmp==PROBABILISTIC_BNF_VAR)	{
				_bnfType = PROBABILISTIC;
				_cmd += " -";
				_cmd += PROBABILISTIC_BNF_VAR;
			}
		}
		getOptionVar(MAX_BNF_DEPTH_VAR, _maxBNFDepth, _cmd, true, remove);
		getOptionVar(MAX_EDGE_TYPES_VAR, _maxEdgeTypes, _cmd, true, remove);
		getOptionVar(LOAD_GRAMMAR_VAR, _loadGrammarFile, _cmd, false, true);
		getOptionVar(SAVE_GRAMMAR_VAR, _saveGrammarFile, _cmd, true, remove);
	}
	//Genetic engine
	if(getOptionVar(POPULATION_SIZE_VAR, _populationSize, _cmd, true, remove))
		//Set the fitness vector
		initFitnessVector();
	getOptionVar(MUTATION_RATE_VAR, _mutationRate, _cmd, true, remove);
	getOptionVar(CROSSOVER_POINTS_VAR, _crossoverPoints, _cmd, true, remove);
	getOptionVar(TOURNAMENT_SIZE_VAR, _tournamentSize, _cmd, true, remove);
	getOptionVar(ELITES_VAR, _elites, _cmd, true, remove);
	getOptionVar(GENERATIONS_VAR, _generations, _cmd, true, remove);
	getOptionVar(COPY_INDIVIDUAL_VAR, _copyIndivdual, remove);
	getOptionVar(NUMBER_OF_COPIES_VAR, _numberOfCopies, remove);
	getOptionVar(EXACT_COPIES_VAR, _exactCopies, remove);
	if(!_exactCopies)
		copyind << " -" << SEED_POPULATION_VAR << " " << _copyIndivdual << " " << _numberOfCopies;
	else
		copyind << " -" << COPY_INDIVIDUAL_VAR << " " << _copyIndivdual << " " << _numberOfCopies;
	_cmd += copyind.str();
	getOptionVar(CONTINUE_VAR, _continue, remove);
	getOptionVar(GENOME_LENGTH_VAR, _genomeLength, _cmd, true, remove);
	getOptionVar(LOAD_GENOME_VAR, _loadGenomeFile, _cmd, true, remove);
	getOptionVar(SAVE_GENOME_VAR, _saveGenomeFile, _cmd, true, remove);
	//Fitness
	getOptionVar(SIZE_X_VAR, _sizeX, _cmd, true, remove);
	getOptionVar(SIZE_Y_VAR, _sizeY, _cmd, true, remove);
	getOptionVar(SIZE_WEIGHT_VAR, _sizeWeight, _cmd, true, remove);
	getOptionVar(SMOOTHNESS_VAR, _smoothness, _cmd, true, remove);
	getOptionVar(SMOOTHNESS_WEIGHT_VAR, _smoothnessWeight, _cmd, true, remove);
	getOptionVar(SOFT_BOUNDARY_WEIGHT_VAR, _softBoundaryWeight, _cmd, true, remove);
	getOptionVar(SUBDIVISIONS_VAR, _subdivisions, _cmd, true, remove);
	getOptionVar(SUBDIVISIONS_WEIGHT_VAR, _subdivisionsWeight, _cmd, true, remove);
	getOptionVar(SYMMETRY_VAR, _symmetry, _cmd, true, remove);
	getOptionVar(SYMMETRY_WEIGHT_VAR, _symmetryWeight, _cmd, true, remove);
	getOptionVar(UNDULATION_VAR, _undulation, _cmd, true, remove);
	getOptionVar(UNDULATION_WEIGHT_VAR, _undulationWeight, _cmd, true, remove);
	//cout << "Command is: " << endl << _cmd << endl;
}


///////////////////////////////////////////////////////////////
//
//	modifyDialog
//
///////////////////////////////////////////////////////////////

//This command is used to modify and update the dialog for genr8
//It is basically a utility command


class modifyDialog : public MPxCommand
{
public:
				modifyDialog();
	virtual		~modifyDialog();

	MStatus		doIt( const MArgList& );
	bool		isUndoable() const	{return false;};

	static		void* creator();

private:
	MSyntax		setSyntax();
	bool		saveFile(bool txt, bool genome, bool scene, string grammar = "");
	string		loadFile(bool txt);
	void		saveString(string fileName, string grammar);
	void		saveOptionVars(ofstream &file);
};

modifyDialog::modifyDialog()
{
	
}

modifyDialog::~modifyDialog()
{
}

void* modifyDialog::creator()
{
	return new modifyDialog();
}

MStatus modifyDialog::doIt( const MArgList& args )
{
	MStatus *status = new MStatus(MS::kSuccess);
	MSyntax	syntax = setSyntax();
	MString Mstr;
	string tmp, fileName, grammar;
	bool predefined, activeWalls;
	unsigned tiles;
	MString mstr;
	ostringstream stream;
	
	MArgParser argParser(syntax, args, status);
	//Parse arguments
	//If someone selects or deselects the active walls
	if(argParser.isFlagSet(ACTIVE_WALLS_VAR, status))	{
		argParser.getFlagArgument(ACTIVE_WALLS_VAR, 0, activeWalls);
		stream << "intFieldGrp -edit -enable " << boolalpha << activeWalls << " " << WALL_EXPONENT_FIELD << ";";
		stream << "floatFieldGrp -edit -enable " << boolalpha << activeWalls << " " << WALL_CONSTANT_FIELD << ";";
		if(activeWalls)
			stream << "optionVar -sv " << WALL_BEHAVIOUR_VAR << " " << ACTIVE_WALLS_VAR << ";";
	}
	//If the user changes BNF
	if(argParser.isFlagSet(BNF_TYPE_VAR, status))	{
		argParser.getFlagArgument(BNF_TYPE_VAR, 0, mstr);
		stream << "optionVar -sv " << BNF_TYPE_VAR << " " << mstr.asChar() << " ;";
		if(mstr.asChar()==PROBABILISTIC_BNF_VAR)
			stream << "checkBoxGrp -edit -enable true " << FULL_RANDOM_REWRITE_BOX << ";";
		else
			stream << "checkBoxGrp -edit -enable false " << FULL_RANDOM_REWRITE_BOX << ";";
		stream << "optionVar -remove " << FRACTAL_VAR << ";";
		stream << "optionVar -iv " << TILES_VAR << " 0;";
		stream << "intFieldGrp -edit -enable true " << MAX_BNF_DEPTH_FIELD << ";";
		stream << "intFieldGrp -edit -enable true " << MAX_EDGE_TYPES_FIELD << ";";
	}
	//Draw as Bsplines
	if(argParser.isFlagSet(BSPLINES_VAR, status))
		stream << "optionVar -sv " << SURFACE_VAR << " " << BSPLINES_VAR << ";";
	//The scale is changed
	if(argParser.isFlagSet(CHANGE_SCALE_VAR, status))	{
		double scale;
		argParser.getFlagArgument(CHANGE_SCALE_VAR, 0, scale);
		stream << "optionVar -fv " << SCALE_VAR << " " << scale << ";";
		stream << "floatFieldGrp -edit -value1 " << scale << " " << SCALE_X_FIELD << ";";
		stream << "floatFieldGrp -edit -value1 " << scale << " " << SCALE_Y_FIELD << ";";
		stream << "floatFieldGrp -edit -value1 " << scale << " " << SCALE_Z_FIELD << ";";
	}
	//Draw a fractal
	if(argParser.isFlagSet(FRACTAL_VAR, status))	{
		argParser.getFlagArgument(FRACTAL_VAR, 0, mstr);
		stream << "optionVar -sv " << FRACTAL_VAR << " " << mstr.asChar() << " ;";
		stream << "intFieldGrp -edit -enable false " << MAX_BNF_DEPTH_FIELD << ";";
		stream << "intFieldGrp -edit -enable false " << MAX_EDGE_TYPES_FIELD << ";";
		stream << "checkBoxGrp -edit -enable false " << FULL_RANDOM_REWRITE_BOX << ";";
	}
	//Load genome
	if(argParser.isFlagSet(LOAD_GENOME_VAR, status))	{
		stream << "optionVar -sv " << LOAD_GENOME_VAR << " " << (loadFile(true)).c_str() << ";";
	}
	//Load a grammar file
	if(argParser.isFlagSet(LOAD_GRAMMAR_VAR, status))	{
		stream << "optionVar -sv " << LOAD_GRAMMAR_VAR << " " << (loadFile(true)).c_str() << ";";
		stream << "checkBox -edit -enable true " << FULL_RANDOM_REWRITE_BOX << ";";
	}
	//Load a settings file
	if(argParser.isFlagSet(LOAD_SETTINGS_VAR, status))	{
		CFileDialog fileDialog(true, "txt");
		if (fileDialog.DoModal() == IDOK) {
			fileName.assign( fileDialog.GetPathName().GetBuffer(0) );
		}
		ifstream file;
		if(opengenr8File(fileName, file, true, false))
			loadOptionVar(file);
	}
	//Draw as polygons
	if(argParser.isFlagSet(POLYGONS_VAR, status))
		stream << "optionVar -sv " << SURFACE_VAR << " " << POLYGONS_VAR << ";";
	//Save the grammar
	if(argParser.isFlagSet(SAVE_GRAMMAR_VAR, status))	{
		argParser.getFlagArgument(SAVE_GRAMMAR_VAR, 0, mstr);
		saveFile( true, false, false, mstr.asChar() );
	}
	//Toggle the collapsability of genetic engine and fitness
	if(argParser.isFlagSet(PREDEFINED_VAR, status))	{
		argParser.getFlagArgument(PREDEFINED_VAR, 0, predefined);
		stream << "frameLayout -edit -enable " << boolalpha << !predefined << " " << GENETIC_ENGINE_LAYOUT << ";";
		stream << "frameLayout -edit -enable " << boolalpha << !predefined << " " << FITNESS_LAYOUT << " ;";
	}
	if(argParser.isFlagSet(SAVE_GENOME_VAR, status))	{
		argParser.getFlagArgument(SAVE_GENOME_VAR, 0, mstr);
		saveFile( false, true, false, mstr.asChar() );
	}
	if(argParser.isFlagSet(SAVE_SETTINGS_VAR, status))	{
		CFileDialog fileDialog(false, "txt");
		if (fileDialog.DoModal() == IDOK) {
			fileName.assign( fileDialog.GetPathName().GetBuffer(0) );
		}
		ofstream file;
		if(opengenr8File(fileName, file, true, false))
			saveOptionVars(file);
	}
	if(argParser.isFlagSet(SAVE_VISIBLE_VAR, status))	{
		saveFile( false, false, true );
	}
	if(argParser.isFlagSet(SURFACE_VAR, status))
		stream << "optionVar -sv " << SURFACE_VAR << " " << SURFACE_VAR << ";";
	if(argParser.isFlagSet(TILES_VAR, status))	{
		argParser.getFlagArgument(TILES_VAR, 0, tiles);
		stream << "optionVar -iv " << TILES_VAR << " " << tiles << " ;";
		stream << "optionVar -remove " << FRACTAL_VAR << ";";
		stream << "intFieldGrp -edit -enable false " << MAX_BNF_DEPTH_FIELD << ";";
		stream << "intFieldGrp -edit -enable false " << MAX_EDGE_TYPES_FIELD << ";";
		stream << "checkBoxGrp -edit -enable false " << FULL_RANDOM_REWRITE_BOX << ";";
	}

	eval(stream.str());
	
	MStatus stat = MS::kSuccess;

	//string mess = THANK_YOU_MODIFY_DIALOG + getStupidStatement();
	//setResult( mess.c_str() );
	//printf( THANK_YOU_MODIFY_DIALOG );
	//printf( getStupidStatement().c_str() );
	return stat;
}

MSyntax modifyDialog::setSyntax()
{
	MSyntax syntax;

	syntax.enableQuery(false);
	syntax.enableEdit(false);
	syntax.addFlag("aw", ACTIVE_WALLS_VAR, MSyntax::kBoolean);
	syntax.addFlag("b", BNF_TYPE_VAR, MSyntax::kString);
	syntax.addFlag("bs", BSPLINES_VAR);
	syntax.addFlag("cs", CHANGE_SCALE_VAR, MSyntax::kDouble);
	syntax.addFlag("f", FRACTAL_VAR, MSyntax::kString);
	syntax.addFlag("lge", LOAD_GENOME_VAR);
	syntax.addFlag("lg", LOAD_GRAMMAR_VAR);
	syntax.addFlag("ls", LOAD_SETTINGS_VAR);
	syntax.addFlag("p", PREDEFINED_VAR, MSyntax::kBoolean);
	syntax.addFlag("po", POLYGONS_VAR);
	syntax.addFlag("sb", SOFT_BOUNDARY_VAR, MSyntax::kBoolean);
	syntax.addFlag("sg", SAVE_GRAMMAR_VAR, MSyntax::kString);
	syntax.addFlag("sge", SAVE_GENOME_VAR, MSyntax::kString);
	syntax.addFlag("ss", SAVE_SETTINGS_VAR);
	syntax.addFlag("sv", SAVE_VISIBLE_VAR);
	syntax.addFlag("su", SURFACE_VAR);
	syntax.addFlag("t", TILES_VAR, MSyntax::kUnsigned);
	return syntax;
}

//This function opens a dialog so that the user can choose a file for saving
//Then it calls another function that does the actual saving.

bool modifyDialog::saveFile(bool grammar, bool genome, bool scene, string str)
{
	string fileName;
	if(grammar)	{
		CFileDialog fileDialog(false, "txt");
		if (fileDialog.DoModal() == IDOK) {
			fileName.assign( fileDialog.GetPathName().GetBuffer(0) );	//Use string instead of CString
			//cout << "Saving in file: " << fileName.c_str() << endl;
			saveString(fileName, str);
		}
	}
	else if(genome)	{
		CFileDialog fileDialog(false, "txt");
		if (fileDialog.DoModal() == IDOK) {
			fileName.assign( fileDialog.GetPathName().GetBuffer(0) );	//Use string instead of CString
			saveString(fileName, str);
		}
	}
	else if(scene)	{
		ostringstream cmd;
		CFileDialog fileDialog(false, "mb");
		if (fileDialog.DoModal() == IDOK) {
			fileName.assign( fileDialog.GetPathName().GetBuffer(0) );	//Use string instead of CString
			cmd << "select -visible;";
			cmd << "file -moveSelected " << fileName.c_str() << ";";
			cmd << "select -clear;";
			eval(cmd.str());
		}
	}
	return true;
}

//This function fires up the file-browsing dialog and then lets another function do the opening

string modifyDialog::loadFile(bool txt)
{
	string fileName = "";
	if(txt)	{
		CFileDialog fileDialog(true);
		if (fileDialog.DoModal() == IDOK) {
			fileName.assign( fileDialog.GetPathName().GetBuffer(0) );
			encodeString(fileName);
		}
	}
	return fileName;
}

//Save a string

void modifyDialog::saveString(string fileName, string genome)
{
	ofstream file;
	opengenr8File(fileName, file, fileName.find('\\')!=-1);
	//decode
	decodeString(genome);
	file << genome;
	file.close();
}

//Use this function when we are saving settings, ie the values of the optionvars that we have changed from the default

void modifyDialog::saveOptionVars(ofstream &file)
{
	saveOptionVar(STEPS_VAR, file, INTEGER);
	saveOptionVar(SCALE_VAR, file, DOUBBLOAT);
	saveOptionVar(SCALE_X_VAR, file, DOUBBLOAT);
	saveOptionVar(SCALE_Y_VAR, file, DOUBBLOAT);
	saveOptionVar(SCALE_Z_VAR, file, DOUBBLOAT);
	saveOptionVar(UNIFORM_CELLS_VAR, file, INTEGER);
	saveOptionVar(SURFACE_VAR, file, STRING);
	saveOptionVar(FIXED_PERIMETER_VAR, file, INTEGER);
	saveOptionVar(FIXED_CENTER_VAR, file, INTEGER);
	saveOptionVar(DRAW_EACH_STEP_VAR, file, INTEGER);
	saveOptionVar(START_LENGTH_VAR, file, DOUBBLOAT);
	saveOptionVar(START_POSITION_X_VAR, file, DOUBBLOAT);
	saveOptionVar(START_POSITION_Y_VAR, file, DOUBBLOAT);
	saveOptionVar(START_POSITION_Z_VAR, file, DOUBBLOAT);
	saveOptionVar(BRANCH_ANGLE_VAR, file, INTEGER);
	saveOptionVar(BRANCH_LENGTH_VAR, file, DOUBBLOAT);
	//Environment
	saveOptionVar(GRAVITY_DIRECTION_VAR, file, STRING);
	saveOptionVar(GRAVITY_MAGNITUDE_VAR, file, DOUBBLOAT);
	saveOptionVar(WALL_BEHAVIOUR_VAR, file, STRING);
	saveOptionVar(RANDOM_NOISE_VAR, file, DOUBBLOAT);
	saveOptionVar(RANDOM_REWRITE_VAR, file, INTEGER);
	saveOptionVar(WALL_CONSTANT_VAR, file, DOUBBLOAT);
	saveOptionVar(WALL_EXPONENT_VAR, file, INTEGER);
	saveOptionVar(REPELLING_POINTS_CONSTANT_VAR, file, DOUBBLOAT);
	saveOptionVar(REPELLING_POINTS_EXPONENT_VAR, file, INTEGER);
	saveOptionVar(VIEW_INDIVIDUAL_VAR, file, INTEGER);
	saveOptionVar(RANDOM_SEED_VAR, file, INTEGER);
	//Grammar
	saveOptionVar(FRACTAL_VAR, file, STRING);
	saveOptionVar(TILES_VAR, file, INTEGER);
	saveOptionVar(BNF_TYPE_VAR, file, STRING);
	saveOptionVar(MAX_BNF_DEPTH_VAR, file, INTEGER);
	saveOptionVar(MAX_EDGE_TYPES_VAR, file, INTEGER);
	saveOptionVar(FULL_RANDOM_REWRITE_VAR, file, INTEGER);
	saveOptionVar(LOAD_GRAMMAR_VAR, file, STRING);
	saveOptionVar(SAVE_GRAMMAR_VAR, file, STRING);
	//Genetic engine
	saveOptionVar(POPULATION_SIZE_VAR, file, INTEGER);
	saveOptionVar(MUTATION_RATE_VAR, file, INTEGER);
	saveOptionVar(CROSSOVER_POINTS_VAR, file, INTEGER);
	saveOptionVar(TOURNAMENT_SIZE_VAR, file, INTEGER);
	saveOptionVar(ELITES_VAR, file, INTEGER);
	saveOptionVar(GENERATIONS_VAR, file, INTEGER);
	saveOptionVar(COPY_INDIVIDUAL_VAR, file, INTEGER);
	saveOptionVar(NUMBER_OF_COPIES_VAR, file, INTEGER);
	saveOptionVar(EXACT_COPIES_VAR, file, INTEGER);
	saveOptionVar(CONTINUE_VAR, file, INTEGER);
	saveOptionVar(GENOME_LENGTH_VAR, file, INTEGER);
	saveOptionVar(LOAD_GENOME_VAR, file, STRING);
	saveOptionVar(SAVE_GENOME_VAR, file, STRING);
	//Fitness
	saveOptionVar(SIZE_X_VAR, file, DOUBBLOAT);
	saveOptionVar(SIZE_Y_VAR, file, DOUBBLOAT);
	saveOptionVar(SIZE_WEIGHT_VAR, file, DOUBBLOAT);
	saveOptionVar(SMOOTHNESS_VAR, file, DOUBBLOAT);
	saveOptionVar(SMOOTHNESS_WEIGHT_VAR, file, DOUBBLOAT);
	saveOptionVar(SOFT_BOUNDARY_WEIGHT_VAR, file, DOUBBLOAT);
	saveOptionVar(SUBDIVISIONS_VAR, file, DOUBBLOAT);
	saveOptionVar(SUBDIVISIONS_WEIGHT_VAR, file, DOUBBLOAT);
	saveOptionVar(SYMMETRY_VAR, file, DOUBBLOAT);
	saveOptionVar(SYMMETRY_WEIGHT_VAR, file, DOUBBLOAT);
	saveOptionVar(UNDULATION_VAR, file, DOUBBLOAT);
	saveOptionVar(UNDULATION_WEIGHT_VAR, file, DOUBBLOAT);
}

///////////////////////////////////////////////////////////////
//
//	degenr8
//
///////////////////////////////////////////////////////////////

//This command should be used to clean up the scene after genr8.
//It removes everything (except for the optionvars set by the GUI)

class degenr8 : public MPxCommand
{
public:
				degenr8();
	virtual		~degenr8();

	MStatus		doIt( const MArgList& );
	bool		isUndoable() const	{return false;};

	static		void* creator();

private:
	MSyntax setSyntax();
	void	removeAttractors();
	void	removeRepellors();
	void	removegenr8Curves();
	void	removegenr8Layers();
	void	removeOtherStuff();
	void	remove(MSelectionList list);
	void	removegenr8();
	void	removeMELValues();
};

degenr8::degenr8()
{
}

degenr8::~degenr8()
{
}

void* degenr8::creator()
{
	return new degenr8();
}

MStatus degenr8::doIt( const MArgList& args )
{
	MStatus *status = new MStatus(MS::kSuccess);
	MSyntax	syntax = setSyntax();
	bool flag = false;
	
	MArgParser argParser(syntax, args, status);
	//Parse arguments
	if(argParser.isFlagSet("a", status))	{
		removeAttractors();
		flag = true;
	}
	if(argParser.isFlagSet("r", status))	{
		removeRepellors();
		flag = true;
	}
	if(argParser.isFlagSet("c", status))	{
		removegenr8Curves();
		flag = true;
	}
	if(argParser.isFlagSet("l", status))	{
		removegenr8Layers();
		flag = true;
	}
	if(!flag)
		removegenr8();

	if(argParser.isFlagSet("ug", status))
		MGlobal::executeCommand("unloadPlugin GENR8");

	MStatus stat = MS::kSuccess;

	string mess = THANK_YOU_DEGENR8 + getStupidStatement();
	setResult( mess.c_str() );
	printf( THANK_YOU_DEGENR8 );
	printf( getStupidStatement().c_str() );
	return stat;
}

MSyntax degenr8::setSyntax()
{
	MSyntax syntax;

	syntax.enableQuery(false);
	syntax.enableEdit(false);
	syntax.addFlag("a", "attractors");
	syntax.addFlag("r", "repellors");
	syntax.addFlag("c", "curves");
	syntax.addFlag("l", "layers");
	syntax.addFlag("ug", "unloadgenr8");
	
	return syntax;
}

//It is probably a bad idea to do this through MEL, since things are slowed down
//It would be better to remove through the API instead.

void degenr8::remove(MSelectionList list)
{
	MStringArray names;
	for ( MItSelectionList iter( list ); !iter.isDone(); iter.next() )	{
		iter.getStrings(names);
		for(int l=0; l<names.length(); l++)	{
			//cout << names[l].asChar() << endl;
			ostringstream cmd;
			cmd << "delete " << names[0].asChar();
			MGlobal::executeCommand(cmd.str().c_str());
		}
	}
}

//Select the relevant stuff and call the destroying function

void degenr8::removeAttractors()
{
	MSelectionList list;
	MGlobal::getSelectionListByName( ATTR_NAME_STAR, list);
	remove(list);
	//Reset the counter
	removeMELValues();
}

void degenr8::removeRepellors()
{
	MSelectionList list;
	MGlobal::getSelectionListByName( REP_NAME_STAR, list);
	remove(list);
	//Reset the counter
	removeMELValues();
}

void degenr8::removegenr8Curves()
{
	MSelectionList list;
	MGlobal::getSelectionListByName("genr8Curve*", list);
	remove(list);
	MGlobal::getSelectionListByName("genr8Surface*", list);
	remove(list);
}

void degenr8::removegenr8Layers()
{
	MSelectionList list;
	MGlobal::getSelectionListByName("genr8Lay*", list);
	remove(list);
}

void degenr8::removegenr8()
{
	MSelectionList list;
	MGlobal::getSelectionListByName("genr8*", list);
	remove(list);
	removeMELValues();
}

//Removes the optionvars that are used by the attractors and repellors

void degenr8::removeMELValues()
{
	int nAttr, nRep;
	//Reset the counter
	nAttr = getIntResult("optionVar -q nAttractors");
	nRep = getIntResult("optionVar -q nRepellors");
	
	for(int i=0; i<nRep; i++)	{
		ostringstream removeMEL;
		removeMEL << "optionVar -remove " << REPELLOR_CONSTANT_VAR << i << ";";
		removeMEL << "optionVar -rm " << REPELLOR_EXPONENT_VAR << i << ";";
		removeMEL << "optionVar -rm " << REPELLOR_SINGULAR_VAR << i << ";";
		removeMEL << "optionVar -rm repellorPositionX" << i << ";";
		removeMEL << "optionVar -rm repellorPositionY" << i << ";";
		removeMEL << "optionVar -rm repellorPositionZ" << i << ";";
		eval(removeMEL.str());
	}
	//Attractors
	for(int j=0; j<nAttr; j++)	{
		ostringstream removeMEL;
		removeMEL << "optionVar -remove " << ATTRACTOR_CONSTANT_VAR << j << ";";
		removeMEL << "optionVar -rm " << ATTRACTOR_EXPONENT_VAR << j << ";";
		removeMEL << "optionVar -rm " << ATTRACTOR_SINGULAR_VAR << j << ";";
		removeMEL << "optionVar -rm attractorPositionX" << j << ";";
		removeMEL << "optionVar -rm attractorPositionY" << j << ";";
		removeMEL << "optionVar -rm attractorPositionZ" << j << ";";
		eval(removeMEL.str());
	}
	eval("optionVar -rm nAttractors; optionVar -rm nRepellors");
}

//////////////////////////////////////////////////////////////////////////
//
//	regenr8
//
//////////////////////////////////////////////////////////////////////////

//This command is used to toggle the visibility of the layers that were created by genr8

class regenr8 : public MPxCommand
{
public:
				regenr8();
	virtual		~regenr8();

	MStatus		doIt( const MArgList& );
	bool		isUndoable() const	{return false;};

	static		void* creator();

private:
	MSyntax		setSyntax();
	void		showInd(int ID, bool display, int step=-1);
};

regenr8::regenr8()
{
}

regenr8::~regenr8()
{
}

void* regenr8::creator()
{
	return new regenr8();
}

MStatus regenr8::doIt( const MArgList& args )
{
	MStatus *status = new MStatus(MS::kSuccess);
	MSyntax	syntax = setSyntax();
	bool display = true;
	unsigned ID, step;
	
	MArgParser argParser(syntax, args, status);
	MArgList argList;
	//Parse arguments
	if(argParser.isFlagSet("a", status))	{
		argParser.getFlagArgument("a", 0, display);
		showInd(-1, display);
	}
	for(int j=0; j<argParser.numberOfFlagUses("s"); j++)	{
		argParser.getFlagArgumentList("s", j, argList);
		ID = argList.asInt(0);
		display = argList.asBool(1);
		showInd(ID, display);
	}
	for(int k=0; k<argParser.numberOfFlagUses("n"); k++)	{
		argParser.getFlagArgumentList("n", k, argList);
		ID = argList.asInt(0);
		step = argList.asInt(1);
		argList.get(2, display);
		showInd(ID, display, step);
	}
	MStatus stat = MS::kSuccess;

	string mess = THANK_YOU_REGENR8 + getStupidStatement();
	setResult( mess.c_str() );
	printf( THANK_YOU_REGENR8 );
	printf( getStupidStatement().c_str() );

	return stat;
}

MSyntax regenr8::setSyntax()
{
	MSyntax syntax;

	syntax.enableQuery(false);
	syntax.enableEdit(false);
	syntax.addFlag("a", "allIndividuals", MSyntax::kBoolean);
	syntax.addFlag("n", "showStep", MSyntax::kUnsigned, MSyntax::kUnsigned, MSyntax::kBoolean);
	syntax.makeFlagMultiUse("n");
	syntax.addFlag("s", "showIndividual", MSyntax::kUnsigned, MSyntax::kBoolean);
	syntax.makeFlagMultiUse("s");

	return syntax;
}

//This should probably be done through the API to improve the speed

void regenr8::showInd(int ID, bool display, int step)
{
	MSelectionList list;
	if(ID==-1)
		MGlobal::getSelectionListByName("genr8Lay*", list);
	else if(step==-1)	{
		ostringstream ind;
		ind << "genr8Lay" << ID;
		MGlobal::getSelectionListByName(ind.str().c_str(), list);
	}
	else	{
		ostringstream ind;
		ind << "genr8Lay" << ID << "Step" << step;
		MGlobal::getSelectionListByName(ind.str().c_str(), list);
	}
	MStringArray names;
	for ( MItSelectionList iter( list ); !iter.isDone(); iter.next() )	{
		iter.getStrings(names);
		for(int l=0; l<names.length(); l++)	{
			//cout << names[l].asChar() << endl;
			ostringstream cmd;
			cmd << "setAttr " << names[0].asChar() << ".visibility ";
			if(display)
				cmd << 1;
			else
				cmd << 0;
			MGlobal::executeCommand(cmd.str().c_str());
		}
	}
}

//////////////////////////////////////////////////////////////////////////
//
//	regn8
//
//////////////////////////////////////////////////////////////////////////

//This command maps a grammar (from the Reversible BNF) to a genome

class regn8 : public MPxCommand
{
public:
				regn8();
	virtual		~regn8();

	MStatus		doIt( const MArgList& );
	bool		isUndoable() const	{return false;};

	static		void* creator();

private:
	MSyntax		setSyntax();
	void		createWindow();
	Individual*	_individual;
	string*		_window;
	vector<string*>	_textFields;
	string		saveFile;
};

regn8::regn8()
{
	_window = new string;
}

regn8::~regn8()
{
}

void* regn8::creator()
{
	return new regn8();
}

MStatus regn8::doIt( const MArgList& args )
{
	MStatus *status = new MStatus(MS::kSuccess);
	MSyntax	syntax = setSyntax();
	MString line;

	MArgParser argParser(syntax, args, status);
	//Parse arguments
	//We must save the result in a file
	if(argParser.isFlagSet("s", status))	{
		argParser.getFlagArgument("s", 0, line);
		saveFile.assign(line.asChar());
	}
	//Read the grammar from a file
	if(argParser.isFlagSet("f", status))	{
		argParser.getFlagArgument("f", 0, line);
		string filename;
		filename.assign(line.asChar());
		//Create the individual
		_individual = new Individual(0, filename);
		_individual->getSentence()->printSentence();
		//Invert the mapping
		_individual->constructGenome();
		_individual->getGenome()->printGenome();
	}
	//Save the result
	if(saveFile!="")	{
		ofstream file;
		if(!opengenr8File(saveFile, file))
			return MS::kSuccess;
		file << 1 << endl;	//Popsize
		_individual->getGenome()->saveGenome(file);
	}
	
	MStatus stat = MS::kSuccess;

	// Since this class is derived off of MPxCommand, you can use the 
	// inherited methods to return values and set error messages
	//
	string mess = THANK_YOU_REGN8 + getStupidStatement();
	setResult( mess.c_str() );
	printf( THANK_YOU_REGN8 );
	printf( getStupidStatement().c_str() );

	return stat;
}

MSyntax regn8::setSyntax()
{
	MSyntax syntax;

	syntax.enableQuery(false);
	syntax.enableEdit(false);
	syntax.addFlag("f", "grammarFile", MSyntax::kString);
	syntax.addFlag("s", "saveGenome", MSyntax::kString);
	
	return syntax;
}


//////////////////////////////////////////////////////////////////////////
//
//	repellor
//
//////////////////////////////////////////////////////////////////////////

//Repellor command can be used to place repellors interactively.
//It would be desirable to be able to place the repellor with a mouse-click.
//To do this we need to use the tool-context (I think it was called).
//Unfortunately, I was not able to get it to work. Mostly because there
//was a bug in the examples that came with Maya, so I could not learn from them.

class repellor : public MPxCommand
{
public:
					repellor(); 
	virtual			~repellor(); 
	static void*	creator();

	MStatus			doIt(const MArgList& args);
	MStatus			parseArgs(const MArgList& args);
	bool			isUndoable() const	{return false;};
	static MSyntax	newSyntax();
	
private:
	//Window stuff
	void			createWindow(int repID, bool modify = false);
	string*			_window;
	bool			_gui;
	int				_nRepellors, _repID;
	void			getMELValues(int repID, bool pos = true);
	void			setMELValues(int repID);
	void			incCounter();
	//Repellor stuff
	Vector			_pos;
	double			_const;
	int				_exp;
	bool			_singular;
	Force*			_force;
	MDagPath		path;			// The dag path to the curve.
									// Don't save the pointer!
};


void* repellor::creator()
{
	return new repellor;
}

repellor::~repellor() {}

repellor::repellor()
{
	_const = 10;
	_exp = 2;
	_repID = -1;
	_singular = false;

	//Have a MEL-variable that counts the number of repellors
	_nRepellors = getIntResult("optionVar -q \"nRepellors\"");
}
	
MSyntax repellor::newSyntax()
{
	MSyntax syntax;

	syntax.enableQuery(false);
	syntax.enableEdit(false);
	syntax.addFlag("p", "position", MSyntax::kDouble, MSyntax::kDouble, MSyntax::kDouble);
	syntax.addFlag("c", REPELLOR_CONSTANT_VAR, MSyntax::kDouble);
	syntax.addFlag("e", REPELLOR_EXPONENT_VAR, MSyntax::kUnsigned);
	syntax.addFlag("g", GUI_VAR);
	//Secret flag, not to be documented in the help. Used by the gui.
	syntax.addFlag("s", "secret", MSyntax::kUnsigned);

	return syntax;
}

MStatus repellor::doIt(const MArgList &args)
{
	MStatus status;
	status = parseArgs(args);

	if (MS::kSuccess != status)
		return status;

	MSelectionList list;
	MGlobal::getActiveSelectionList(list);
	if(list.length()==0 || _repID!=-1)	{
		//Place a new repellor
		if(_gui)	{
			//Store the data
			setMELValues(_nRepellors);
			//Fire up the GUI
			createWindow(_nRepellors);
		}
		else	{
			if(_repID!=-1)	{
				//Get the results from the GUI
				getMELValues(_repID);
				//Place the repellor
				_force = new Force(false, _pos, _exp, _const, _repID, _singular);
				_force->drawAttractors();
				incCounter();
			}
			else	{	//Commandline
				//Place the repellor
				_force = new Force(false, _pos, _exp, _const, _nRepellors, _singular);
				_force->drawAttractors();
				setMELValues(_nRepellors);
				incCounter();
			}
			delete _force;
		}
	}
	else if(list.length()==1 && _repID==-1)	{
		//Edit the one on the list
		MStringArray mName;
		string name;
		list.getSelectionStrings(0, mName);
		if(mName.length()==1)	{
			name.assign(mName[0].asChar());
			//Find out which repellor was selected
			if(name.find(REP_NAME)!=-1)	{
				_repID = atoi(name.substr(13,1).c_str());
				//Get its values
				getMELValues(_repID, false);
				createWindow(_repID, true);
			}
			else
				cout << "You can not select anything but a repellor for this command." << endl;
		}
	}
	else
		cout << "You can not select more than one repellor for this command." << endl;

	string mess = THANK_YOU_REPELLOR + getStupidStatement();
	setResult( mess.c_str() );
	printf( THANK_YOU_REPELLOR );
	printf( getStupidStatement().c_str() );
	return status;
}

MStatus repellor::parseArgs(const MArgList &args)
{
	MStatus status, *stat = new MStatus;
	MArgParser argParser(newSyntax(), args, stat);
	//Parse arguments
	if (argParser.isFlagSet("p")) {
		double tmp;
		status = argParser.getFlagArgument("p", 0, tmp);
		if (!status) {
			status.perror("position flag parsing failed");
			return status;
		}
		_pos.setX(tmp);
		status = argParser.getFlagArgument("p", 1, tmp);
		if (!status) {
			status.perror("position flag parsing failed");
			return status;
		}
		_pos.setY(tmp);
		status = argParser.getFlagArgument("p", 2, tmp);
		if (!status) {
			status.perror("position flag parsing failed");
			return status;
		}
		_pos.setZ(tmp);
	}
	
	if (argParser.isFlagSet("c")) {
		double tmp;
		status = argParser.getFlagArgument("c", 0, tmp);
		if (!status) {
			status.perror("constant flag parsing failed");
			return status;
		}
		_const = tmp;
	}
	
	if (argParser.isFlagSet("e")) {
		unsigned tmp;
		status = argParser.getFlagArgument("e", 0, tmp);
		if (!status) {
			status.perror("exponent flag parsing failed");
			return status;
		}
		_exp = tmp;
	}

	if (argParser.isFlagSet("g"))
		_gui = true;
	else
		_gui = false;
	
	if (argParser.isFlagSet("s")) {
		unsigned tmp;
		status = argParser.getFlagArgument("s", 0, tmp);
		if (!status) {
			status.perror("exponent flag parsing failed");
			return status;
		}
		_repID = tmp;
	}

	return MS::kSuccess;
}	

void repellor::createWindow(int repID, bool modify)
{
	ostringstream wind, layout, show, konst, exp, posX, posY, posZ, cancel, OK, sing;
	//Make window
	wind << "window -title repellor -s on -tb on -mnb on -mxb on -widthHeight 200 200;";
	_window = getStringPointerResult(wind.str());
	//Fix layout
	layout << "columnLayout;";
	eval(layout.str());
	//Constant
	konst << "floatFieldGrp -label \"Repellor constant\" -value1 " << _const << " -changeCommand \"optionVar -floatValue repellorConstant" << repID << " #1\" ";
	eval(konst.str());
	//Singularity
	sing << "checkBox -label \"Singularity\" -onCommand \"optionVar -intValue " << REPELLOR_SINGULAR_VAR << repID << " 1\" -offCommand \"optionVar -intValue " << REPELLOR_SINGULAR_VAR << repID << " 0\"";
	eval(sing.str());
	//Exponent
	exp << "intFieldGrp -label \"Repellor exponent\" -value1 " << _exp << " -changeCommand \"optionVar -intValue repellorExponent" << repID << " #1\" ";
	eval(exp.str());
	if(!modify)	{
		//Position
		posX << "floatFieldGrp -label \"Repellor position X\" -value1 " << _pos.getX() << " -changeCommand \"optionVar -floatValue " << REP_POS_X << repID << " #1\" ";
		eval(posX.str());
		posY << "floatFieldGrp -label \"Repellor position Y\" -value1 " << _pos.getY() << " -changeCommand \"optionVar -floatValue " << REP_POS_Y << repID << " #1\" ";
		eval(posY.str());
		posZ << "floatFieldGrp -label \"Repellor position Z\" -value1 " << _pos.getZ() << " -changeCommand \"optionVar -floatValue " << REP_POS_Z << repID << " #1\" ";
		eval(posZ.str());
	}
	//Buttons
	//OK
	if(modify)
		OK << "button -label \"OK\" -command (\"deleteUI -window " << _window->c_str() << "\")";
	else
		OK << "button -label \"OK\" -command (\"repellor -s " << repID << "; deleteUI -window " << _window->c_str() << "\")";

	eval(OK.str());
	//Cancel
	cancel << "button -label \"Cancel\" -command (\"deleteUI -window " << _window->c_str() << "\")";
	eval(cancel.str());
	//Show window
	show << "showWindow " << _window->c_str();
	eval(show.str());
}


void repellor::getMELValues(int repID, bool pos)
{
	ostringstream konst, exp, posX, posY, posZ, sing;
	//Get the constant
	konst << "optionVar -q repellorConstant" << repID;
	_const = getDoubleResult(konst.str());
	//Get the exponent
	exp << "optionVar -q repellorExponent" << repID;
	_exp = getDoubleResult(exp.str());
	//Get singularity
	sing << "optionVar -q " << REPELLOR_SINGULAR << repID;
	_singular = getIntResult(sing.str())!=0;
	if(pos)	{
		//Get the position
		//The user is allowed to move the repellor, so we must find out where it is
		posX << "optionVar -q repellorPositionX" << repID;
		_pos.setX(getDoubleResult(posX.str()));
		posY << "optionVar -q repellorPositionY" << repID;
		_pos.setY(getDoubleResult(posY.str()));
		posZ << "optionVar -q repellorPositionZ" << repID;
		_pos.setZ(getDoubleResult(posZ.str()));
	}
}

void repellor::setMELValues(int repID)
{
	ostringstream exp, konst, posX, posY, posZ, sing;
	//Store the other things as MEL-variables as well
	exp << "optionVar -iv repellorExponent" << repID << " " << _exp << ";";
	eval(exp.str());
	konst << "optionVar -fv repellorConstant" << repID << " " << _const << ";";
	eval(konst.str());
	sing << "optionVar -iv " << REPELLOR_SINGULAR << repID << " " << (int)_singular << ";";
	eval(sing.str());
	posX << "optionVar -fv repellorPositionX" << repID << " " << _pos.getX() << ";";
	eval(posX.str());
	posY << "optionVar -fv repellorPositionY" << repID << " " << _pos.getY() << ";";
	eval(posY.str());
	posZ << "optionVar -fv repellorPositionZ" << repID << " " << _pos.getZ() << ";";
	eval(posZ.str());
}

void repellor::incCounter()
{
	//Increase it
	ostringstream nRepellors;
	nRepellors << "optionVar -intValue \"nRepellors\" " << ++_nRepellors;
	eval(nRepellors.str());
}

//////////////////////////////////////////////////////////////////////////
//
//	attractor
//
//////////////////////////////////////////////////////////////////////////

//Except for the names, this is the same as repellor.

class attractor : public MPxCommand
{
public:
					attractor(); 
	virtual			~attractor(); 
	static void*	creator();

	MStatus			doIt(const MArgList& args);
	MStatus			parseArgs(const MArgList& args);
	bool			isUndoable() const	{return false;};
	static MSyntax	newSyntax();
	
private:
	//Window stuff
	void			createWindow(int repID, bool modify = false);
	string*			_window;
	bool			_gui;
	int				_nAttractors, _repID;
	void			getMELValues(int repID, bool pos = true);
	void			setMELValues(int repID);
	void			incCounter();
	//Attractor stuff
	Vector			_pos;
	double			_const;
	int				_exp;
	bool			_singular;
	Force*			_force;
	MDagPath		path;			// The dag path to the curve.
									// Don't save the pointer!
};


void* attractor::creator()
{
	return new attractor;
}

attractor::~attractor() {}

attractor::attractor()
{
	_const = 10;
	_exp = 2;
	_repID = -1;
	_singular = false;

	//Have a MEL-variable that counts the number of attractors
	_nAttractors = getIntResult("optionVar -q \"nAttractors\"");
}
	
MSyntax attractor::newSyntax()
{
	MSyntax syntax;

	syntax.enableQuery(false);
	syntax.enableEdit(false);
	syntax.addFlag("p", "position", MSyntax::kDouble, MSyntax::kDouble, MSyntax::kDouble);
	syntax.addFlag("c", "const", MSyntax::kDouble);
	syntax.addFlag("e", "exponent", MSyntax::kUnsigned);
	syntax.addFlag("g", "graphical");
	//Secret flag, not to be documented in the help. Used by the gui.
	syntax.addFlag("s", "secret", MSyntax::kUnsigned);

	return syntax;
}

MStatus attractor::doIt(const MArgList &args)
{
	MStatus status;
	status = parseArgs(args);

	if (MS::kSuccess != status)
		return status;

	MSelectionList list;
	MGlobal::getActiveSelectionList(list);
	if(list.length()==0 || _repID!=-1)	{
		//Place a new attractor
		if(_gui)	{
			//Store the data
			setMELValues(_nAttractors);
			//Fire up the GUI
			createWindow(_nAttractors);
		}
		else	{
			if(_repID!=-1)	{
				//Get the results from the GUI
				getMELValues(_repID);
				//Place the attractor
				_force = new Force(true, _pos, _exp, _const, _repID, _singular);
				_force->drawAttractors();
				incCounter();
			}
			else	{	//Commandline
				//Place the attractor
				_force = new Force(true, _pos, _exp, _const, _nAttractors, _singular);
				_force->drawAttractors();
				setMELValues(_nAttractors);
				incCounter();
			}
			delete _force;
		}
	}
	else if(list.length()==1 && _repID==-1)	{
		//Edit the one on the list
		MStringArray mName;
		string name;
		list.getSelectionStrings(0, mName);
		if(mName.length()==1)	{
			name.assign(mName[0].asChar());
			//Find out which attractor was selected
			if(name.find(ATTR_NAME, 0)!=-1)	{
				_repID = atoi(name.substr(13,1).c_str());
				//Get its values
				getMELValues(_repID, false);
				createWindow(_repID, true);
			}
		}
	}
	else
		cout << "You can not select more than one attractor for this command." << endl;

	string mess = THANK_YOU_ATTRACTOR + getStupidStatement();
	setResult( mess.c_str() );
	printf( THANK_YOU_ATTRACTOR );
	printf( getStupidStatement().c_str() );
	return status;
}

MStatus attractor::parseArgs(const MArgList &args)
{
	MStatus status, *stat = new MStatus;
	MArgParser argParser(newSyntax(), args, stat);
	//Parse arguments
	if (argParser.isFlagSet("p")) {
		double tmp;
		status = argParser.getFlagArgument("p", 0, tmp);
		if (!status) {
			status.perror("position flag parsing failed");
			return status;
		}
		_pos.setX(tmp);
		status = argParser.getFlagArgument("p", 1, tmp);
		if (!status) {
			status.perror("position flag parsing failed");
			return status;
		}
		_pos.setY(tmp);
		status = argParser.getFlagArgument("p", 2, tmp);
		if (!status) {
			status.perror("position flag parsing failed");
			return status;
		}
		_pos.setZ(tmp);
	}
	
	if (argParser.isFlagSet("c")) {
		double tmp;
		status = argParser.getFlagArgument("c", 0, tmp);
		if (!status) {
			status.perror("constant flag parsing failed");
			return status;
		}
		_const = tmp;
	}
	
	if (argParser.isFlagSet("e")) {
		unsigned tmp;
		status = argParser.getFlagArgument("e", 0, tmp);
		if (!status) {
			status.perror("exponent flag parsing failed");
			return status;
		}
		_exp = tmp;
	}

	if (argParser.isFlagSet("g"))
		_gui = true;
	else
		_gui = false;
	
	if (argParser.isFlagSet("s")) {
		unsigned tmp;
		status = argParser.getFlagArgument("s", 0, tmp);
		if (!status) {
			status.perror("exponent flag parsing failed");
			return status;
		}
		_repID = tmp;
	}

	return MS::kSuccess;
}	

void attractor::createWindow(int repID, bool modify)
{
	ostringstream wind, layout, show, konst, exp, posX, posY, posZ, cancel, OK, sing;
	//Make window
	wind << "window -title attractor -s on -tb on -mnb on -mxb on -widthHeight 200 200;";
	_window = getStringPointerResult(wind.str());
	//Fix layout
	layout << "columnLayout;";
	eval(layout.str());
	//Constant
	konst << "floatFieldGrp -label \"Attractor constant\" -value1 " << _const << " -changeCommand \"optionVar -floatValue attractorConstant" << repID << " #1\" ";
	eval(konst.str());
	//Exponent
	exp << "intFieldGrp -label \"Attractor exponent\" -value1 " << _exp << " -changeCommand \"optionVar -intValue attractorExponent" << repID << " #1\" ";
	eval(exp.str());
	//Singularity
	sing << "checkBox -label \"Singularity\" -onCommand \"optionVar -intValue " << ATTRACTOR_SINGULAR << repID << " 1\" -offCommand \"optionVar -intValue " << ATTRACTOR_SINGULAR << repID << " 0\"";
	eval(sing.str());
	if(!modify)	{
		//Position
		posX << "floatFieldGrp -label \"Attractor position X\" -value1 " << _pos.getX() << " -changeCommand \"optionVar -floatValue " << ATTR_POS_X << repID << " #1\" ";
		eval(posX.str());
		posY << "floatFieldGrp -label \"Attractor position Y\" -value1 " << _pos.getY() << " -changeCommand \"optionVar -floatValue " << ATTR_POS_Y << repID << " #1\" ";
		eval(posY.str());
		posZ << "floatFieldGrp -label \"Attractor position Z\" -value1 " << _pos.getZ() << " -changeCommand \"optionVar -floatValue " << ATTR_POS_Z << repID << " #1\" ";
		eval(posZ.str());
	}
	//Buttons
	//OK
	if(modify)
		OK << "button -label \"OK\" -command (\"deleteUI -window " << _window->c_str() << "\")";
	else
		OK << "button -label \"OK\" -command (\"attractor -s " << repID << "; deleteUI -window " << _window->c_str() << "\")";

	eval(OK.str());
	//Cancel
	cancel << "button -label \"Cancel\" -command (\"deleteUI -window " << _window->c_str() << "\")";
	eval(cancel.str());
	//Show window
	show << "showWindow " << _window->c_str();
	eval(show.str());
}


void attractor::getMELValues(int repID, bool pos)
{
	ostringstream konst, exp, posX, posY, posZ, sing;
	//Get the constant
	konst << "optionVar -q attractorConstant" << repID;
	_const = getDoubleResult(konst.str());
	//Get the exponent
	exp << "optionVar -q attractorExponent" << repID;
	_exp = getDoubleResult(exp.str());
	//Singularity
	sing << "optionVar -q " << ATTRACTOR_SINGULAR << repID;
	_singular = getIntResult(sing.str())!=0;
	if(pos)	{
		//Get the position
		//The user is allowed to move the attractor, so we must find out where it is
		posX << "optionVar -q attractorPositionX" << repID;
		_pos.setX(getDoubleResult(posX.str()));
		posY << "optionVar -q attractorPositionY" << repID;
		_pos.setY(getDoubleResult(posY.str()));
		posZ << "optionVar -q attractorPositionZ" << repID;
		_pos.setZ(getDoubleResult(posZ.str()));
	}
}

void attractor::setMELValues(int repID)
{
	ostringstream exp, konst, posX, posY, posZ, sing;
	//Store the other things as MEL-variables as well
	exp << "optionVar -iv attractorExponent" << repID << " " << _exp << ";";
	eval(exp.str());
	konst << "optionVar -fv attractorConstant" << repID << " " << _const << ";";
	eval(konst.str());
	sing << "optionVar -iv " << ATTRACTOR_SINGULAR << repID << " " << (int)_singular << ";";
	eval(konst.str());
	posX << "optionVar -fv attractorPositionX" << repID << " " << _pos.getX() << ";";
	eval(posX.str());
	posY << "optionVar -fv attractorPositionY" << repID << " " << _pos.getY() << ";";
	eval(posY.str());
	posZ << "optionVar -fv attractorPositionZ" << repID << " " << _pos.getZ() << ";";
	eval(posZ.str());
}

void attractor::incCounter()
{
	//Increase it
	ostringstream nAttractors;
	nAttractors << "optionVar -intValue \"nAttractors\" " << ++_nAttractors;
	eval(nAttractors.str());
}

/*
//This is the tool stuff that I can not get to work...
class repellorTool : public MPxToolCommand
{
public:
					repellorTool(); 
	virtual			~repellorTool(); 
	static void*	creator();

	MStatus			doIt(const MArgList& args);
	MStatus			parseArgs(const MArgList& args);
	//MStatus			redoIt();
	//MStatus			undoIt();
	bool			isUndoable() const;
	MStatus			finalize();
	static MSyntax	newSyntax();
	
	
private:
	Vector			_pos;
	double			_const;
	int				_exp;
	MDagPath		path;			// The dag path to the curve.
									// Don't save the pointer!
};


void* repellorTool::creator()
{
	cout << "Repellortool creator used." << endl;
	return new repellorTool;
}

repellorTool::~repellorTool() {}

repellorTool::repellorTool()
{
	cout << "Repellortool created." << endl;
	_const = 10;
	_exp = 2;
	setCommandString("repellorToolCmd");
}
	
MSyntax repellorTool::newSyntax()
{
	cout << "Repellortool syntax created." << endl;
	MSyntax syntax;

	syntax.enableQuery(false);
	syntax.enableEdit(false);
	syntax.addFlag("p", "position", MSyntax::kDouble, MSyntax::kDouble, MSyntax::kDouble);
	syntax.addFlag("c", "const", MSyntax::kDouble);
	syntax.addFlag("e", "exponent", MSyntax::kUnsigned);

	return syntax;
}

MStatus repellorTool::doIt(const MArgList &args)
//
// Description
//     Sets up the repellor parameters from arguments passed to the
//     MEL command.
//
{
	MStatus status;

	status = parseArgs(args);

	if (MS::kSuccess != status)
		return status;

	//return redoIt();
	Force* force = new Force(false, _pos, _exp, _const);
	delete force;
	return status;
}

MStatus repellorTool::parseArgs(const MArgList &args)
{
	MStatus status;
	MArgDatabase argData(syntax(), args);
	cout << "Parsing args" << endl;
	if (argData.isFlagSet("p")) {
		double tmp;
		status = argData.getFlagArgument("p", 0, tmp);
		if (!status) {
			status.perror("position flag parsing failed");
			return status;
		}
		_pos.setX(tmp);
		status = argData.getFlagArgument("p", 1, tmp);
		if (!status) {
			status.perror("position flag parsing failed");
			return status;
		}
		_pos.setY(tmp);
		status = argData.getFlagArgument("p", 2, tmp);
		if (!status) {
			status.perror("position flag parsing failed");
			return status;
		}
		_pos.setZ(tmp);
	}
	
	if (argData.isFlagSet("c")) {
		double tmp;
		status = argData.getFlagArgument("c", 0, tmp);
		if (!status) {
			status.perror("constant flag parsing failed");
			return status;
		}
		_const = tmp;
	}
	
	if (argData.isFlagSet("e")) {
		unsigned tmp;
		status = argData.getFlagArgument("e", 0, tmp);
		if (!status) {
			status.perror("numCVs flag parsing failed");
			return status;
		}
		_exp = tmp;
	}
	
	return MS::kSuccess;
}	

/*
MStatus repellorTool::redoIt()
//
// Description
//     This method creates a repellor
//
{
	MStatus stat;

	return stat;
}

MStatus repellorTool::undoIt()
//
// Description
//     Removes the helix curve from the model.
//
{
	MStatus stat; 
	MObject transform = path.transform();
	stat = MGlobal::removeFromModel( transform );
	return stat;
}*

bool repellorTool::isUndoable() const
//
// Description
//     Set this command to be undoable.
//
{
	return false;	
}

MStatus repellorTool::finalize()
//
// Description
//     Command is finished, construct a string for the command
//     for journalling.
//
{
	MArgList command;
	/*command.addArg(commandString());
	command.addArg(MString(kRadiusFlag));
	command.addArg(radius);
	command.addArg(MString(kPitchFlag));
	command.addArg(pitch);
	command.addArg(MString(kNumberCVsFlag));
	command.addArg((int) numCV);
	command.addArg(MString(kUpsideDownFlag));
	command.addArg(upDown);
	return MPxToolCommand::doFinalize( command );*
	return MS::kSuccess;
}*/

class repellorContext : public MPxContext
{
public:
					repellorContext();
	virtual void	toolOnSetup(MEvent &event);
	virtual MStatus doPress(MEvent &event);
	
	virtual	void	getClassName(MString & name) const;

private:
	M3dView			view;
	short			x_pos, y_pos;
};

repellorContext::repellorContext() 
{
	setTitleString("Repellor Tool");
	cout << "Repellorcontext created." << endl;
	setCursor( MCursor::defaultCursor );

	// Tell the context which XPM to use so the tool can properly
	// be a candidate for the 6th position on the mini-bar.
	setImage("helixTool.xpm", MPxContext::kImage1 );
}

void repellorContext::toolOnSetup(MEvent &)
{
	//setHelpString(REP_HELP);
}

MStatus repellorContext::doPress(MEvent &event)
{
	MPoint point;
	MVector vec;
    event.getPosition(x_pos, y_pos);
    view = M3dView::active3dView();
	view.viewToWorld(x_pos, y_pos, point, vec);
	cout << "The mouse was pressed in " << x_pos << ", " << y_pos << " and it was mapped to " << point[0] << ", " << point[1] << ", " << point[2] << endl;
    return MS::kSuccess;
}

void repellorContext::getClassName( MString & name ) const
{
	name.set("repellor");
}

class repellorContextCmd : public MPxContextCommand
{
public:	
						repellorContextCmd();
	virtual	MStatus		doEditFlags();
	virtual MPxContext* makeObj();
	static void*		creator();

protected:
    repellorContext*		_repellorContext;

};

repellorContextCmd::repellorContextCmd() {}

MPxContext* repellorContextCmd::makeObj()
//
// Description
//    When the context command is executed in maya, this method
//    be used to create a context.
//
{
	_repellorContext = new repellorContext();
	return _repellorContext;
}

void* repellorContextCmd::creator()
//
// Description
//    This method creates the context command.
//
{
	return new repellorContext;
}


MStatus repellorContextCmd::doEditFlags()
{
	MStatus status = MS::kSuccess;
	/*
	MArgParser argData = parser();
	
	if (argData.isFlagSet(kNumberCVsFlag)) {
		unsigned numCVs;
		status = argData.getFlagArgument(kNumberCVsFlag, 0, numCVs);
		if (!status) {
			status.perror("numCVs flag parsing failed.");
			return status;
		}
		fHelixContext->setNumCVs(numCVs);
	}

	if (argData.isFlagSet(kUpsideDownFlag)) {
		bool upsideDown;
		status = argData.getFlagArgument(kUpsideDownFlag, 0, upsideDown);
		if (!status) {
			status.perror("upsideDown flag parsing failed.");
			return status;
		}
		fHelixContext->setUpsideDown(upsideDown);
	}
	*/
	return MS::kSuccess;
}


//////////////////////////////////////////////////////////////////////////
//
//	Register the commands
//
//////////////////////////////////////////////////////////////////////////

MStatus initializePlugin( MObject obj )
{ 
	MStatus   status;
	MFnPlugin plugin( obj, "MIT - Emergent Design Group", "3.0", "Any");

	status = plugin.registerCommand( "genr8", genr8::creator );
	if (!status) {
		status.perror("registerCommand");
		return status;
	}
	status = plugin.registerCommand( "degenr8", degenr8::creator );
	if (!status) {
		status.perror("registerCommand");
		return status;
	}
	status = plugin.registerCommand( "regenr8", regenr8::creator );
	if (!status) {
		status.perror("registerCommand");
		return status;
	}
	status = plugin.registerCommand( "regn8", regn8::creator );
	if (!status) {
		status.perror("registerCommand");
		return status;
	}
	status = plugin.registerCommand( "repellor", repellor::creator );
	if (!status) {
		status.perror("registerCommand");
		return status;
	}
	status = plugin.registerCommand( "attractor", attractor::creator );
	if (!status) {
		status.perror("registerCommand");
		return status;
	}
	status = plugin.registerCommand( "modifyDialog", modifyDialog::creator );
	if (!status) {
		status.perror("registerCommand");
		return status;
	}
	//status = plugin.registerContextCommand("repellorContext", repellorContextCmd::creator);//, "repellorContextCmd", repellorTool::creator, repellorTool::newSyntax);
	if (!status) {
		status.perror("registerContextCommand");
		return status;
	}

	return status;
}

MStatus uninitializePlugin( MObject obj )
{
	MStatus   status;
	MFnPlugin plugin( obj );

	status = plugin.deregisterCommand( "genr8" );
	if (!status) {
		status.perror("deregisterCommand");
		return status;
	}
	status = plugin.deregisterCommand( "degenr8" );
	if (!status) {
		status.perror("deregisterCommand");
		return status;
	}
	status = plugin.deregisterCommand( "regenr8" );
	if (!status) {
		status.perror("deregisterCommand");
		return status;
	}
	status = plugin.deregisterCommand( "regn8" );
	if (!status) {
		status.perror("deregisterCommand");
		return status;
	}
	status = plugin.deregisterCommand( "repellor" );
	if (!status) {
		status.perror("deregisterCommand");
		return status;
	}
	status = plugin.deregisterCommand( "attractor" );
	if (!status) {
		status.perror("deregisterCommand");
		return status;
	}
	status = plugin.deregisterCommand( "modifyDialog" );
	if (!status) {
		status.perror("deregisterCommand");
		return status;
	}
	//status = plugin.deregisterContextCommand( "repellorContext");//, "repellorContextCmd" );
	if (!status) {
		status.perror("deregisterContextCommand");
		return status;
	}

	return status;
}
