//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Force.cpp: implementation of the Force class.
//
//////////////////////////////////////////////////////////////////////

#include "Force.h"
#include <iostream>
#include <sstream>
#include "MELUtilities.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Force::Force()
{

}

//Use this constructor for the repellor command
Force::Force(bool attr, Vector pos, double c, int e, int ID, bool singularity)
{
	_nAttractor = 0;
	if(attr)
		createAttractorLayer();
	else
		createRepellorLayer();

	newAttractor(attr, pos, e, c, ID, singularity);
}

Force::Force(vector<Vector> attrPos, vector<Vector> repPos, double force, Direction dir, double ac, double rc, int ae, int re, bool attrSing, bool repSing)
{
	//Create the attractors and repellers
	_nAttractor = 0;

	for(int i=0; i< attrPos.size(); i++)
		newAttractor(true, attrPos[i], ae, ac, attrSing);
	for(int j=0; j< repPos.size(); j++)
		newAttractor(false, repPos[j], re, rc, repSing);

	if(attrPos.size()>0)
		createAttractorLayer();

	if(repPos.size()>0)
		createRepellorLayer();
	//Create the gravity
	if(force!=0)
		_gravity = new Gravity(dir, 0, force);
	else
		_gravity = NULL;
}

Force::~Force()
{

}

//Use these commands when constructing new attractors. They automatically keep track of the IDs

void Force::newAttractor(bool attractor, Vector pos, int exp, int konst, bool drawn, bool singularity)
{
	Attractor *attr = new Attractor(pos, _nAttractor++, attractor, exp, konst, drawn, singularity);
	_attractors.push_back(attr);
}

void Force::newAttractor(bool attractor, Vector pos, int exp, int konst, int ID, bool singularity)
{
	//To be used by attractor/repellor-commands
	Attractor *attr = new Attractor(pos, ID, attractor, exp, konst, false, singularity);
	_nAttractor++;
	_attractors.push_back(attr);
}

void Force::drawAttractors()
{
	for(int i=0; i<_nAttractor; i++)
		_attractors[i]->drawAttractor(_attractorLayer, _repellorLayer);
}

void Force::createAttractorLayer()
{
	//Check if layer already exists, for instance from another use of the repellor command
	if(getIntResult("catch(`getAttr genr8attrLayer.id`)")==1)	{
		ostringstream newName;
		newName << "createDisplayLayer -empty -mc -n genr8attrLayer";
		eval(newName.str());
		_attractorLayer = getStringResults("editDisplayLayerGlobals -q -cdl");
	}
	else
		eval("editDisplayLayerGlobals -cdl genr8attrLayer");
}

void Force::createRepellorLayer()
{
	//Check if layer already exists, for instance from another use of the repellor command
	if(getIntResult("catch(`getAttr genr8repellorLayer.id`)")==1)	{
		ostringstream newName;
		newName << "createDisplayLayer -empty -mc -n genr8repellorLayer";
		eval(newName.str());
		_repellorLayer = getStringResults("editDisplayLayerGlobals -q -cdl");
	}
	else
		eval("editDisplayLayerGlobals -cdl genr8repellorLayer");
}