//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Force.h: interface for the Force class.
//
//	This class keeps track of all the forces in the scene; attractors and gravity
//	It is this class that should be queried when we wish to find out the resultant force for a point
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FORCE_H__4BE27F5D_1D34_11D5_95D7_00C04F797C24__INCLUDED_)
#define AFX_FORCE_H__4BE27F5D_1D34_11D5_95D7_00C04F797C24__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Attractor.h"
#include "Gravity.h"
#include <vector>

//using namespace std;

class Force  
{
public:
	Force();
	Force(bool attr, Vector pos, double c, int e, int ID, bool singularity);
	Force(std::vector<Vector> attrPos, std::vector<Vector> repPos, double force, Direction dir, double ac, double rc, int ae, int re, bool attrSing, bool repSing);
	virtual ~Force();

	int			numberOfAttractors()	{return _nAttractor;};
	Attractor*	getAttractor(int i)	{return _attractors[i];};
	bool		hasGravity()	{return (_gravity!=NULL);};
	Vector		getGravitationalForce(Vector pos)	{return _gravity->getForce(pos);};
	void		drawAttractors();
	void		addAttractor();
	void		newAttractor(bool attractor, Vector pos, int exp, int konst, int ID, bool singularity);
	void		newAttractor(bool attractor, Vector pos, int exp, int konst, bool drawn = false, bool singularity = true);
protected:
	//Methods
	//The attractors and repellors are drawn in separate layers to make it easier to toggle visibilty	
	void		createAttractorLayer();
	void		createRepellorLayer();
	//Data
	std::vector<Attractor*> _attractors;
	int			_nAttractor;
	Gravity*	_gravity;
	string		_attractorLayer, _repellorLayer;	//The names of the layers

};

#endif // !defined(AFX_FORCE_H__4BE27F5D_1D34_11D5_95D7_00C04F797C24__INCLUDED_)
