//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// GeneticEngine.cpp: implementation of the GeneticEngine class.
//
//////////////////////////////////////////////////////////////////////

#include <iostream>
#include "GeneticEngine.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

GeneticEngine::GeneticEngine()
{

}

GeneticEngine::GeneticEngine(int mutProb, int crossProb)
{
	_mutationProbability = mutProb;
	_crossoverProbability = crossProb;
}

GeneticEngine::~GeneticEngine()
{

}

void GeneticEngine::mutate(Genotype* &genome)
{
	int length = genome->getLength();
	//cout << "Mutating ggenome with length " << length << endl;
	for(int i=0; i<length; i++)	{
		if( (rand() % 100) < _mutationProbability )	{
			//The mutations are small adjustments, plus or minus one
			if((rand() % 2)==0)
				genome->incGene(i);
			else
				genome->decGene(i);
		}
	}
}

void GeneticEngine::continuousCrossover(Genotype* mother, Genotype* father, Genotype* &offspring)
{
	offspring->clear();
	int length = MIN(mother->getLength(), father->getLength());
	//Take the mean of the parents genes
	for(int i=0; i<length; i++)
		offspring->addGene( (mother->getGene(i) + father->getGene(i)) / 2);
}

//This is just straightforward crossover of two arrays of integers
//It is constructed to cope with genomes of different length.
//We should only get those if the user loads a genome into a population

void GeneticEngine::discreteCrossover(Genotype* mother, Genotype* father, Genotype* &son, Genotype* &daughter, int points)
{
	bool mother2son = true;
	son->clear();
	daughter->clear();
	vector<int> crossoverPoints;
	int sonLength = father->getLength(), daughterLength = mother->getLength();
	int interval = MIN( sonLength, daughterLength);
	//Determine the crossover points
	crossoverPoints.push_back(0);
	crossoverPoints.push_back(interval);
	for(int i=0; i<points; i++)
		crossoverPoints.push_back( rand() % interval );

	sort(crossoverPoints.begin(), crossoverPoints.end());

	for(int j=0; j<points+1; j++)	{
		for(int k=crossoverPoints[j]; k<crossoverPoints[j+1]; k++)	{
			if(mother2son)	{
				son->addGene( mother->getGene(k) );
				daughter->addGene( father->getGene(k) );
			}
			else	{
				son->addGene( father->getGene(k) );
				daughter->addGene( mother->getGene(k) );
			}
		}
		mother2son = !mother2son;
	}
	//Take the remaining genes
	if(sonLength > interval)	{
		for(int l=interval; l<sonLength; l++)
			son->addGene( father->getGene(l) );
	}
	else	{
		for(int m=interval; m<daughterLength; m++)
			daughter->addGene( mother->getGene(m) );
	}
}
