//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Individual.cpp: implementation of the Individual class.
//
//////////////////////////////////////////////////////////////////////

#include <iostream>
#include <sstream>
#include "Individual.h"
#include "Sentence.h"
#include "Word.h"
#include "MELUtilities.h"
//#include "Genotype.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Individual::Individual()
{

}

//Use this constructor when constructing individuals for regn8

Individual::Individual(int ID, string fileName)
{
	_ID = ID;
	_fractal = NO_FRACTAL;
	_tiles = 0;
	_sentence.push_back(new Sentence());
	_grammarLoaded = loadGrammar(fileName);
}

Individual::Individual(int ID, World* world, int tiles, bool fixedPerimeter, bool fixedCenter, double pc, int pe, Fractals fractal, string grammarFile, double limitedBranches, int branchAngle, bool softBoundary, BNFtype bnfType, int genomeLength, int randRewrite, bool fullRandomRewrite, int maxBNFDepth, int maxEdgeTypes, int numberOfScaffolds, bool differentGrammars, double branchLength, string encodedGrammar)
{
	_ID = ID;
	_rank = -1;
	//Fitness
	_impression = -1;
	_sizeFitness = 0;
	_smoothnessFitness = 0;
	_softBoundaryFitness = 0;
	_subdivisionsFitness = 0;
	_symmetryFitness = 0;
	_undulatingFitness = 0;
	_length = genomeLength;
	//Create the scaffolds
	for(int i=0; i<numberOfScaffolds; i++)	{
		Scaffold* scaffold = new Scaffold(world, i, fixedPerimeter, fixedCenter, pc, pe, limitedBranches, branchAngle, softBoundary, randRewrite, fullRandomRewrite, branchLength);
		_scaffold.push_back(scaffold);
	}
	//Store some data that is used by the scaffolds
	_numberOfScaffolds = numberOfScaffolds;	//If we have several scaffolds
	_differentGrammars = differentGrammars;	//We can have different gramamrs for the scaffolds
	_tiles = tiles;	//Predefined grammars
	_fractal = fractal;	//Predefined
	_grammarLoaded = false;
	_genome = NULL;
	_BNFtype = bnfType;	//We have four BNFs
	_maxBNFDepth = maxBNFDepth;	//Controls how deep we should expand the grammar
	_maxEdgeTypes = maxEdgeTypes;	//How many different edge type we allow
	_evolution = false;
	if(grammarFile!="")	{
		//Open grammar file and parse the contents
		_sentence.push_back(new Sentence());
		_grammarLoaded = loadGrammar(grammarFile);
	}
	//This is used when we use the "Grow individual" function in the GUI.
	else if(encodedGrammar!="")	{
		//Decode the grammar and store the contents
		decodeString(encodedGrammar);
		//cout << "Sentence decoded" << endl;
		_sentence.push_back(new Sentence(encodedGrammar));
		//cout << "Storing sentence, there are " << _numberOfScaffolds << " scaffolds." << endl;
		//Get the parameter values
		for(int j=0; j<_numberOfScaffolds; j++)	{
			int branchanglepos, anglepos = _sentence[0]->findWord("Angle") + 1;
			_scaffold[j]->setAngle( atoi(_sentence[0]->getWord(anglepos)->getName().c_str()) );
			if(_sentence[0]->findWord("Sync")!=-1)
				_scaffold[j]->setSyncronous(true);
			//cout << "Sync is " << boolalpha << (_sentence[0]->findWord("Sync")!=-1) << endl;
			branchanglepos = _sentence[0]->findWord("BranchAngle");
			if(branchanglepos!=-1)
				_scaffold[j]->setBranchAngle( atoi(_sentence[0]->getWord(branchanglepos + 1)->getName().c_str()) );
			//cout << "Angle is " << _sentence[0]->getWord(anglepos)->getName().c_str() << " and branchangle is " << _sentence[0]->getWord(branchanglepos+1)->getName().c_str() << endl;
		}
		_grammarLoaded = true;
	}
	else if(tiles==3)	{
		_sentence.push_back(new Sentence());
		//Construct a grammar that gives tiles
		//Axiom
		ostringstream stream;
		//stream << EDGE << "0 + " << EDGE << "1 + " << EDGE << "2 " << END_VAR << " ";
		newWord("Edge0");
		newWord("+");
		newWord("Edge1");
		newWord("+");
		newWord("Edge2");
		newWord(END_VAR);
		//Rewrite rule 1
		//stream << EDGE << "0 -> " << EDGE << "0 [ [ [ [ + " << EDGE << "2 ] + + " << EDGE << "1 ] - " << EDGE << "1 - - " << EDGE << "2 ] " << EDGE << "0 " << END_VAR << " ";
		newWord("Edge0");
		newWord("->");
		newWord("Edge0");
		newWord("[");
		newWord("[");
		newWord("[");
		newWord("[");
		newWord("+");
		newWord("Edge2");
		newWord("]");
		newWord("+");
		newWord("+");
		newWord("Edge1");
		newWord("]");
		newWord("-");
		newWord("Edge1");
		newWord("]");
		newWord("-");
		newWord("-");
		newWord("Edge2");
		newWord("]");
		newWord("Edge0");
		newWord(END_VAR);
		//Rule 2
		//stream << EDGE << "1 -> " << EDGE << "1 [ [ [ [ + " << EDGE << "0 ] + + " << EDGE << "2 ] - " << EDGE << "2 - - " << EDGE << "0 ] " << EDGE << "1 " << END_VAR << " ";
		newWord("Edge1");
		newWord("->");
		newWord("Edge1");
		newWord("[");
		newWord("[");
		newWord("[");
		newWord("[");
		newWord("+");
		newWord("Edge0");
		newWord("]");
		newWord("+");
		newWord("+");
		newWord("Edge2");
		newWord("]");
		newWord("-");
		newWord("Edge2");
		newWord("]");
		newWord("-");
		newWord("-");
		newWord("Edge0");
		newWord("]");
		newWord("Edge1");
		newWord(END_VAR);
		//Rule 3
		//stream << EDGE << "2 -> " << EDGE << "2 [ [ [ [ + " << EDGE << "1 ] + + " << EDGE << "0 ] - " << EDGE << "0 - - " << EDGE << "1 ] " << EDGE << "2 " << END_VAR;
		newWord("Edge2");
		newWord("->");
		newWord("Edge2");
		newWord("[");
		newWord("[");
		newWord("[");
		newWord("[");
		newWord("+");
		newWord("Edge1");
		newWord("]");
		newWord("+");
		newWord("+");
		newWord("Edge0");
		newWord("]");
		newWord("-");
		newWord("Edge0");
		newWord("]");
		newWord("-");
		newWord("-");
		newWord("Edge1");
		newWord("]");
		newWord("Edge2");
		newWord(END_VAR);
		//_sentence.push_back(new Sentence(stream.str()));
		//Set parameters
		setParameters(60, true, 0);
		for(int j=0; j<_numberOfScaffolds; j++)
			_scaffold[j]->setNumberOfEdgeTypes(3);
	}
	else if(tiles==4)	{
		_sentence.push_back(new Sentence());
		//Construct a grammar that gives tiles
		//Axiom
		newWord("Edge0");
		newWord("+");
		newWord("~");
		newWord("Edge1");
		newWord("+");
		newWord("~");
		newWord("Edge0");
		newWord("+");
		newWord("Edge1");
		newWord(END_VAR);
		//Rewrite rule 1
		newWord("Edge0");
		newWord("->");
		newWord("Edge0");
		newWord("[");
		newWord("[");
		newWord("+");
		newWord("Edge1");
		newWord("]");
		newWord("-");
		newWord("~");
		newWord("Edge1");
		newWord("]");
		newWord("Edge0");
		newWord(END_VAR);
		//Rule 2
		newWord("Edge1");
		newWord("->");
		newWord("Edge1");
		newWord("[");
		newWord("[");
		newWord("+");
		newWord("Edge0");
		newWord("]");
		newWord("-");
		newWord("~");
		newWord("Edge0");
		newWord("]");
		newWord("Edge1");
		newWord(END_VAR);
		//Parameters
		setParameters(90, false, 0);
		for(int j=0; j<_numberOfScaffolds; j++)
			_scaffold[j]->setNumberOfEdgeTypes(2);
	}
	else if(fractal==KOCH)	{
		_sentence.push_back(new Sentence());
		//Axiom
		newWord("Edge0");
		newWord("+");
		newWord("Edge0");
		newWord("+");
		newWord("Edge0");
		newWord(END_VAR);
		//Rewrite rule
		newWord("Edge0");
		newWord("->");
		newWord("Edge0");
		newWord("-");
		newWord("Edge0");
		newWord("+");
		newWord("+");
		newWord("Edge0");
		newWord("-");
		newWord("Edge0");
		newWord(END_VAR);
		setParameters(60, false, 0);
		//Change the angle
		for(int j=0; j<_numberOfScaffolds; j++)
			_scaffold[j]->setNumberOfEdgeTypes(1);
	}
	else if(fractal==Q_KOCH)	{
		_sentence.push_back(new Sentence());
		//Axiom
		newWord("Edge0");
		newWord("+");
		newWord("Edge0");
		newWord("+");
		newWord("Edge0");
		newWord("+");
		newWord("Edge0");
		newWord(END_VAR);
		//Rewrite rule
		newWord("Edge0");
		newWord("->");
		newWord("Edge0");
		newWord("+");
		newWord("Edge0");
		newWord("-");
		newWord("Edge0");
		newWord("-");
		newWord("Edge0");
		newWord("Edge0");
		newWord("+");
		newWord("Edge0");
		newWord("+");
		newWord("Edge0");
		newWord("-");
		newWord("Edge0");
		newWord(END_VAR);
		setParameters(90, false, 0);
		for(int j=0; j<_numberOfScaffolds; j++)
			_scaffold[j]->setNumberOfEdgeTypes(0);
	}
	//We are going to evolve a grammar for this individual
	else	{
		//Create a genome
		_genome = new Genotype(genomeLength);
		//_genome->printGenome();
		_evolution = true;
	}
	//_sentence->printSentence();
}

Individual::~Individual()
{

}

//	Parameters:
//	Return:
//	Note:
//		This function is called by the world when it is time for the individual to interpret
//		its genome and create a grammar.

void Individual::generateGrammar()
{
	if(_evolution)	{
		//Interpret it
		BNF* bnf = new BNF(_genome, _BNFtype, _maxEdgeTypes);
		_BNF.push_back(bnf);
		_BNF[0]->getLSystem()->expand(_maxBNFDepth);
		_sentence.push_back(_BNF[0]->getFullSentence());
		//Set the parameters
		int angle = ANGLE, branchAngle = BRANCH_ANGLE;
		bool sync = false;
		if(_scaffold.size()>0)	{
			setParameters(angle, sync, branchAngle, 0, true);
			_scaffold[0]->setAngle(angle);
			_scaffold[0]->setSyncronous(sync);
			_scaffold[0]->setBranchAngle(branchAngle);
			_scaffold[0]->setNumberOfEdgeTypes(_BNF[0]->getEdgeTypes());
		}
		else	{	//Use this for regn8, we do not have a scaffold
			setParameters(angle, sync, branchAngle, 0, true);
		}
		//If we have several surface with different grammars
		if(_differentGrammars)	{
			for(int i=1; i<_numberOfScaffolds; i++)	{
				bnf = new BNF(_genome, _BNFtype, _maxEdgeTypes);
				_BNF.push_back(bnf);
				_BNF[i]->getLSystem()->expand(_maxBNFDepth);
				_sentence.push_back(_BNF[i]->getFullSentence());
				_sentence[i]->printSentence();
				setParameters(angle, sync, branchAngle, i, true);
				_scaffold[i]->setAngle(angle);
				_scaffold[i]->setSyncronous(sync);
				_scaffold[i]->setBranchAngle(branchAngle);
				_scaffold[i]->setNumberOfEdgeTypes(_BNF[i]->getEdgeTypes());
			}
		}
		
	}
	else if(_grammarLoaded)
		_scaffold[0]->setNumberOfEdgeTypes(_sentence[0]->numberOfEdgeTypes());

}

//	Parameters:
//		int &angle
//			The turn angle for the turtle
//		bool &sync
//			The style for joining branches
//		int &branchangle
//			The branchangle parameter
//		int sent
//			The index of the sentence we are currently working with
//		bool evol
//			True if the parameters are set by an evolved grammars
//	Return:
//	Note:
//		This function adds the parameters to the grammar

void Individual::setParameters(int &angle, bool &sync, int &branchAngle, int sent, bool evol)
{
	//Angle
	newWord("Angle", sent);
	if(_BNFtype==SYMMETRIC)
		angle = 45;
	else
		angle = evol ? ( (_BNF[sent]->getNextGene() % 2) + 2 )*15 : angle;
	newWord(angle, sent);
	newWord(END_VAR, sent);
	//Syncronous
	sync = evol ? (_BNF[sent]->getNextGene() % 2)!=0 : sync;
	if(sync)	{
		newWord("Sync", sent);
		newWord(END_VAR, sent);
	}
	//Branch Angle
	newWord("BranchAngle", sent);
	branchAngle = evol ? ( (_BNF[sent]->getNextGene() % 4) + 2 )*15 : branchAngle;
	newWord(branchAngle, sent);
	newWord(END_VAR, sent);
}

//An overloaded version of the previous one.
//It is used by the predefined grammars

void Individual::setParameters(const int angle, const bool sync, int sent)
{
	//Angle
	newWord("Angle", sent);
	newWord(angle, sent);
	newWord(END_VAR, sent);
	//Syncronous
	if(sync)	{
		newWord("Sync", sent);
		newWord(END_VAR, sent);
	}
}

//	Parameters:
//	Return:
//	Note:
//		This function tells the scaffolds to parse the sentences held by the individual
//		It is called by the world.
//		The reason that we do not parse the sentence immediately is that we do not always
//		have a scaffold that can parse (if we use regn8).

void Individual::interpretGrammar()
{
	cout << endl << "The grammar of individual " << _ID << " is:" << endl;
	_sentence[0]->printSentence();
	for(int i=0; i<_numberOfScaffolds; i++)	{
		if(_differentGrammars)
			_scaffold[i]->parseSentence(_sentence[i]);
		else
			_scaffold[i]->parseSentence(_sentence[0]);
	}
	//cout << "Angle " << _scaffold->getAngle() << endl;
}

//This function is just a middle step between the world and the scaffold, because we do not
//want the world to have immediate access to the scaffold. See scaffold for more details

bool Individual::makeCellsUniform(double minDifference, double minRelDifference)
{
	for(int i=0; i<_numberOfScaffolds; i++)
		_scaffold[i]->uniformRegions(minDifference, minRelDifference);
	return true;
}

//A utility function that creates the word and adds it to the sentence

void Individual::newWord(string name, int sent)
{
	Word* word = new Word(name);
	_sentence[sent]->addWord(word);
}

void Individual::newWord(int name, int sent)
{
	ostringstream stream;
	stream << name;
	Word* word = new Word(stream.str());
	_sentence[sent]->addWord(word);
}

//The individual is drawn in a separate layer

void Individual::createNewLayer(int num, int step)
{
	ostringstream newName;
	if(step==-1)
		newName << "createDisplayLayer -empty -mc -n genr8Lay" << num;
	else
		newName << "createDisplayLayer -empty -mc -n genr8Lay" << num << "Step" << step;
	eval(newName.str());
	_layerName = getStringResults("editDisplayLayerGlobals -q -cdl");
}

//This function toggles the visibility of the layer

void Individual::setVisibility(bool visible)
{
	ostringstream cmd;
	cmd << "setAttr " << _layerName << ".visibility " << boolalpha << visible;
	eval(cmd.str());
}

//	Parameters:
//		surfaceRepresentation surfRep
//			The type of surface to draw
//		bool visible
//			True if the surface shall be displayed immediately
//	Return:
//	Note:
//		Called by the world to draw the surface that has been grown

void Individual::drawScaffold(surfaceRepresentation surfRep, bool visible, int step)
{
	//Set the layer visible
	ostringstream cmd;
	//cout << "The layer name is " << _layerName << endl;
	if(step!=-1 || _layerName=="")
		createNewLayer(_ID, step);
	if(visible)
		cmd << "setAttr " << _layerName << ".visibility 1; editDisplayLayerGlobals -cdl " << _layerName;
	else
		cmd << "setAttr " << _layerName << ".visibility 0; editDisplayLayerGlobals -cdl " << _layerName;
	eval(cmd.str());
	//Draw scaffold
	for(int i=0; i<_numberOfScaffolds; i++)	{
		if(_tiles==4 && surfRep==SURFACE)
			//We should draw a nurbs-surface instead of an ordinary, since we have the predefined grammar
			_scaffold[i]->drawScaffold(QUILT);
		else
			_scaffold[i]->drawScaffold(surfRep);
	}
}

//Clear the stuff from the scene

void Individual::eraseScaffold()
{
	//Set layer invisible
	ostringstream cmd;
	cmd << "setAttr " << _layerName << ".visibility 0; editDisplayLayerGlobals -cdl " << _layerName;
	eval(cmd.str());
	//Erase scaffold
	for(int i=0; i<_numberOfScaffolds; i++)
		_scaffold[i]->eraseDrawings();
}

//Clear the internal model

void Individual::clear()
{
	if(_evolution)	{
		_BNF.clear();
		_sentence.clear();
	}
	for(int j=0; j<_numberOfScaffolds; j++)
		_scaffold[j]->clear();
}

//Load a grammar from file to the individual

bool Individual::loadGrammar(string file)
{
	ifstream grammarFile;
	if(!opengenr8File(file, grammarFile, true, true))
		return false;
	//Parse the contents
	Word* word;
	string tmp;
	char buf[800], *name;
	int size;
	while(!grammarFile.eof())	{
		grammarFile.getline(buf, 800);
		size = grammarFile.gcount();
		//cout << "Read " << size << " characters: " << buf << endl;
		if(size>0)	{
			//tokenize
			name = strtok(buf, " \n\r\t");
			if(name)
				tmp.assign(name);
			if(tmp!="//")	{	//Comment
				word = new Word(name);
				_sentence[0]->addWord(word);
				do	{
					name = strtok(NULL, " \n\r\t");
					if(name)	{
						if( ispunct(*name) || isalnum(*name) )
							newWord(name);
					}
					else
						newWord(END_VAR);
						
				}	while(name!=NULL);
			}
		}
	}
	grammarFile.close();
	_sentence[0]->printSentence();
	return true;
}

void Individual::saveGrammar(ofstream &file)
{
	cout << "Saving grammar for individual " << _ID << endl;
	file << "// Grammar for individual " << _ID << '\n';
	for(int i=0; i<_numberOfScaffolds; i++)
		_sentence[i]->saveSentence(file);
	cout << "Saved grammar." << endl;
}