//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Individual.h: interface for the Individual class.
//
//	This class represents the individuals of the population. The individual basically
//	consists of a genome and a scaffold, but we need to store some additional info.
//	Currently there is support for multiple scaffolds. However, there are no interactions implemented,
//	that remains to be done.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_INDIVIDUAL_H__2B8405DB_2380_11D5_95D7_00C04F797C24__INCLUDED_)
#define AFX_INDIVIDUAL_H__2B8405DB_2380_11D5_95D7_00C04F797C24__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

typedef enum {NO_FRACTAL, KOCH, Q_KOCH, DRAGON} Fractals;

#include "Scaffold.h"
#include "BNF.h"
#include "Genotype.h"
#include "Vector.h"
#include <string>
#include <vector>
#include <fstream>


using namespace std;



class Sentence;

class Individual  
{
public:
	Individual();
	Individual(int ID, string fileName);
	Individual(int ID, World* world, int tiles, bool fixedPerimeter, bool fixedCenter, double pc, int pe, Fractals fractal, string grammarFile, double limitedBranches, int branchAngle, bool softBoundary, BNFtype bnfType, int genomeLength, int randRewrite, bool fullRandomRewrite, int maxBNFDepth, int maxEdgeTypes, int numberOfScaffolds, bool differentGrammars, double branchLength, string encodedGrammar);
	virtual ~Individual();

	int			getID()	{return _ID;};
	Scaffold*	getScaffold(int scaff = 0)	{return _scaffold[scaff];};
	int			numberOfScaffolds()	{return _numberOfScaffolds;};
	//Fitness, we need to store the separate values for the GUI
	double		getImpression()	{return _impression;};
	void		setImpression(double impression)	{_impression = impression;};
	double		getSizeFitness()	{return _sizeFitness;};
	void		setSizeFitness(double sizeFitness)	{_sizeFitness = sizeFitness;};
	double		getSmoothnessFitness()	{return _smoothnessFitness;};
	void		setSmoothnessFitness(double smoothnessFitness)	{_smoothnessFitness = smoothnessFitness;};
	double		getSoftBoundaryFitness()	{return _softBoundaryFitness;};
	void		setSoftBoundaryFitness(double softBoundaryFitness)	{_softBoundaryFitness = softBoundaryFitness;};
	double		getSubdivisionsFitness()	{return _subdivisionsFitness;};
	void		setSubdivisionsFitness(double subdivisionsFitness)	{_subdivisionsFitness = subdivisionsFitness;};
	double		getSymmetryFitness()	{return _symmetryFitness;};
	void		setSymmetryFitness(double symmetryFitness)	{_symmetryFitness = symmetryFitness;};
	double		getUndulatingFitness()	{return _undulatingFitness;};
	void		setUndulatingFitness(double undulatingFitness)	{_undulatingFitness = undulatingFitness;};

	int			getRank()	{return _rank;};
	void		getGenes(vector<int> &genes);
	Genotype*	getGenome()	{return _genome;};
	void		setGenome(Genotype* genome)	{_genome = genome;};
	void		generateGrammar();	//Interpret the genome with the BNF
	void		interpretGrammar();	//Let the scaffold parse the grammar
	bool		makeCellsUniform(double minDifference, double minRelDifference);
	//Construct the genotypically created seed
	void		setSeed(Vector startPos, double startLength, int scaff = 0)	{_scaffold[scaff]->setSeed(startPos, startLength);};
	//Construct the user defined seed
	void		setSeed(vector<Vector> seed, vector<double> masses, int scaff = 0)	{_scaffold[scaff]->setSeed(seed, masses, _BNF[scaff]->getEdgeTypes());};
	//void		step(double scale)	{for(int i=0; i<_numberOfScaffolds; i++)	_scaffold[i]->step(scale);};
	//Grow the scaffold
	void		step(double scaleX, double scaleY, double scaleZ)	{for(int i=0; i<_numberOfScaffolds; i++)	_scaffold[i]->step(scaleX, scaleY, scaleZ);};
	void		drawScaffold(surfaceRepresentation surfRep, bool visible, int step = -1);
	void		eraseScaffold();
	void		createNewLayer(int num, int step = -1);
	void		setVisibility(bool visible);
	string		getLayerName()	{return _layerName;};
	Sentence*	getSentence(int i=0)	{return _sentence[i];};
	void		saveGrammar(ofstream &file);
	void		clear();
	//For regn8
	void		constructGenome()	{_genome = _BNF[0]->constructGenome(_sentence[0]);};
protected:
	//Methods
	void		newWord(string name, int sent = 0);
	void		newWord(int name, int sent = 0);
	bool		loadGrammar(string file);
	//Set the parameters from the genome
	void		setParameters(int &angle, bool &sync, int &branchAngle, int sent = 0, bool evol = true);
	void		setParameters(const int angle, const bool sync, int sent);
	//Data
	vector<Scaffold*>	_scaffold;
	vector<BNF*>	_BNF;	//This contains the info to construct a grammar
	//The fitness values
	double		_impression, _sizeFitness, _smoothnessFitness, _softBoundaryFitness, _subdivisionsFitness, _symmetryFitness, _undulatingFitness;
	int			_ID;
	int			_rank;
	string		_layerName;
	//For multiple scaffolds
	int			_numberOfScaffolds;
	bool		_differentGrammars;
	//Rememeber some things about the scaffold
	Fractals	_fractal;
	int			_tiles;
	//The realization of the BNF
	vector<Sentence*>	_sentence;
	bool		_grammarLoaded, _evolution;
	BNFtype		_BNFtype;
	int			_maxBNFDepth, _maxEdgeTypes;
	//Genome
	Genotype*	_genome;
	int			_length;

};

#endif // !defined(AFX_INDIVIDUAL_H__2B8405DB_2380_11D5_95D7_00C04F797C24__INCLUDED_)
