//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// MELUtilities.cpp: implementation of the MELUtilities class.
//
//////////////////////////////////////////////////////////////////////

#include "MELUtilities.h"
#include <sstream>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
/*
MELUtilities* MELUtilities::theMELUtilities = NULL;

MELUtilities::MELUtilities()
{
	theMELUtilities = NULL;
}

MELUtilities::~MELUtilities()
{

}*/

//Execute a MEL-command

void eval(std::string command)
{
	//cout << "Running command: " << command << endl;

	MGlobal::executeCommand(command.c_str());
}

//Executes "refresh"
void refresh()
{
	eval("refresh");
}

// Utilities to get MEL values from internal variables

void getMELvalue(string name, int &value)
{
	long result;
	eval(name);
	MPxCommand::getCurrentResult(result);
	value = result;
}

void getMELvalue(string name, string &value)
{
	MString result;
	eval(name);
	MPxCommand::getCurrentResult(result);
	value.assign( result.asChar() );
}

char* getCharResults(string cmd)
{	
	string value;
	MString result;
	MStringArray strings;
	MGlobal::executeCommand(cmd.c_str(), strings);
	
	//cout << "The string array has length " << strings.length() << endl;
	value.assign( strings[0].asChar() );
	//cout << "The result was " << value << endl;
	return const_cast<char*>(value.c_str());
}

string getStringResults(string cmd)
{	
	string value;
	MString result;
	MStringArray strings;
	MGlobal::executeCommand(cmd.c_str(), strings);
	
	//cout << "The string array has length " << strings.length() << endl;
	value.assign( strings[0].asChar() );
	//cout << "The result was " << value << endl;
	return value;
}

string* getStringPointerResults(string cmd)
{	
	string *value = new string;
	MString result;
	MStringArray strings;
	MGlobal::executeCommand(cmd.c_str(), strings);
	
	//cout << "The string array has length " << strings.length() << endl;
	value->assign( strings[0].asChar() );
	//cout << "The result was " << value->c_str() << endl;
	return value;
}

char* getCharResult(string cmd)
{	
	MString result;
	MGlobal::executeCommand(cmd.c_str(), result);
	//cout << "getCharResult returns " << result.asChar() << endl;
	return const_cast<char*>(result.asChar());
}

string* getStringPointerResult(string cmd)
{	
	string *value = new string;
	MString result;
	MGlobal::executeCommand(cmd.c_str(), result);
	//cout << "getCharResult returns " << result.asChar() << endl;
	value->assign( result.asChar() );
	return value;
}

int getIntResult(string cmd)
{	
	int result;
	//cout << "getIntResult executes: " << cmd.c_str() << endl;
	MGlobal::executeCommand(cmd.c_str(), result);
	//cout << "getIntResult returns " << result << endl;
	return result;
}

double	getDoubleResult(string cmd)
{
	double result;
	MGlobal::executeCommand(cmd.c_str(), result);
	return result;
}

vector<double>	getDoubleResults(string cmd)
{
	vector<double> retval;
	MDoubleArray result;
	MGlobal::executeCommand(cmd.c_str(), result);
	for(int i=0; i<result.length(); i++)
		retval.push_back(result[i]);
	return retval;
}


///////////////////////////////
//
//	OptionVars
//
///////////////////////////////

//Optionvars are used by the GUI to store values changed by the user.
//We first check if the variable exist and if so we query it for the result and store it.

bool getOptionVar(string variable, int &value, string &cmd, bool valueOnly, bool remove)
{
	ostringstream exist, opt, rem, tmp;
	exist << "optionVar -exists \"" << variable.c_str() << "\"";
	if(getIntResult(exist.str())==0)
		return false;
	opt << "optionVar -q \"" << variable.c_str() << "\"";
	value = getIntResult(opt.str());
	//Create a string that could be executed
	//cout << "Found " << variable.c_str() << " with value " << value << endl;
	if(!valueOnly)
		tmp << " -" << variable << ' ' << value;
	else
		tmp << " " << value;
	cmd += tmp.str();
	//Clean up
	if(remove)	{
		rem << "optionVar -remove \"" << variable.c_str() << "\"";
		eval(rem.str());
	}

	return true;
}

bool getOptionVar(string variable, double &value, string &cmd, bool valueOnly, bool remove)
{
	ostringstream exist, opt, rem, tmp;
	exist << "optionVar -exists \"" << variable.c_str() << "\"";
	if(getIntResult(exist.str())==0)
		return false;
	opt << "optionVar -q \"" << variable.c_str() << "\"";
	value = getDoubleResult(opt.str());
	//Create a string that could be executed
	if(!valueOnly)
		tmp << " -" << variable << ' ' << value;
	else
		tmp << " " << value;
	cmd += tmp.str();
	//Clean up
	if(remove)	{
		rem << "optionVar -remove \"" << variable.c_str() << "\"";
		eval(rem.str());
	}

	return true;
}

bool getOptionVar(string variable, bool &value, string &cmd, bool valueOnly, bool remove)
{
	ostringstream exist, opt, rem, tmp;
	exist << "optionVar -exists \"" << variable.c_str() << "\"";
	if(getIntResult(exist.str())==0)
		return false;
	opt << "optionVar -q \"" << variable.c_str() << "\"";
	value = getIntResult(opt.str())!=0;
	//Create a string that could be executed
	if(!valueOnly)
		tmp << " -" << variable;
	cmd += tmp.str();
	//Clean up
	if(remove)	{
		rem << "optionVar -remove \"" << variable.c_str() << "\"";
		eval(rem.str());
	}

	return true;
}

bool getOptionVar(string variable, string &value, string &cmd, bool valueOnly, bool remove)
{
	ostringstream exist, opt, rem, tmp;
	exist << "optionVar -exists \"" << variable.c_str() << "\"";
	if(getIntResult(exist.str())==0)
		return false;
	opt << "optionVar -q \"" << variable.c_str() << "\"";
	value = (*getStringPointerResult(opt.str()));
	//Create a string that could be executed
	if(!valueOnly)
		tmp << " -" << variable << ' ' << value.c_str() << "\"";
	else
		tmp << " " << value.c_str() << "\"";
	cmd += tmp.str();
	//Clean up
	if(remove)	{
		rem << "optionVar -remove \"" << variable.c_str() << "\"";
		eval(rem.str());
	}

	return true;
}

bool getOptionVar(string variable, int &value, bool remove)
{
	ostringstream exist, opt, rem, tmp;
	exist << "optionVar -exists \"" << variable.c_str() << "\"";
	if(getIntResult(exist.str())==0)
		return false;
	opt << "optionVar -q \"" << variable.c_str() << "\"";
	value = getIntResult(opt.str());
	//Clean up
	if(remove)	{
		rem << "optionVar -remove \"" << variable.c_str() << "\"";
		eval(rem.str());
	}

	return true;
}

bool getOptionVar(string variable, double &value, bool remove)
{
	ostringstream exist, opt, rem, tmp;
	exist << "optionVar -exists \"" << variable.c_str() << "\"";
	if(getIntResult(exist.str())==0)
		return false;
	opt << "optionVar -q \"" << variable.c_str() << "\"";
	value = getDoubleResult(opt.str());
	//Clean up
	if(remove)	{
		rem << "optionVar -remove \"" << variable.c_str() << "\"";
		eval(rem.str());
	}

	return true;
}

bool getOptionVar(string variable, bool &value, bool remove)
{
	ostringstream exist, opt, rem, tmp;
	exist << "optionVar -exists \"" << variable.c_str() << "\"";
	if(getIntResult(exist.str())==0)
		return false;
	opt << "optionVar -q \"" << variable.c_str() << "\"";
	value = getIntResult(opt.str())!=0;
	//Clean up
	if(remove)	{
		rem << "optionVar -remove \"" << variable.c_str() << "\"";
		eval(rem.str());
	}

	return true;
}

bool getOptionVar(string variable, string &value, bool remove)
{
	ostringstream exist, opt, rem, tmp;
	exist << "optionVar -exists \"" << variable.c_str() << "\"";
	if(getIntResult(exist.str())==0)
		return false;
	opt << "optionVar -q \"" << variable.c_str() << "\"";
	value = (*getStringPointerResult(opt.str()));
	//Clean up
	if(remove)	{
		rem << "optionVar -remove \"" << variable.c_str() << "\"";
		eval(rem.str());
	}

	return true;
}

bool getOptionVar(string variable, vector<double> &value, bool remove)
{
	ostringstream exist, opt, rem, tmp;
	exist << "optionVar -exists \"" << variable.c_str() << "\"";
	if(getIntResult(exist.str())==0)
		return false;
	opt << "optionVar -q \"" << variable.c_str() << "\"";
	value = getDoubleResults(opt.str());
	//Clean up
	if(remove)	{
		rem << "optionVar -remove \"" << variable.c_str() << "\"";
		eval(rem.str());
	}

	return true;
}

//Save the optionvar in a file, we also store the type of the value

void saveOptionVar(string variable, ofstream &file, varType var)
{
	ostringstream exist, opt;
	//Check if the variable exists
	exist << "optionVar -exists \"" << variable.c_str() << "\"";
	if(getIntResult(exist.str())==0)
		return;
	opt << "optionVar -q \"" << variable.c_str() << "\"";
	//Save the name and the type
	file << variable << " ";
	if(var==INTEGER)
		file << INT_VAR << " " << getIntResult(opt.str()) << endl;
	else if(var==DOUBBLOAT)
		file << DOUBLE_VAR << " " << getDoubleResult(opt.str()) << endl;
	else if(var==STRING)
		file << STRING_VAR << " " << (getStringPointerResult(opt.str()))->c_str() << endl;
}

//Read all the optionvars that were saved in a file

void loadOptionVar(ifstream &file)
{
	char buf[800], *variable, *type;
	string var;
	while(!file.eof())	{
		//Read a line
		file.getline(buf, 800);
		if(file.gcount()>0)	{
			//Tokenize and get the variable name
			variable = strtok(buf, " ");
			//Get the type of variable
			type = strtok(NULL, " ");
			var.assign(type);
			//Get the value
			ostringstream opt;
			if(var==INT_VAR)
				opt << "optionVar -iv " << variable << " " << atoi( strtok(NULL, " ") ) << ";";
			else if(var==DOUBLE_VAR)
				opt << "optionVar -fv " << variable << " " << atof( strtok(NULL, " ") ) << ";";
			else if(var==STRING_VAR)
				opt << "optionVar -sv " << variable << " " << strtok(NULL, " ") << ";";
			eval(opt.str());
		}
	}
}


///////////////////////////////
//
//	File-handling
//
///////////////////////////////

//We use thes functions for file-opening

bool opengenr8File(string fileName, ifstream &file, bool fullPath, bool encoded)
{
	if(encoded)
		decodeString(fileName);
	string fname;
	if(!fullPath)	{
		//Get the location of the file
		string mayaLoc = getStringResults("getenv \"MAYA_LOCATION\"");
		//Simple replacement of key-characters
		for (int i = 0; i < mayaLoc.size(); i++)	{
			if (mayaLoc[i] == '/')
				mayaLoc[i] = '\\';
		}

		fname = mayaLoc + DEFAULT_LOCATION + fileName;
	}
	else
		fname = fileName;
	cout << "Trying to open file in " << fname << endl;
	//Open file
	file.open(fname.c_str());
	if(!file)	{
		//Produce a window with error message
		cout << "D\'oh! Failed to open " << fname << endl;
		return false;
	}
	cout << "Wuhu! The file was opened." << endl;
	return true;
}

bool opengenr8File(string fileName, ofstream &file, bool fullPath, bool encoded)
{
	if(encoded)
		decodeString(fileName);
	string fname;
	if(!fullPath)	{
		//Get the location of the file
		string mayaLoc = getStringResults("getenv \"MAYA_LOCATION\"");
		//Simple replacement of key-characters
		for (int i = 0; i < mayaLoc.size(); i++)	{
			if (mayaLoc[i] == '/')
				mayaLoc[i] = '\\';
		}

		fname = mayaLoc + DEFAULT_LOCATION + fileName;
	}
	else
		fname = fileName;
	cout << "Trying to open file in " << fname << endl;
	//Open file
	file.open(fname.c_str());
	if(!file)	{
		//Produce a window with error message
		cout << "D\'oh! Failed to open " << fname << endl;
		return false;
	}
	cout << "Wuhu! The file was opened." << endl;
	return true;
}

//Since Maya has a hard time coping with special characters and even numbers (sometimes) we circumvent
//the problem by encoding the characters using letters.

string encodeString(string &str)
{
	for(int i=0; i<str.size(); i++)	{
		if(str[i]=='+')
			str.replace(i, 1, PLUS_VAR);
		else if(str[i]=='-' && str[i+1]=='>')
			str.replace(i, 2, ARROW_VAR);
		else if(str[i]=='-')
			str.replace(i, 1, MINUS_VAR);
		else if(str[i]=='^')
			str.replace(i, 1, HAT_VAR);
		else if(str[i]=='&')
			str.replace(i, 1, AMPERSAND_VAR);
		else if(str[i]=='\\')
			str.replace(i, 1, BACKSLASH_VAR);
		else if(str[i]=='/')
			str.replace(i, 1, FORWARDSLASH_VAR);
		else if(str[i]=='~')
			str.replace(i, 1, TILDE_VAR);
		else if(str[i]=='[')
			str.replace(i, 1, LEFT_BRACKET_VAR);
		else if(str[i]==']')
			str.replace(i, 1, RIGHT_BRACKET_VAR);
		else if(str[i]=='>')
			str.replace(i, 1, GREATER_THAN_VAR);
		else if(str[i]=='<')
			str.replace(i, 1, LESSER_THAN_VAR);
		else if(str[i]=='0')
			str.replace(i, 1, ZERO_VAR);
		else if(str[i]=='1')
			str.replace(i, 1, ONE_VAR);
		else if(str[i]=='2')
			str.replace(i, 1, TWO_VAR);
		else if(str[i]=='3')
			str.replace(i, 1, THREE_VAR);
		else if(str[i]=='4')
			str.replace(i, 1, FOUR_VAR);
		else if(str[i]=='5')
			str.replace(i, 1, FIVE_VAR);
		else if(str[i]=='6')
			str.replace(i, 1, SIX_VAR);
		else if(str[i]=='7')
			str.replace(i, 1, SEVEN_VAR);
		else if(str[i]=='8')
			str.replace(i, 1, EIGHT_VAR);
		else if(str[i]=='9')
			str.replace(i, 1, NINE_VAR);
		else if(str[i]=='.')
			str.replace(i, 1, DOT_VAR);
		else if(str[i]==' ')
			str.replace(i, 1, SPACE_VAR);
		else if(str[i]=='\n')
			str.replace(i, 1, END_VAR);
	}
	//Make sure that we do not end with '.'
	if(str[str.size()-1]=='.')
		str.erase(str.size()-1);
	return str;
}

string decodeString(string &str)
{
	//Replace the characters one at a time
	int pos = str.find(PLUS_VAR);
	while(pos!=-1)	{
		str.replace(pos, PLUS_VAR_LENGTH, "+");
		pos = str.find(PLUS_VAR, pos);
	}
	pos = str.find(ARROW_VAR);
	while(pos!=-1)	{
		str.replace(pos, ARROW_VAR_LENGTH, "->");
		pos = str.find(ARROW_VAR, pos);
	}
	pos = str.find(MINUS_VAR);
	while(pos!=-1)	{
		str.replace(pos, MINUS_VAR_LENGTH, "-");
		pos = str.find(MINUS_VAR, pos);
	}
	pos = str.find(HAT_VAR);
	while(pos!=-1)	{
		str.replace(pos, HAT_VAR_LENGTH, "^");
		pos = str.find(HAT_VAR, pos);
	}
	pos = str.find(AMPERSAND_VAR);
	while(pos!=-1)	{
		str.replace(pos, AMPERSAND_VAR_LENGTH, "&");
		pos = str.find(AMPERSAND_VAR, pos);
	}
	pos = str.find(BACKSLASH_VAR);
	while(pos!=-1)	{
		str.replace(pos, BACKSLASH_VAR_LENGTH, "\\");
		pos = str.find(BACKSLASH_VAR, pos);
	}
	pos = str.find(FORWARDSLASH_VAR);
	while(pos!=-1)	{
		str.replace(pos, FORWARDSLASH_VAR_LENGTH, "/");
		pos = str.find(FORWARDSLASH_VAR, pos);
	}
	pos = str.find(TILDE_VAR);
	while(pos!=-1)	{
		str.replace(pos, TILDE_VAR_LENGTH, "~");
		pos = str.find(TILDE_VAR, pos);
	}
	pos = str.find(LEFT_BRACKET_VAR);
	while(pos!=-1)	{
		str.replace(pos, LEFT_BRACKET_VAR_LENGTH, "[");
		pos = str.find(LEFT_BRACKET_VAR, pos);
	}
	pos = str.find(RIGHT_BRACKET_VAR);
	while(pos!=-1)	{
		str.replace(pos, RIGHT_BRACKET_VAR_LENGTH, "]");
		pos = str.find(RIGHT_BRACKET_VAR, pos);
	}
	pos = str.find(GREATER_THAN_VAR);
	while(pos!=-1)	{
		str.replace(pos, GREATER_THAN_VAR_LENGTH, ">");
		pos = str.find(GREATER_THAN_VAR, pos);
	}
	pos = str.find(LESSER_THAN_VAR);
	while(pos!=-1)	{
		str.replace(pos, LESSER_THAN_VAR_LENGTH, "<");
		pos = str.find(LESSER_THAN_VAR, pos);
	}
	pos = str.find(ZERO_VAR);
	while(pos!=-1)	{
		str.replace(pos, ZERO_VAR_LENGTH, "0");
		pos = str.find(ZERO_VAR, pos);
	}
	pos = str.find(ONE_VAR);
	while(pos!=-1)	{
		str.replace(pos, ONE_VAR_LENGTH, "1");
		pos = str.find(ONE_VAR, pos);
	}
	pos = str.find(TWO_VAR);
	while(pos!=-1)	{
		str.replace(pos, TWO_VAR_LENGTH, "2");
		pos = str.find(TWO_VAR, pos);
	}
	pos = str.find(THREE_VAR);
	while(pos!=-1)	{
		str.replace(pos, THREE_VAR_LENGTH, "3");
		pos = str.find(THREE_VAR, pos);
	}
	pos = str.find(FOUR_VAR);
	while(pos!=-1)	{
		str.replace(pos, FOUR_VAR_LENGTH, "4");
		pos = str.find(FOUR_VAR, pos);
	}
	pos = str.find(FIVE_VAR);
	while(pos!=-1)	{
		str.replace(pos, FIVE_VAR_LENGTH, "5");
		pos = str.find(FIVE_VAR, pos);
	}
	pos = str.find(SIX_VAR);
	while(pos!=-1)	{
		str.replace(pos, SIX_VAR_LENGTH, "6");
		pos = str.find(SIX_VAR, pos);
	}
	pos = str.find(SEVEN_VAR);
	while(pos!=-1)	{
		str.replace(pos, SEVEN_VAR_LENGTH, "7");
		pos = str.find(SEVEN_VAR, pos);
	}
	pos = str.find(EIGHT_VAR);
	while(pos!=-1)	{
		str.replace(pos, EIGHT_VAR_LENGTH, "8");
		pos = str.find(EIGHT_VAR, pos);
	}
	pos = str.find(NINE_VAR);
	while(pos!=-1)	{
		str.replace(pos, NINE_VAR_LENGTH, "9");
		pos = str.find(NINE_VAR, pos);
	}
	pos = str.find(SPACE_VAR);
	while(pos!=-1)	{
		str.replace(pos, SPACE_VAR_LENGTH, " ");
		pos = str.find(SPACE_VAR, pos);
	}
	pos = str.find(DOT_VAR);
	while(pos!=-1)	{
		str.replace(pos, DOT_VAR_LENGTH, ".");
		pos = str.find(DOT_VAR, pos);
	}
	pos = str.find(END_VAR);
	while(pos!=-1)	{
		str.insert(pos + END_VAR_LENGTH, " ");
		pos = str.find(END_VAR, pos + 2);
	}
	return str;
}