//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Modifier.cpp: implementation of the Modifier class.
//
//////////////////////////////////////////////////////////////////////

#include "Modifier.h"
#include "Edge.h"
#include <iostream>
#include <sstream>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Modifier::Modifier()
{
	
}

Modifier::Modifier(BNF* grammar, BNFtype bnfType)
: NonTerminal(grammar, bnfType)
{
	_name = MODIFIER;
}

Modifier::~Modifier()
{

}

Sentence* Modifier::getProductionRule(int gene, bool terminal)
{
	int prod;
	ostringstream str;
	if(_BNFtype==REVERSIBLE)	{
		if(terminal)	{
			prod = gene % 3;
			switch(prod)	{
			case 0:	str << "{ " << EDGE << " }";
					break;
			case 1:	str << "[ [ + + " << EDGE_X << " ] - - " << EDGE_X << " ]";
					break;
			default:	str << "[ [ + " << EDGE_X << " ] - " << EDGE_X << " ]";
			}
		}
		else	{
			prod = gene % 10;
			switch(prod)	{
			case 0:	str << "{ " << EDGE << " }";
				break;
			case 1:	str << "[ [ + + " << EDGE_X << " ] - - " << EDGE_X << " ]";
					break;
			case 2:	str << "[ [ + " << EDGE_X << " ] - " << EDGE_X << " ]";
					break;
			case 3:	str << "+ " << MODIFIER << " -";
					break;
			case 4:	str << "- " << MODIFIER << " +";
					break;
			case 5:	str << "& " << MODIFIER << " ^";
					break;
			case 6:	str << "^ " << MODIFIER << " &";
					break;
			case 7:	str << "/ " << MODIFIER << " \\";
					break;
			case 8:	str << "\\ " << MODIFIER << " /";
					break;
			default:	str << "~ " << MODIFIER;
					break;
			}
		}
	}
	else if(_BNFtype==SYMMETRIC)	{
		if(terminal)	{
			prod = gene % 2;
			switch(prod)	{
			case 0:	str << "{ " << EDGE << " }";
					break;
			default:	str << "[ [ + + " << EDGE_X << " ] - - " << EDGE_X << " ]";
					break;
			}
		}
		else	{
			prod = gene % 9;
			switch(prod)	{
			case 0:	str << "{ " << EDGE << " }";
				break;
			case 1:	str << "[ [ + + " << EDGE_X << " ] - - " << EDGE_X << " ]";
					break;
			case 2:	str << "+ " << MODIFIER << " -";
					break;
			case 3:	str << "- " << MODIFIER << " +";
					break;
			case 4:	str << "& " << MODIFIER << " ^";
					break;
			case 5:	str << "^ " << MODIFIER << " &";
					break;
			case 6:	str << "/ " << MODIFIER << " \\";
					break;
			case 7:	str << "\\ " << MODIFIER << " /";
					break;
			default:	str << "~ " << MODIFIER;
					break;
			}
		}
	}
	else	{
		if(terminal)	{
			prod = gene % 3;
			switch(prod)	{
			case 0:	str << "{ " << EDGE << " }";
					break;
			case 1:	str << "[ [ + + " << EDGE_X << " ] - - " << EDGE_X << " ]";
					break;
			default:	str << "[ [ + " << EDGE_X << " ] - " << EDGE_X << " ]";
			}
		}
		else	{
			prod = gene % 12;
			switch(prod)	{
			case 0:	str << "{ " << EDGE << " }";
				break;
			case 1:	str << "[ [ + + " << EDGE_X << " ] - - " << EDGE_X << " ]";
					break;
			case 2:	str << "[ [ + " << EDGE_X << " ] - " << EDGE_X << " ]";
					break;
			case 3:	str << "+ " << MODIFIER << " -";
					break;
			case 4:	str << "- " << MODIFIER << " +";
					break;
			case 5:	str << "& " << MODIFIER << " ^";
					break;
			case 6:	str << "^ " << MODIFIER << " &";
					break;
			case 7:	str << "/ " << MODIFIER << " \\";
					break;
			case 8:	str << "\\ " << MODIFIER << " /";
					break;
			case 9:	str << EDGE << " [ [ + + " << EDGE_X << " ] - - " << EDGE_X << " ] " << EDGE;
					break;
			case 10:	str << EDGE << " [ [ + " << EDGE_X << " ] - " << EDGE_X << " ] " << EDGE;
					break;
			default:	str << "~ " << MODIFIER;
					break;
			}
		}
	}
	return new Sentence(str.str());
}

void Modifier::reconstruct(Sentence* sent, Genotype* &genome, int multi)
{
	//cout << "Reconstructing Modifier: " << endl;
	sent->printSentence();
	int last = sent->numberOfWords() - 1;
	if(last==-1)	{
		//Get the correct rule
		genome->addGene(0);
		//And the correct number of edges
		genome->addGene(0);
		return;
	}
	string name = sent->getWord(0)->getName();
	if(name.substr(0, 4)==EDGE)	{
		//Get the correct rule
		genome->addGene(0);
		for(int i=0; i<last+1; i++)	{
			Edge* edg = new Edge;
			edg->reconstruct(sent->getWord(i)->getEdgeType(), genome, i+1);
		}
		//Make sure that we do not get any more edges
		genome->addGene(0);
	}
	else if(name=="+")	{
		genome->addGene(1);
		Modifier* mod = new Modifier;
		mod->reconstruct(sent->getSentence(1, last), genome);
	}
	else if(name=="-")	{
		genome->addGene(2);
		Modifier* mod = new Modifier;
		mod->reconstruct(sent->getSentence(1, last), genome);
	}
	else if(name=="&")	{
		genome->addGene(3);
		Modifier* mod = new Modifier;
		mod->reconstruct(sent->getSentence(1, last), genome);
	}
	else if(name=="^")	{
		genome->addGene(4);
		Modifier* mod = new Modifier;
		mod->reconstruct(sent->getSentence(1, last), genome);
	}
	else if(name=="\\")	{
		genome->addGene(5);
		Modifier* mod = new Modifier;
		mod->reconstruct(sent->getSentence(1, last), genome);
	}
	else if(name=="/")	{
		genome->addGene(6);
		Modifier* mod = new Modifier;
		mod->reconstruct(sent->getSentence(1, last), genome);
	}
	else if(name=="~")	{
		genome->addGene(7);
		Modifier* mod = new Modifier;
		mod->reconstruct(sent->getSentence(1, last+1), genome);
	}
	else if(name=="[")	{
		if(sent->getWord(2)->getName()=="+")	{
			genome->addGene(8);
			Edge* edg = new Edge;
			edg->reconstruct(sent->getWord(3)->getEdgeType(), genome);
		}
		else if(sent->getWord(2)->getName()=="-")	{
			genome->addGene(9);
			Edge* edg = new Edge;
			edg->reconstruct(sent->getWord(4)->getEdgeType(), genome);
		}
	}
	//cout << "Finished reconstructing modifier." << endl;
}