//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// NonTerminal.h: interface for the NonTerminal class.
//
//	This is the parent class for all non-terminals of the grammar.
//	It contains all the functions necessary for the non-terminals.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_NONTERMINAL_H__E9F51B11_17B7_11D5_95D7_00C04F797C24__INCLUDED_)
#define AFX_NONTERMINAL_H__E9F51B11_17B7_11D5_95D7_00C04F797C24__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Terminal.h"
#include "ProductionRule.h"
#include <vector>


typedef enum {REGULAR, REVERSIBLE, SYMMETRIC, PROBABILISTIC} BNFtype;

class BNF;

class NonTerminal: public Terminal 
{
public:
	NonTerminal();
	NonTerminal(BNF* grammar, BNFtype bnfType);
	virtual ~NonTerminal();

	//Expand the non-terminal with a production rule
	virtual void		expand(int depth, int predecessor = -1);
	//Get a production rule
	virtual Sentence* getProductionRule(int gene, bool terminal) = 0;
	bool		isExpandable()	{return _expandable;};
	void		printNonTerminal();
	
	//Use these functions when creating the new (non)terminals after having used the production-rules
	//Non-terminals
	void		addAxiom();	//We want to control the multiplicity depending on the rule
	void		addRewriteRule(bool multi = false);
	void		addPredecessor();
	void		addSuccessor(bool multi = false);
	void		addCondition(bool multi = false);
	void		addOperator(bool multi = false);
	void		addModifier(bool multi = false);
	//Terminals
	void		addOperatorType(string op);
	void		addEdge(bool multi, bool newEdge, int &edgetype);
	void		addEdge(bool multi, bool newEdge);
	void		addEdge(int predecessor);
	void		addStartEdge(bool multi);
	
	//Use this with regn8
	virtual void	reconstruct(Sentence* sent, Genotype* &genome, int multi = 0) = 0;
	
protected:
	//Data
	bool			_expandable;
	//We have a pointer to the BNF. From there we get genes for the productions
	//and info about the global status of the grammar.
	BNF*			_BNF;
	//Know what type of BNF we are using
	BNFtype			_BNFtype;
};

#endif // !defined(AFX_NONTERMINAL_H__E9F51B11_17B7_11D5_95D7_00C04F797C24__INCLUDED_)
