//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// NurbsSurface.cpp: implementation of the NurbsSurface class.
//
//////////////////////////////////////////////////////////////////////

#include "NurbsSurface.h"
#include "PolygonSurface.h"
#include <maya/MDagPath.h>
#include <maya/MStatus.h>
#include <maya/MPoint.h>
#include <maya/MFn.h>
#include <maya/MVector.h>
#include <maya/MPointArray.h>
#include <maya/MDoubleArray.h>
#include <iostream>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

NurbsSurface::NurbsSurface()
{

}

NurbsSurface::NurbsSurface(MDagPath surf, int ID)
{
	_ID = ID;
	MStatus *stat = new MStatus(MS::kSuccess);
	_surf = new MFnNurbsSurface(surf,stat);
	//if((*stat)==MS::kSuccess)
	//	cout << "Created a nurbs surface." << endl;
}

NurbsSurface::~NurbsSurface()
{

}

//This is used by the cut-off wall behaviour

bool NurbsSurface::getIntersection(Vector pos, Vector dir, Vector &intersect)
{
	bool *hit = false;
	MDoubleArray u, v, *dist = 0;
	MStatus *stat= new MStatus(MS::kSuccess);
	MPoint point(pos.getX(), pos.getY(), pos.getZ(), 1.0);
	MVector ray(dir.getX(), dir.getY(), dir.getZ());
	MPointArray intersections;

	//Let Maya do the hard work...
	_surf->intersect(point, ray, u, v, intersections, TOL, MSpace::kWorld, false, dist, true, hit, stat);

	//The first point in intersections contains the closest intersection point
	if(intersections.length()==0)
		return false;

	double vec[4];
	intersections[0].get(vec);
	intersect.setVector(vec[0], vec[1], vec[2]);
	//if((*stat)==MS::kSuccess)
		//cout << "Found intersection point at " << intersect.printVector().c_str() << endl;
	return true;
}

//This is used by the default wall-behaviour
//It finds out what component has coolided and do not interfere with the other components unless necessary

bool NurbsSurface::getIntersection(Vector pos, Vector dir, Vector &xintersect, Vector &yintersect, Vector &zintersect)
{
	double vec[4];
	bool *hit = NULL, inter = false;
	MDoubleArray uvec, vvec, *dist = new MDoubleArray;
	MStatus *stat= new MStatus(MS::kSuccess);
	MPoint point(pos.getX(), pos.getY(), pos.getZ(), 1.0), inters;
	MVector xray(dir.getX(), 0, 0), yray(0, dir.getY(), 0), zray(0, 0, dir.getZ());
	MPointArray intersections;

	//Let Maya do the hard work...
	_surf->intersect(point, xray, uvec, vvec, intersections, TOL, MSpace::kWorld, false, dist, false, hit, stat);
	//_surf->intersect(point, xray, u, v, inters, TOL, MSpace::kWorld, false, dist, false, hit, stat);
	//The first point in intersections contains the closest intersection point
	if(intersections.length()>0)	{
		intersections[0].get(vec);
		xintersect.setVector(vec[0], vec[1], vec[2]);
		//if((*stat)==MS::kSuccess)
			//cout << "Found intersection point at " << xintersect.printVector().c_str() << endl;
		inter = true;
	}
	
	intersections.clear();
	//Let Maya do the hard work...
	_surf->intersect(point, yray, uvec, vvec, intersections, TOL, MSpace::kWorld, false, dist, false, hit, stat);
	//The first point in intersections contains the closest intersection point
	if(intersections.length()>0)	{
		intersections[0].get(vec);
		yintersect.setVector(vec[0], vec[1], vec[2]);
		//if((*stat)==MS::kSuccess)
			//cout << "Found intersection point at " << yintersect.printVector().c_str() << endl;
		inter = true;
	}
	
	intersections.clear();
	//Let Maya do the hard work...
	_surf->intersect(point, yray, uvec, vvec, intersections, TOL, MSpace::kWorld, false, dist, false, hit, stat);
	//The first point in intersections contains the closest intersection point
	if(intersections.length()>0)	{
		intersections[0].get(vec);
		zintersect.setVector(vec[0], vec[1], vec[2]);
		//if((*stat)==MS::kSuccess)
			//cout << "Found intersection point at " << zintersect.printVector().c_str() << endl;
		inter = true;
	}
	return inter;
}
