//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// PolygonSurface.cpp: implementation of the PolygonSurface class.
//
//////////////////////////////////////////////////////////////////////

#include "PolygonSurface.h"
#include <maya/MDagPath.h>
#include <maya/MStatus.h>
#include <maya/MPoint.h>
#include <maya/MPointArray.h>
//#include <maya/MSpace.h>
#include <maya/MFn.h>
#include <maya/MVector.h>
#include <maya/MIntArray.h>
#include <iostream>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PolygonSurface::PolygonSurface()
{

}

PolygonSurface::PolygonSurface(MDagPath surf, int ID)
{
	_ID = ID;
	MStatus *stat = new MStatus(MS::kSuccess);
	_surf = new MFnMesh(surf,stat);
	//if((*stat)==MS::kSuccess)
		//cout << "Created a polygon surface." << endl;
}

PolygonSurface::~PolygonSurface()
{

}

bool PolygonSurface::getIntersection(Vector pos, Vector dir, Vector &intersect)
{
	MPointArray intersections;
	MIntArray* polys = new MIntArray;
	MStatus *stat= new MStatus(MS::kSuccess);
	MPoint point(pos.getX(), pos.getY(), pos.getZ(), 1.0), tmp;
	MVector ray(dir.getX(), dir.getY(), dir.getZ());

	//Let Maya do the hard work...
	_surf->intersect(point, ray, intersections, TOL, MSpace::kWorld, polys, stat);

	//The first point in intersections contains the closest intersection point
	if(intersections.length()==0)
		return false;

	tmp = intersections[0];
	double vec[4];
	tmp.get(vec);
	intersect.setVector(vec[0], vec[1], vec[2]);
	//if((*stat)==MS::kSuccess)
		//cout << "Found intersection point at " << intersect.printVector().c_str() << endl;
	return true;
}

bool PolygonSurface::getIntersection(Vector pos, Vector dir, Vector &xintersect, Vector &yintersect, Vector &zintersect)
{
	double vec[4];
	bool inter = false;
	MStatus *stat= new MStatus(MS::kSuccess);
	MPoint point(pos.getX(), pos.getY(), pos.getZ(), 1.0);
	MVector xray(dir.getX(), 0, 0), yray(0, dir.getY(), 0), zray(0, 0, dir.getZ());
	MPointArray intersections;
	MIntArray* polys = new MIntArray;

	//Let Maya do the hard work...
	_surf->intersect(point, xray, intersections, TOL, MSpace::kWorld, polys, stat);

	//The first point in intersections contains the closest intersection point
	if(intersections.length()>0)	{
		intersections[0].get(vec);
		xintersect.setVector(vec[0], vec[1], vec[2]);
		//if((*stat)==MS::kSuccess)
			//cout << "Found intersection point at " << xintersect.printVector().c_str() << endl;
		inter = true;
	}
	
	intersections.clear();
	//Let Maya do the hard work...
	_surf->intersect(point, yray, intersections, TOL, MSpace::kWorld, polys, stat);

	//The first point in intersections contains the closest intersection point
	if(intersections.length()>0)	{
		intersections[0].get(vec);
		yintersect.setVector(vec[0], vec[1], vec[2]);
		//if((*stat)==MS::kSuccess)
			//cout << "Found intersection point at " << yintersect.printVector().c_str() << endl;
		inter = true;
	}
	
	intersections.clear();
	//Let Maya do the hard work...
	_surf->intersect(point, zray, intersections, TOL, MSpace::kWorld, polys, stat);

	//The first point in intersections contains the closest intersection point
	if(intersections.length()>0)	{
		intersections[0].get(vec);
		zintersect.setVector(vec[0], vec[1], vec[2]);
		//if((*stat)==MS::kSuccess)
			//cout << "Found intersection point at " << zintersect.printVector().c_str() << endl;
		inter = true;
	}

	return inter;
}