//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Predecessor.cpp: implementation of the Predecessor class.
//
//////////////////////////////////////////////////////////////////////

#include "Predecessor.h"
#include "Edge.h"
#include <iostream>
#include <sstream>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


Predecessor::Predecessor()
{
	
}

Predecessor::Predecessor(BNF* grammar, BNFtype bnfType)
: NonTerminal(grammar, bnfType)
{
	_name = PREDECESSOR;
}

Predecessor::~Predecessor()
{

}

Sentence* Predecessor::getProductionRule(int gene, bool terminal)
{
	int prod = gene % 4;
	ostringstream str;
	switch(prod)	{
	case 0:	str << EDGE;
			break;
	case 1:	str << EDGE << " < " << EDGE;
			break;
	case 2:	str << EDGE << " > " << EDGE;
			break;
	default:	str << EDGE << " < " << EDGE << " > " << EDGE;
			break;
	}
	return new Sentence(str.str());
}

void Predecessor::reconstruct(Sentence* sent, Genotype* &genome, int multi)
{
	//cout << "Reconstructing Predecessor: " << endl;
	sent->printSentence();
	int nWords = sent->numberOfWords();
	Edge* edg = new Edge;
	if(nWords==1)	{
		genome->addGene(0);
		edg->reconstruct(sent->getWord(0)->getEdgeType(), genome);
	}
	else if(nWords==3)	{
		if(sent->getWord(1)->getName()=="<")
			genome->addGene(1);
		else
			genome->addGene(2);
		
		edg->reconstruct(sent->getWord(0)->getEdgeType(), genome);
		edg = new Edge;
		edg->reconstruct(sent->getWord(2)->getEdgeType(), genome);
	}
	else	{
		genome->addGene(3);
		edg->reconstruct(sent->getWord(0)->getEdgeType(), genome);
		edg = new Edge;
		edg->reconstruct(sent->getWord(2)->getEdgeType(), genome);
		edg = new Edge;
		edg->reconstruct(sent->getWord(4)->getEdgeType(), genome);
	}
}

