//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Region.h: interface for the Region class.
//
//	This class represents the regions of the scaffold, ie the areas enclosed by the segments.
//	There are two main purposes for this class.
//	One is to help the Scaffold merging branches.
//	The other is to provide the Scaffold with required information when we are using
//	the Maya function for creating a mesh.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_REGION_H__46203A33_1718_11D5_95D7_00C04F797C24__INCLUDED_)
#define AFX_REGION_H__46203A33_1718_11D5_95D7_00C04F797C24__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Vertex.h"
#include "Segment.h"
#include "Branch.h"
#include "Vector.h"
#include <vector>
#include <string>
#include <fstream>

using namespace std;

class Segment;
class Branch;
class Vertex;

class Region  
{
public:
	Region();
	Region(int ID);
	virtual ~Region();

	void		printInfo();
	int			getID()	{return _ID;};
	//Segments
	//The segments are stored randomly
	void		addSegment(Segment* &segment);
	Segment*	getSegment(int i)	{return _segments[i];};
	int			numberOfSegments()	{return _segments.size();};
	//Use these functions to traverse the perimeter of the region in the order of the segments
	void		getNextSegment(Segment* &seg, Vertex* &ver);
	void		getNextSegment(Segment* &seg, Vertex* &ver, vector<Segment*>::iterator &it);
	void		removeSegments(Vertex* start, Vertex* end);
	//Vertices
	//Same for the vertices, they are not stored in any particualr order
	void			addVertex(Vertex* vertex);
	void			removeVertex(Vertex* vertex);
	bool			surroundedByVertex(Vertex* ver);
	Vertex*			getVertex(int i)	{return _vertices[i];};
	vector<Vertex*>	getVertices()	{return _vertices;};
	vector<Vertex*>	getOrderedVertices();
	int				numberOfVertices()	{return _vertices.size();};
	
	//Use this to split the region into two when we have a segment going through
	void	splitRegion(Vertex* start, Vertex* end, vector<Segment*> &segmentsForNewRegion/*, vector<Vertex*> &verticesForNewRegion*/);
	
	//Saving and loading
	void	saveRegion(ofstream &file);
	void	loadRegion(ifstream &file, vector<Segment*> &segs, vector<Vertex*> &vers);
private:
	int _ID;
	vector<Segment*> _segments;
	int		_nSegments;
	vector<Vertex*>	_vertices;
	int		_nVertices;
};

#endif // !defined(AFX_REGION_H__46203A33_1718_11D5_95D7_00C04F797C24__INCLUDED_)
