//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// RewriteRule.cpp: implementation of the RewriteRule class.
//
//////////////////////////////////////////////////////////////////////

#include "RewriteRule.h"
#include "Predecessor.h"
#include "Successor.h"
#include <iostream>
#include <sstream>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

RewriteRule::RewriteRule()
{
	
}

RewriteRule::RewriteRule(BNF* grammar, BNFtype bnfType)
: NonTerminal(grammar, bnfType)
{
	_name = REWRITE_RULE;
}

RewriteRule::~RewriteRule()
{

}

Sentence* RewriteRule::getProductionRule(int gene, bool terminal)
{
	ostringstream str;
	if(_BNFtype==PROBABILISTIC)
		str << PREDECESSOR << " " << SUCCESSOR << " { " << SUCCESSOR << " }";
	else
		str << PREDECESSOR << " " << SUCCESSOR;
	return new Sentence(str.str());
}

void RewriteRule::reconstruct(Sentence* sent, Genotype* &genome, int multi)
{
	//cout << "Reconstructing RewriteRule: " << endl;
	if(multi!=0)
		genome->addGene(multi+1);
	sent->printSentence();
	//Automatically expanded to pred and succ
	genome->addRandGene();
	//Get the predecessor
	int arrowpos = sent->findWord("->");
	Predecessor* pred = new Predecessor;
	//Get the successor
	int nWords = sent->numberOfWords();
	Successor* succ = new Successor;
	//Reconstruct them
	pred->reconstruct(sent->getSentence(0, arrowpos), genome);
	succ->reconstruct(sent->getSentence(arrowpos+1, nWords), genome);
}