//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Segment.h: interface for the Segment class.
//
//	This class is used to represent the segment (edges) of the scaffold.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SEGMENT_H__46203A32_1718_11D5_95D7_00C04F797C24__INCLUDED_)
#define AFX_SEGMENT_H__46203A32_1718_11D5_95D7_00C04F797C24__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include "Vector.h"
#include "OperatorType.h"
#include <vector>
#include <fstream>

using namespace std;

class Region;
class Vertex;
struct ConditionData;

class Segment  
{
public:
	Segment();
	Segment(int ID, bool perimeter, int count = 0);
	Segment(int ID, const Segment &oldSeg);
	virtual ~Segment();

	//Used by splitSegment
	void	divide(int parts);
	//Get and set the member variables
	bool	isPerimeter()	{return _perimeter;};
	void	setPerimeter(bool perimeter)	{_perimeter = perimeter;};	
	void	setLength(double length)	{_length = length;};
	double	getLength()	{return _length;};
	int		getType()	{return _type;};
	void	setType(int type, int randRewrite, int nTypes);
	void	setType(int type)	{_type = type;};
	Vector	getDirection()	{return _dir;};
	void	setDirection(Vector dir)	{_dir = dir;};
	Vertex* getStart()	{return _start;};
	void	setStart(Vertex* start)	{_start = start;};
	Vertex* getEnd()	{return _end;};
	void	setEnd(Vertex* end)	{_end = end;};
	Region* getRight()	{return _rightRegion;};
	void	setRight(Region* right)	{_rightRegion = right;};
	Region*	getLeft()	{return _leftRegion;};
	void	setLeft(Region* left)	{_leftRegion = left;};
	int		getCurve()	{return _curve;};
	void	setCurve(int curve)	{_curve = curve;};
	int		getID()	{return _ID;};
	//For checking the Kondition
	void	getPreviousSegs(vector<Segment*> &segs);
	void	getFollowingSegs(vector<Segment*> &segs);
	bool	isTypePrevious(int type);
	bool	isTypeAndCountPrevious(EdgeTypeAndCount edge);
	bool	isTypeFollowing(int type);
	bool	isTypeAndCountFollowing(EdgeTypeAndCount edge);
	bool	isRewritten()	{return _rewritten;};
	void	setRewritten(bool rewrite)	{_rewritten = rewrite;};
	int		getAge()	{return _age;};
	void	incAge()	{_age++;};
	int		getCounter()	{return _counter;};
	void	setCounter(int count)	{_counter = count;};
	void	incCounter()	{_counter++;};
	void	decCounter()	{_counter--;};

	//For rewriting
	void	setOrientation();	//Updates the orientation, use after the vertices has been moved
	void	switchDirection();	//Switches the direction of the segment, ie _start and _end
	double	changeOrientation(vector<OperatorTypes> ops, double angle, Vector &dir);	//Returns the length of the segment
	void	changeLength(double newLength, int depth);	//This function makes sure that all segments have the same length after a split
	bool	checkCondition(ConditionData condData);
	bool	checkCounter(EdgeTypeAndCount edgeTAndC);

	//Saving and loading
	void	saveSegment(ofstream &file);
	void	loadSegment(ifstream &file, vector<Vertex*> &vers, vector<Region*> &regs);

private:
	int		_ID;
	int		_type;
	int		_curve;	//What curve is the segment member of
	Vertex* _start;	//The segment connects these vertices
	Vertex*	_end;
	Region* _rightRegion;	//Keep track of the adjacent regions
	Region*	_leftRegion;
	bool	_perimeter;	//Is the segment on the perimeter of the surface
	Vector	_dir;		//Spatial orientation
	double	_length;	//Distance between _start and _end
	int		_age;	//This is incremented after each step
	int		_counter;	//The counter may be incremented and set more freely than the age
	
	bool	_rewritten;	//If we wish to have syncronous growth
};

#endif // !defined(AFX_SEGMENT_H__46203A32_1718_11D5_95D7_00C04F797C24__INCLUDED_)
