//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Sentence.cpp: implementation of the Sentence class.
//
//////////////////////////////////////////////////////////////////////

#include "Sentence.h"
#include "MELUtilities.h"
#include "Terminal.h"
#include <iostream>
#include <sstream>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Sentence::Sentence()
{

}

//Construct from a string, tokenize and make each words

Sentence::Sentence(string sent)
{
	string tmp;
	Word* word;
	int prevspacepos = -1, spacepos = sent.find(' ');
	while(spacepos!=-1)	{
		word = new Word(sent.substr(prevspacepos + 1, spacepos - prevspacepos - 1));
		_sentence.push_back(word);
		prevspacepos = spacepos;
		spacepos = sent.find(' ', prevspacepos + 1);
	}
	//Fix last word
	word = new Word(sent.substr(prevspacepos + 1, sent.size() - prevspacepos - 1));
	_sentence.push_back(word);
}

Sentence::~Sentence()
{

}

void Sentence::printSentence()
{
	for(int i=0; i<_sentence.size(); i++)
		_sentence[i]->printWord();

	cout << endl;
}

//Find next occurence of a word in a string. Returns the index into the sentence

int Sentence::findWord(string word, int i)
{
	//cout << "Looking for " << word.c_str() << ", between " << i << " and " << numberOfWords() << endl;
	for(int j=i; j<numberOfWords(); j++)	{
		//cout << "Found word: " << getWord(j)->getName().c_str() << " at " << j << endl;
		if(getWord(j)->getName()==word)
			return j;
	}
	//Could not find the word
	//cout << "Failed to find " << word.c_str() << endl;
	return -1;
}

int Sentence::numberOfWords(string name)
{
	int count = 0;
	for(int i=0; i<numberOfWords(); i++)	{
		if(getWord(i)->getName().find(name)!=-1)
			count++;
	}
	return count;
}

Sentence* Sentence::getSentence(int start, int end)
{
	Sentence* sent = new Sentence();
	end = end < numberOfWords() ? end : numberOfWords();
	for(int i=start; i<end; i++)
		sent->addWord(getWord(i));

	return sent;
}

//Returns a pointer to a part of the sentence between two END_VAR words

Sentence* Sentence::getLine(int line)
{
	Sentence* sent = new Sentence();
	//Find the line
	int endpos = -1, startpos;
	for(int i=0; i<line+1; i++)	{
		startpos = endpos + 1;
		endpos = findWord(END_VAR, startpos);
		if(endpos==-1)
			return NULL;
	}
	return getSentence(startpos, endpos);
}

int Sentence::numberOfLines()
{
	int count = 0, endpos = findWord(END_VAR, 0);
	while(endpos!=-1)	{
		count++;
		endpos = findWord(END_VAR, endpos+1);
	}
	return count;
}

//Counts the number of different edgetypes present in the sentence

int Sentence::numberOfEdgeTypes()
{
	int count = 0, edgepos = findWord("Edge0", 0);
	while(edgepos!=-1)	{
		count++;
		ostringstream edge;
		edge << EDGE << count;
		edgepos = findWord(edge.str(), 0);
	}
	return count;
}

//We need the dots for the stupid Maya-environment

string Sentence::str(bool end, bool dotsInsteadOfSpace)
{
	ostringstream stream;
	for(int i=0; i<numberOfWords(); i++)	{	
		if(_sentence[i]->getName()==END_VAR && !end)
			stream << endl;
		else //if(!dotsInsteadOfSpace)
			stream << _sentence[i]->getName().c_str() << " ";
		//else
		//	stream << _sentence[i]->getName(true).c_str() << ".";
	}
	//Trim
	string st = stream.str();
	int size = st.size();
	if(st.substr(size - 1, 1)=="." || " ")
		st.erase(size - 1, 1);
	return st;
}

void Sentence::saveSentence(ofstream &file)
{
	for(int i=0; i<_sentence.size(); i++)	{
		//cout << "Found word " << _sentence[i]->getName() << endl;
		if(_sentence[i]->getName()==END_VAR)
			file << '\n';
		else
			file << _sentence[i]->getName() << " ";
	}
}