//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Vector.h: interface for the Vector class.
//
//	This class is used for 3D-vectors and defines a bunch of useful operations on those.
//	It is your typical mathematical notion of a geometric vector.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_VECTOR_H__E9F51B0B_17B7_11D5_95D7_00C04F797C24__INCLUDED_)
#define AFX_VECTOR_H__E9F51B0B_17B7_11D5_95D7_00C04F797C24__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define MAXV(a,b) (a < b ? b : a)
#define MINV(a,b) (a < b ? a : b)

#include <string>
//#include <algorithm>

using namespace std;

class Vector  
{
public:
	Vector();
	Vector(const Vector& vec);
	virtual ~Vector();

	inline Vector(double x, double y, double z);
	//Operators
	const bool operator==(const Vector &vec);
	const Vector operator+(const Vector &vec);
	const Vector& operator+=(const Vector &vec);
	const Vector& operator=(const Vector &vec);
	const Vector operator/(const double &denominator);
	const Vector& operator/=(const double &denominator);
	Vector operator-(const Vector &vec);
	const Vector& operator-=(const Vector &vec);
	const Vector operator*(const double &factor);
	const Vector& operator*=(const double &factor);
	const double operator*(const Vector &vec);	//Dot-product
	const Vector operator&(const Vector &vec);	//Cross-product
	inline const double maxv();//	{return MAX(MAX(_x, _y), _z);};
	inline const double minv();//	{return MIN(MIN(_x, _y), _z);};

	double getX()	{return _x;};
	double getY()	{return _y;};
	double getZ()	{return _z;};
	void getVector(double &x, double &y, double &z)	{x=_x; y=_y; z=_z;};
	void setVector(double x, double y, double z)	{_x=x; _y=y; _z=z; calculateLength();};
	void setX(double x)	{_x = x; calculateLength();};
	void setY(double y)	{_y = y; calculateLength();};
	void setZ(double z)	{_z = z; calculateLength();};

	double normalize();
	bool isNormalized()	{return _normalized;};
	inline void setLength(double length);
	double getLength()	{return _length;};
	void randv(bool normalize = true);	//Make the vector random
	void clear()	{_x=0; _y=0; _z=0; _length=0; _normalized=false;};

	void turn(double angle);
	void pitch(double angle);
	void roll(double angle);

	string printVector();
	
private:
	inline void	calculateLength();

	double _x, _y, _z, _length;
	bool _normalized;
};

#endif // !defined(AFX_VECTOR_H__E9F51B0B_17B7_11D5_95D7_00C04F797C24__INCLUDED_)
