//
// Copyright (C) 2001 Martin Hemberg. You are allowed to use or modify the code, but this note must be kept intact.
// 
// Vertex.cpp: implementation of the Vertex class.
//
//////////////////////////////////////////////////////////////////////

#include <iostream.h>
//#include <maya/MGlobal.h>
#include "Vertex.h"
#include "Boundary.h"
#include "Force.h"
#include "Noise.h"
#include "Branch.h"
#include "Scaffold.h"
#include <cmath>
//#include "MELUtilities.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Vertex::Vertex()
{

}

Vertex::Vertex(int ID, Boundary* boundary, Force* force, Noise* noise, double limitedBranches, double branchAngle, bool outside, bool perimeter, double mass)
{
	_ID = ID;
	_boundary = boundary;
	_force = force;
	_noise = noise;
	_mass = mass;
	_perimeter = perimeter;
	_branchRange = limitedBranches;
	_branchAngle = cos(branchAngle);
	_outside = outside;
}

Vertex::~Vertex()
{

}
/*
const Vertex& Vertex::operator=(const Vertex &ver)
{
	_ID = ver._ID;
	_pos = ver._pos;
	_segments = ver._segments;
	_boundary = ver._boundary;
	_force = ver._force;
	_mass = ver._mass;
	_perimeter = ver._perimeter;
	_regions = ver._regions;
	_branches = ver._branches;
	_noise = ver._noise;
	_branchRange = ver._branchRange;
	_branchAngle = ver._branchAngle;
	_outside = ver._outside;
	return *this;
}*/


void Vertex::moveVertex(Vector move, bool walls)
{
	if(walls)	{
		//cout << "Moving vertex " << _ID << " from " << _pos.printVector().c_str() << " with step " << move.printVector().c_str() << endl;
		_pos += move - _boundary->getWallDisplacement(_pos, move);
		//cout << "Vertex moved to " << _pos.printVector().c_str() << endl;
	}
	else
		_pos += move;

}

//When storing the segments, we do not have any particular ordering

void Vertex::addSegment(Segment* seg)
{
	//cout << "Adding segment " << seg->getID() << " to vertex " << _ID << endl;
	_segments.push_back(seg);
	Region* reg;
	reg = seg->getLeft();
	if(reg!=NULL)
		addRegion(reg);
	reg = seg->getRight();
	if(reg!=NULL)
		addRegion(reg);
}

//This is used when a segment is split

bool Vertex::removeSegment(Segment* seg)
{
	vector<Segment*>::iterator iter=_segments.begin();
	int i = 0;
	while(iter != _segments.end())	{
		if(_segments[i]->getID()==seg->getID())	{
			//cout << "Removing segment " << seg->getID() << " from vertex " << _ID << endl;
			_segments.erase(iter);
			return true;
		}
		i++;
		iter++;
	}
	return false;
}

//////////////////////////////////////////////////
//
//	Regions
//
//////////////////////////////////////////////////

//Keep track of the surrounding regions

void Vertex::addRegion(Region* reg)
{
	if(!regionAdjacent(reg))
		_regions.push_back(reg);
}

//Used when we split a region

bool Vertex::removeRegion(Region* reg)
{
	vector<Region*>::iterator iter=_regions.begin();
	int i = 0;
	while(iter != _regions.end())	{
		if(_regions[i]->getID()==reg->getID())	{
			//cout << "Removing segment " << seg->getID() << " from vertex " << _ID << endl;
			_regions.erase(iter);
			return true;
		}
		iter++;
		i++;
	}
	return false;
}

bool Vertex::regionAdjacent(Region* reg)
{
	//cout << "Checking if vertex " << _ID << " is adjacent to region " << reg->getID() << endl;
	int nRegs = _regions.size();
	for(int i=0; i<nRegs; i++)	{
		if(reg->getID()==_regions[i]->getID())
			return true;
	}
	return false;
}

void Vertex::adjacentRegions(vector<Region*> &regions)
{
	regions.clear();
	for(int i=0; i<_regions.size(); i++)
		regions.push_back(_regions[i]);
}

///////////////////////////////////////////////////////
//
//	Curves
//
//////////////////////////////////////////////////////

//Find out if we are the first (or last) node on a curve.

bool Vertex::curveEnds(int curve, Segment* &seg)
{
	int count = 0;
	for(int i=0; i<_segments.size(); i++)	{
		if(_segments[i]->getCurve()==curve && _segments[i]->getID()!=seg->getID())	{
			seg = _segments[i];
			return false;
		}
	}
	return true;
}

//Traverse a curve
//&ver will continue the next vertex on the curve (if there is one)
//&seg will contain the segment that was used to get there.

bool Vertex::getNextOnCurve(int curve, Vertex* &ver, Segment* &seg)
{
	//cout << "Looking for the next segment in curve " << curve << " following segment " << seg->getID() << " on vertex " << _ID << endl;
	for(int i=0; i<_segments.size(); i++)	{
		//cout << "Segment " << _segments[i]->getID() << " has curve " << _segments[i]->getCurve() << ", begins at " << _segments[i]->getStart()->getID() << " and ends at " << _segments[i]->getEnd()->getID() << endl;
		if(_segments[i]->getCurve()==curve && _segments[i]->getID()!=seg->getID())	{
			seg = _segments[i];
			ver = seg->getEnd()->getID()==_ID ? seg->getStart() : seg->getEnd();
			//cout << "Next vertex is " << ver->getID() << " and the segment is " << seg->getID() << endl;
			return true;
		}
	}
	//cout << "Could not find next segment on curve." << endl;
	return false;
}

//Find out if a vertex is adjacent

bool Vertex::adjacent(Vertex* ver)
{
	//cout << "Checking if vertex " << _ID << " is adjacent to vertex " << ver->getID() << endl;
	for(int i=0; i<_segments.size(); i++)	{
		//cout << "Segment " << _segments[i]->getID() << " starts at " << _segments[i]->getEnd()->getID() << " and ends at " << _segments[i]->getStart()->getID() << endl;
		if(_segments[i]->getEnd()->getID()==ver->getID() ||
			_segments[i]->getStart()->getID()==ver->getID())	{
			//cout << "Returning true from adjacent." << endl;
			return true;
		}
	}
	//cout << "Returning false from adjacent." << endl;
	return false;
}

//Get all teh adjacent vertices

void Vertex::adjacentVertices(vector<Vertex*> &vertices)
{
	vertices.clear();
	for(int i=0; i<_segments.size(); i++)	{
		if(_segments[i]->getEnd()->getID()!=_ID)
			vertices.push_back(_segments[i]->getEnd());
		else
			vertices.push_back(_segments[i]->getStart());
	}
}

//Find out if a curve goes through the vertex

bool Vertex::hasCurve(int curve)
{
	for(int i=0; i<_segments.size(); i++)	{
		if(_segments[i]->getCurve()==curve)
			return true;
	}
	return false;
}

//Get a list of all the curvevs passing through the vertex

void Vertex::curvesPassing(vector<int> &curves)
{
	curves.clear();
	int curveID;
	bool alreadyFound;
	for(int i=0; i<_segments.size(); i++)	{
		curveID = _segments[i]->getCurve();
		alreadyFound = false;
		for(int j=0; j<curves.size(); j++)
			alreadyFound = curves[j]==curveID;
		if(!alreadyFound)
			curves.push_back(curveID);
	}
}

//////////////////////////////////////////////
//
//	Branches
//
////////////////////////////////////////////////

//Add a branch to the vertex.
//We also get a list of operators to get the new orientation of the branch

void Vertex::placeBranch(BranchData branchData, Vector dir, double angle, int branchID)
{
	Branch* branch = new Branch(branchData._type, this, branchID);
	for(int i=0; i<branchData._operators.size(); i++)	{
		//cout << "Branch had operator " << branchData._operators[i] << endl;
		if(branchData._operators[i]==TURN_RIGHT)
			dir.turn(angle);
		else if(branchData._operators[i]==TURN_LEFT)
			dir.turn(-angle);
		else if(branchData._operators[i]==PITCH_DOWN)
			dir.pitch(angle);
		else if(branchData._operators[i]==PITCH_UP)
			dir.pitch(-angle);
		else if(branchData._operators[i]==ROLL_RIGHT)
			dir.roll(angle);
		else if(branchData._operators[i]==ROLL_LEFT)
			dir.roll(-angle);
		//else if(branchData._operators[i]==CHANGE_DIRECTION)
		//	branch->changeInto();
	}
	branch->setDirection(dir);
	_branches.push_back(branch);
}

void Vertex::removeBranch(Branch* branch)
{
	Branch* copy = branch;
	vector<Branch*>::iterator iter=_branches.begin();
	int i = 0;
	//cout << "Removing branch " << branch->getID() << " from vertex " << _ID << endl;
	while(iter != _branches.end())	{
		if(_branches[i]->getID()==branch->getID())	{
			//cout << "Found branch " << _branches[i]->getID() << endl;
			_branches.erase(iter);
			delete copy;
			return;
		}
		iter++;
		i++;
	}
	//cout << "Failed to remove the branch." << endl;
}

void Vertex::removeAllBranches()
{
	Branch* copy;
	for(int i=0; i<_branches.size(); i++)	{
		copy = _branches[i];
		delete copy;
	}
	_branches.clear();
}

//Match all the branches of the vertex with branch and see if it is possible to connect.

bool Vertex::matchBranch(Branch* branch, int &type, Region* &reg, bool &start, Vector center, double characteristicLength)
{
	Vertex* startVer;
	Vector d1, d2, centerDir1, centerDir2;
	vector<int> curves;
	bool cont = true;
	startVer = branch->getStart();
	d1 = branch->getDirection();
	d1.normalize();
	centerDir1 = startVer->getPosition() - center;
	//centerDir1.normalize();
	centerDir2 = getPosition() - center;
	//centerDir2.normalize();
	//The branches must have same type, originate from non-adjacent branches, roughly the same orientation, 
	if(numberOfBranches()>0)	{
		//cout << "Trying to match branch " << branch->getID() << " of type " << branch->getType();
		//cout << " beginning at vertex " << branch->getStart()->getID() << endl;
		//cout << "and has direction " << d1.printVector().c_str() << ", the vector to the center is " << centerDir1.printVector().c_str() << endl;
		for(int i=0; i<numberOfBranches(); i++)	{
			d2 = _branches[i]->getDirection();
			d2.normalize();
			double dotProd = d1*d2, diff = directionAndDifference(startVer->getPosition(), d2);
			//cout << "With branch " << _branches[i]->getID() << " of type " << _branches[i]->getType();
			//cout << " beginning at vertex " << _branches[i]->getStart()->getID() << endl;
			//cout << "and with direction " << d2.printVector().c_str() << ", the vector to the center is " << centerDir2.printVector().c_str() << endl;
			//cout << ", the dot product between the two vectors is " << dotProd << endl;
			//cout << "The diff is " << diff << endl;
			if( _branches[i]->getType()==branch->getType() && 
				//_branches[i]->intoRegion()!=branch->intoRegion() &&
				dotProd < -_branchAngle && diff > _branchAngle && closeEnough(startVer, characteristicLength) &&
				!_branches[i]->getStart()->adjacent( branch->getStart() ) )	{
				//cout << "Match successful." << endl;
				//Check if they are in the same curve
				branch->getStart()->curvesPassing(curves);
				//cout << "Vertex " << branch->getStart()->getID() << " has curves: ";
				for(int j=0; j<curves.size(); j++)	{
					if( hasCurve(curves[j]) && curves[j]!=0 )	{
						cont = false;
						break;
					}
				}
				if(!cont)
					continue;

				//These two branches should be connected
				//See which one is the starting point
				//start = _branches[i]->intoRegion();
				type = branch->getType();
				//Remove the branches
				startVer->removeBranch(branch);
				removeBranch(_branches[i]);
				return true;
			}
		}
	}
	return false;
}

//Get the dotproduct between the orientation of the branches

double Vertex::directionAndDifference(Vector otherBranchPos, Vector branchDirection)
{
	Vector diff = otherBranchPos - _pos;
	diff.normalize();
	return diff*branchDirection;
}

bool Vertex::closeEnough(Vertex* otherBranchStart, double characteristicLength)
{
	if(_branchRange < 0)
		return true;
	Vector force, diff = otherBranchStart->getPosition() - _pos;
	double dist = diff.getLength();
	//force = getDisplacement(false);
	//cout << "Resulting force for " << _ID << ": " << force.printVector().c_str() << endl;
	//cout << "The distance is " << dist << " and the resulting force has magnitude " << force.getLength() << endl;
	return dist < characteristicLength*_branchRange;
	
}

//////////////////////////////////////////////
//
//	Movement
//
////////////////////////////////////////////////

//Used when we grow the scaffold. The vertices move away from the center

void Vertex::scale(double factor, Vector center)
{
	//Get direction of growth
	Vector start = _pos, wall, step, res, centerDir = _pos - center;
	//cout << "Getting displacement for vertex " << _ID << " in " << start.printVector().c_str() << endl;
	//res = centerDir*factor;
	//Only the scaling
	res = center + centerDir*(factor);
	//cout << "Before attractors, growing to " << res.printVector().c_str() << endl;
	//Attroactors and repellers
	res = res + getDisplacement();
	//Walls
	step = res - start;
	//cout << "After attractors, growing to " << res.printVector().c_str() << endl;
	wall = _boundary->getWallDisplacement(start, step);
	_pos = res - wall;
	//cout << "New position: " << _pos.printVector().c_str() << endl;
}

void Vertex::scale(double factorX, double factorY, double factorZ, Vector center, bool softBoundary)
{
	//Get direction of growth
	Vector start = _pos, wall, step, res, centerDir = _pos - center;
	//cout << "Getting displacement for vertex " << _ID << " in " << start.printVector().c_str() << endl;
	//res = centerDir*factor;
	//Only the scaling
	res.setX( center.getX() + centerDir.getX()*(factorX) );
	res.setY( center.getY() + centerDir.getY()*(factorY) );
	res.setZ( center.getZ() + centerDir.getZ()*(factorZ) );
	//cout << "Before attractors, growing to " << res.printVector().c_str() << endl;
	//Attroactors and repellers
	res = res + getDisplacement();
	//Walls
	step = res - start;
	//cout << "After attractors, growing to " << res.printVector().c_str() << endl;
	if(softBoundary)
		_outside = _boundary->passedWall(start, step) ? !_outside : _outside;
	else
		wall = _boundary->getWallDisplacement(start, step);
	_pos = res - wall;
	//cout << "New position: " << _pos.printVector().c_str() << endl;
}

//Get the forces from teh environment affecting the vertex

Vector Vertex::getDisplacement(bool noise)
{
	Vector res;
	//Attractors and repellers
	int nAttractors = _force->numberOfAttractors();
	for(int i=0; i<nAttractors; i++)
		res += _force->getAttractor(i)->getForce(_pos)/_mass;
	//Gravity
	if(_force->hasGravity())
		res += _force->getGravitationalForce(_pos);
	//Noise
	if(noise)
		res += _noise->getNoise();
	//cout << "Total displacement: " << res.printVector().c_str() << endl;
	return res;
}

//Used for repelling points

Vector Vertex::getForce(Vector pos, double konst, int exp)
{
	//Calculate the direction
	Vector diff = _pos - pos;
	double dist = diff.getLength();
	diff.normalize();
	diff *= magnitude(dist, konst, exp);
	return diff;
}



void Vertex::printInfo()
{
	cout << "Vertex " << _ID << " has position " << _pos.printVector().c_str() << endl << "and is surrounded by regions: ";
	for(int i=0; i<_regions.size(); i++)
		cout << _regions[i]->getID() << ", ";
	cout << endl << "It has segments: ";
	for(int j=0; j<_segments.size(); j++)
		cout << _segments[j]->getID() << ", ";
	cout << endl;
}

//Used by uniformRegions. We use this to move to the center of the neighbours

double Vertex::moveToCenterOfNeighbours()
{
	//cout << "Moving vertex " << _ID << " in " << _pos.printVector().c_str() << endl;
	Vector center, oldPos = _pos;
	vector<Vertex*> neighbours;
	adjacentVertices(neighbours);
	double nNeighbours = neighbours.size();
	for(int i=0; i<nNeighbours; i++)	{
		//cout << "Found neighbour " << neighbours[i]->getID() << " in " << neighbours[i]->getPosition().printVector().c_str() << endl;
		center += neighbours[i]->getPosition();
	}
	center /= nNeighbours;
	//cout << "Moving " << _ID << " to " << (center).printVector().c_str() << ". The distance moved is " << (center - oldPos).getLength() << endl;
	setPosition(center);
	return (center - oldPos).getLength();
}

/////////////////////////////////////////////
//
//	Saving and loading
//
/////////////////////////////////////////////

void Vertex::saveVertex(ofstream &file)
{
	//Stream the data into the file
	//file << _ID << endl;
	file << _mass << endl;
	file << (int)_segments.size() << endl;
	for(int i=0; i<_segments.size(); i++)
		file << _segments[i]->getID() << " ";
	file << endl;
	file << _pos.getX() << " " << _pos.getY() << " " << _pos.getZ() << endl;
	file << _perimeter << endl;
	file << (int)_regions.size() << endl;
	for(int j=0; j<_regions.size(); j++)
		file << _regions[j]->getID() << " ";
	file << endl;
	file << _branchRange << endl;
	file << _branchAngle << endl;
}

void Vertex::loadVertex(ifstream &file, vector<Segment*> &segs, vector<Region*> &regs)
{
	//cout << "Loading vertex " << _ID << endl;
	double x, y, z;
	int size, ID;
	if(file.fail())
		file.clear();
	file >> _mass;
	file >> size;
	for(int i=0; i<size; i++)	{
		file >> ID;
		_segments.push_back(segs[ID]);
	}
	file >> x;
	file >> y;
	file >> z;
	_pos.setVector(x, y, z);
	file >> _perimeter;
	file >> size;
	for(int j=0; j<size; j++)	{
		file >> ID;
		_regions.push_back(regs[ID]);
	}
	file >> _branchRange;
	file >> _branchAngle;
}